/****
 *
 * Exemplo: Mtodo de Horner
 *
 * Seo: B.3
 *
 ****/

#include <stdio.h>

/****
 *
 * Horner(): Implementao do mtodo de Horner para avaliar o
 *           valor de um polinmio sobre um dado valor
 *
 * Parmetros:
 *      pol (entrada) - o polinmio representado por seus
 *                      coeficientes armazenados num array
 *      grau (entrada) - o grau do polinmio
 *      x (entrada) - o valor para o qual o polinmio ser
 *                    calculado
 *
 * Retorno: O valor calculado
 *
 * Observao: Para simplificar, o domnio dos polinmios  o
 *             conjunto dos nmeros inteiros
 *
 ****/
int Horner(const int pol[], int grau, int x)
{
   int i,
       resultado;

      /* Inicia a sequncia de nmeros (bi) com o */
      /* coeficiente (an) do termo de maior grau  */
   resultado = pol[grau];

      /* Repetidamente, multiplica o valor do resultado   */
      /* por x e adiciona o valor do coeficiente seguinte */
   for (i = grau - 1; i >= 0; --i)
      resultado = resultado*x + pol[i];

   return resultado;
}

/****
 *
 * main(): Testa a funo Horner()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int p[] = {-1, 2, 3, -5, 4}, /* Coeficientes do polinmio */
       grau; /* grau do polinmio */

   grau = sizeof(p)/sizeof(p[0]) - 1;

   printf( "\n\t>>> Valor de P(5): %d\n", Horner(p, grau, 5) );

   return 0;
}

/****************** Resultado do Programa *****************

        >>> Valor de P(5): 1959

**********************************************************/
