/****
 *
 * Ttulo: Srie de Taylor para Clculo de Seno
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 15/06/2012
 * ltima modificao: 15/06/2012
 *
 * Descrio: Calcula o seno de um nmero de
 *            ponto flutuante, que representa um
 *            ngulo em radianos, usando srie de Taylor
 *
 * Entrada: Nada
 *
 * Sada: v. resultado do programa
 *
 ****/

#include <stdio.h> /* Entrada e sada */
#include <math.h>  /* Funo sin()    */

#define PI 3.141592

/****
 *
 * Fatorial(): Calcula o fatorial de um nmero inteiro no negativo
 *
 * Parmetros:
 *      n (entrada): nmero cujo fatorial ser calculado
 *
 * Retorno: O fatorial de n, se n >= 0. Zero, se n < 0
 *
 ****/
int Fatorial(int n)
{
   int i,
       produto = 1; /* Acumula o produto */

      /* Se n for negativo, retorna 0. Este valor no     */
      /* representa um valor de fatorial vlido; ele      */
      /* significa que a funo foi chamada indevidamente */
   if (n < 0) {
      return 0; /* Chamada incorreta da funo */
   }

      /* Calcula: 1*2*...*(n-1)*n */
   for (i = 2; i <= n; ++i) {
      produto = produto*i;
   }

   return produto;
}

/****
 *
 * Sinal(): Determina qual  o sinal do termo de
 *          ordem n da srie de Taylor de seno
 *
 * Parmetros:
 *      n (entrada): nmero de ordem do termo
 *
 * Retorno: -1, se o sinal do termo for negativo;
 *          1, se o sinal do termo for positivo
 *
 ****/
int Sinal(int n)
{
      /* Se o nmero de ordem do termo for par,   */
      /* o termo  positivo e a funo retorna 1. */
      /* Caso contrario, ele  negativo e a       */
      /* funo retorna -1.                       */
   return n%2 ? -1 : 1;
}

/****
 *
 * Termo(): Calcula o valor do termo de ordem
 *          n da srie de Taylor de seno
 *
 * Parmetros:
 *      x (entrada): nmero cujo seno est sendo calculado
 *      n (entrada): nmero de ordem do termo
 *
 * Retorno: O valor do termo de ordem n da srie
 *
 ****/
double Termo(double x, int n)
{
      /* Se n for igual a zero, o valor do termo  x */
   if (!n) {
      return x;
   }

      /* Calcula e retorna o valor do n-simo termo */
   return Sinal(n) * pow(x, 2*n + 1) / Fatorial(2*n + 1);
}

/****
 *
 * Seno(): Calcula o seno de um nmero real,
 *         que representa um ngulo em radianos,
 *         usando srie de Taylor
 *
 * Parmetros:
 *      x (entrada): nmero cujo seno ser calculado
 *      n (entrada): nmero de termos da srie de Taylor
 *
 * Retorno: O seno calculado
 *
 ****/
double Seno(double x, int n)
{
   double soma = 0.0; /* Soma dos termos da srie */
   int    i;

      /* Soma os n termos da srie */
   for (i = 0; i < n; ++i) {
      soma = soma + Termo(x, i);
   }

   return soma;
}

/****
 *
 * main(): Testa a funo Seno()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   printf( "\n\t>>> Este programa calcula senos usando"
           "\n\t>>> serie de Taylor e a funcao sin().\n" );

   printf( "\n\t>>> Seno de %3.2f com 3 termos: %3.2f",
           PI/4, Seno(PI/4, 3) );

   printf( "\n\t>>> Seno de %3.2f com 5 termos: %3.2f",
           PI/4, Seno(PI/4, 5) );

   printf( "\n\t>>> Seno de %3.2f usando sin(): %3.2f\n",
           PI/4, sin(PI/4) );

   return 0;
}

/****************** Resultado do programa ******************

        >>> Este programa calcula senos usando
        >>> serie de Taylor e a funcao sin()

        >>> Seno de 0.79 com 3 termos: 0.71
        >>> Seno de 0.79 com 5 termos: 0.71
        >>> Seno de 0.79 usando sin(): 0.71

***********************************************************/
