/****
 *
 * Ttulo: Validando Datas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Verifica se uma data representada como estrutura 
 *            vlida e, se for o caso, apresenta-a na tela em
 *            trs formatos diferentes
 *
 * Entrada: Uma data
 *
 * Sada: A data introduzida exibida na tela em trs formatos
 *        diferentes (se ela for vlida)
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>  /* printf() */
#include <limits.h> /* INT_MAX  */

/****************** Constantes Simblicas *****************/

#define MENOR_ANO 1752     /* Menor ano utilizado */
#define MAIOR_ANO INT_MAX  /* Maior ano utilizado */

/******************* Definies de Tipos ******************/

typedef struct {
           int  dia, mes, ano;
        } tData;

/************************* Aluses ************************/

extern int LeIntEntre(int min, int max);
extern int EhAnoBissexto(int ano);
extern int NumeroDeDiasNoMes(int mes, int ano);
extern tData* LeDataValida(tData *data);
extern void ExibeDataProlixa(const tData *data);
extern void ExibeDataConvencional(const tData *data);
extern void ExibeDataISO(const tData *data);

/****************** Definies de Funes *****************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeIntEntre(): L um nmero inteiro entre os valores
 *               especificados pelos parmetros
 *
 * Parmetros: inf (entrada) - o menor valor permitido
 *             sup (entrada) - o maior valor permitido
 *
 * Retorno: O nmero lido
 *
 * Observao: Esta funo assume que realmente o primeiro
 *             parmetro  o menor e o segundo parmetro
 *              o maior
 *
 ****/
int LeIntEntre(int inf, int sup)
{
   int num;

   /* Volta para c a cada tentativa frustrada de leitura */
inicioLeitura:
      /* Apresenta prompt e faz uma tentativa de leitura */
   printf( "\n\t>>> Digite um valor inteiro entre %d e "
           "%d\n\t\t> ", inf, sup );
   num = LeInteiro();

      /* Verifica se o valor introduzido satisfaz os   */
      /* critrios especificados. Se no for o caso,   */
      /* apresenta uma mensagem informando o erro ao   */
      /* usurio, e faz uma nova tentativa de leitura. */
   if (num < inf || num > sup) {
      printf("\n\t>>> Valor invalido\n");
      goto inicioLeitura; /* No cria cdigo espaguete */
   }

      /* Seguramente, o nmero retornado */
      /* satisfaz as especificaes      */
   return num;
}

/****
 *
 * EhAnoBissexto(): Verifica se um ano  bissexto
 *
 * Parmetros: ano (entrada): o ano que ser testado
 *
 * Retorno: 1, se o ano for bissexto
 *          0, se o ano no for bissexto
 *
 * Observao: Um ano  bissexto quando:
 *             * Ele  mltiplo de 400 ou
 *             * Ele  mltiplo de 4, mas no
 *                mltiplo de 100
 *
 ****/
int EhAnoBissexto(int ano)
{
      /* Se o ano for anterior ao primeiro */
      /* ano considerado bissexto, ele no */
      /* pode ser assim considerado        */
   if (ano < MENOR_ANO) {
      return 0;
   }

   return !(ano%400) || (!(ano%4) && ano%100);
}

/****
 *
 * NumeroDeDiasNoMes(): Calcula o nmero de dias
 *                      de um determinado ms
 *
 * Parmetros: mes (entrada): o ms em questo
 *             ano (entrada): o ano do ms em questo
 *
 * Retorno: O nmero de dias do ms especificado se o nmero do
 *          ms estiver entre 1 e 12. Zero, em caso contrrio.
 *
 ****/
int NumeroDeDiasNoMes(int mes, int ano)
{
      /* Se o ms no for vlido retorna 0 */
   if (mes < 1 || mes > 12) {
      return 0; /* Ms invlido */
   }

      /* Retorna o nmero de dias do ms. Note que o ms  */
      /* 2  um caso especial, pois o valor retornado     */
      /* depende do fato de o ano ser bissexto ou no. Os */
      /* demais messes possuem nmeros de dias fixos.     */
   switch(mes) {
      case 1:
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12:
         return 31;
      case 2:
         return EhAnoBissexto(ano) ? 29 : 28;
      default:
         return 30;
   }
}

/****
 *
 * LeDataValida(): L uma data vlida
 *
 * Parmetros:
 *     data (sada) - ponteiro para a estrutura contendo a data
 *                    lida e validada
 *
 * Retorno: O endereo da estrutura que armazena os dados lidos
 *
 ****/
tData* LeDataValida(tData *data)
{
   int diasNoMes;

      /* L o ano */
   printf("\n\t\t>>> Introduza o ano <<<");
   data->ano = LeIntEntre(MENOR_ANO, MAIOR_ANO);

      /* L o ms */
   printf("\n\t\t>>> Introduza o mes <<<");
   data->mes = LeIntEntre(1, 12);

      /* Determina o maior valor para o dia */
   diasNoMes = NumeroDeDiasNoMes(data->mes, data->ano);

      /* L o dia */
   printf("\n\t\t>>> Introduza o dia <<<");
   data->dia = LeIntEntre(1, diasNoMes);

   return data; /* Servio completo */
}

/****
 *
 * ExibeDataProlixa(): Exibe uma data na tela em formato prolixo
 *
 * Parmetros:
 *     data (entrada) - ponteiro para a estrutura contendo a data
 *                      a ser exibida
 *
 * Retorno: Nada
 *
 * Observao: Esta funo no checa se a data  vlida
 *
 ****/
void ExibeDataProlixa(const tData *data)
{
   printf("\n\t\t>>> Dia:\t%4.2d", data->dia);
   printf("\n\t\t>>> Mes:\t%4.2d", data->mes);
   printf("\n\t\t>>> Ano:\t%4.2d\n", data->ano);
}

/****
 *
 * ExibeDataConvencional(): Exibe uma data na tela em formato
 *                          convencional
 *
 * Parmetros:
 *     data (entrada): ponteiro para a estrutura contendo a data
 *                     a ser exibida
 *
 * Retorno: Nada
 *
 * Observao: Esta funo no checa se a data  vlida
 *
 ****/
void ExibeDataConvencional(const tData *data)
{
   printf( "\n\t\t--> %.2d/%.2d/%d\n", data->dia,
           data->mes, data->ano );
}

/****
 *
 * ExibeDataISO(): Exibe uma data na tela em formato ISO 8601
 *
 * Parmetros:
 *     data (entrada): ponteiro para a estrutura contendo a data
 *                     a ser exibida
 *
 * Retorno: Nada
 *
 * Observao: Esta funo no checa se a data  vlida
 *
 ****/
void ExibeDataISO(const tData *data)
{
   printf( "\n\t\t--> %d-%.2d-%.2d\n", data->ano,
           data->mes, data->dia );
}

/****
 *
 * main(): Verifica se uma data representada como estrutura 
 *         vlida e, se for o caso, apresenta-a na tela em
 *         trs formatos diferentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tData umaData;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le uma data valida e apresenta-a"
           "\n\t>>> na tela em tres formatos diferentes.\n" );

   LeDataValida(&umaData); /* Garante leitura de uma data vlida */

   printf("\n\t>>> Data em formato prolixo <<<\n");
   ExibeDataProlixa(&umaData);

   printf("\n\t>>> Data em formato convencional <<<\n");
   ExibeDataConvencional(&umaData);

   printf("\n\t>>> Data em formato ISO 8601 <<<\n");
   ExibeDataISO(&umaData);

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa le uma data valida e apresenta-a
        >>> na tela em tres formatos diferentes.

                >>> Introduza o ano <<<
        >>> Digite um valor inteiro entre 1752 e 2147483647
                > 1752

                >>> Introduza o mes <<<
        >>> Digite um valor inteiro entre 1 e 12
                > 2

                >>> Introduza o dia <<<
        >>> Digite um valor inteiro entre 1 e 29
                > 29

        >>> Data em formato prolixo <<<

                >>> Dia:          29
                >>> Mes:          02
                >>> Ano:        1752

        >>> Data em formato convencional <<<

                --> 29/02/1752

        >>> Data em formato ISO 8601 <<<

                --> 1752-02-29

***********************************************************/
