/****
 *
 * Ttulo: Operaes com Vetores ( Vera)
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Executa operaes sobre vetores
 *
 * Entrada: V. exemplo de execuo
 *
 * Sada: V. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()             */
#include <math.h>    /* sqrt() e atan2()     */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */

/***************** Constantes Simblicas ******************/

#define PI               3.1416
#define GRAU_POR_RADIANO 57.3

/****************** Definies de Tipos *******************/

typedef struct {
           double x; /* Componente horizontal */
           double y; /* Componente vertical   */
        } tVetor;

/************************* Aluses ************************/

extern int LeCaractere(void);
extern double LeReal(void);
extern int LeOpcao(const char *opcoes);

extern tVetor *LeVetor(tVetor *vetor);
extern void ExibeVetor(const tVetor *v);
extern tVetor *VetorVezesConstante( tVetor *res, const tVetor *v, double cte );
extern tVetor *SomaVetores( tVetor *res, const tVetor *v1, const tVetor *v2 );
extern double Norma(const tVetor *v);
extern double ProdutoInterno(const tVetor *v1, const tVetor *v2);
extern double Orientacao(const tVetor *v);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com exceo do caractere '\n'
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>>Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeReal(): L um valor do tipo double via teclado
 *           deixando o buffer associado ao teclado
 *           sem caracteres remanescentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido e validado
 *
 ****/
double LeReal(void)
{
   int    teste, /* Armazena o retorno de scanf() */
          nResto = 0; /* Nmero de caracteres */
                      /* excedentes           */
   double valorLido; /* O valor digitado pelo usurio */

      /* O lao encerra quando o usurio se comporta bem */
   while (1) {
         /* Tenta ler um valor do tipo double */
      teste = scanf("%lf", &valorLido);

         /* Se no ocorreu erro de leitura ou de final */
         /* de arquivo, h caracteres remanescentes    */
         /* que precisam ser removidos                 */
      if (teste != EOF) {
         nResto = LimpaBuffer();
      }

         /* Se o valor retornado por scanf() */
         /* foi 1, a leitura foi OK          */
      if (teste == 1) {
         break; /* Leitura foi nota 10 */
      } else { /* Usurio foi mal comportado */
         printf( "\a\n\t>>> O valor digitado e' invalido. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* Retorna um valor vlido do tipo double */
   return valorLido;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

/****
 *
 * LeVetor(): L uma estrutura do tipo tVetor
 *
 * Parmetros:
 *     *vetor (sada) - estrutura que conter o resultado
 *
 * Retorno: O endereo da estrutura que contm o resultado
 *
 ****/
tVetor *LeVetor(tVetor *vetor)
{
   printf("\tComponente horizontal: ");
   vetor->x = LeReal();

   printf("\tComponente vertical: ");
   vetor->y = LeReal();

   return vetor;
}

/****
 *
 * ExibeVetor(): Exibe na tela uma estrutura do tipo tVetor
 *
 * Parmetros:
 *     v (entrada) - ponteiro para a estrutura que ser exibida
 *
 * Retorno: Nada
 *
 ****/
void ExibeVetor(const tVetor *v)
{
   printf( "(%3.1f, %3.1f)\n", v->x, v->y );
}

/****
 *
 * VetorVezesConstante(): Multiplica um vetor por uma constante
 *
 * Parmetros:
 *      *res (sada) - resultado da operao
 *      *v1 (entrada) - vetor a ser multiplicado
 *       cte (entrada) - a constante
 *
 * Retorno: Endereo da estrutura contendo o resultado da operao
 *
 ****/
tVetor *VetorVezesConstante( tVetor *res, const tVetor *v,
                             double cte )
{
   res->x = cte*v->x;
   res->y = cte*v->y;

   return res;
}

/****
 *
 * SomaVetores(): Soma duas estruturas do tipo tVetor
 *
 * Parmetros:
 *      *res (sada): resultado da soma
 *      *v1 (entrada): primeiro vetor
 *      *v2 (entrada): segundo vetor
 *
 * Retorno: O endereo da estrutura que armazena o resultado
 *
 ****/
tVetor *SomaVetores( tVetor *res, const tVetor *v1,
                     const tVetor *v2 )
{
   res->x = v1->x + v2->x;
   res->y = v1->y + v2->y;

   return res;
}

/****
 *
 * Norma(): Calcula a norma de um vetor representado por uma
 *          estrutura do tipo tVetor
 *
 * Parmetros:
 *      *v (entrada): vetor cujo mdulo ser calculado
 *
 * Retorno: O mdulo do vetor recebido como parmetro
 *
 ****/
double Norma(const tVetor *v)
{
   return sqrt(v->x*v->x + v->y*v->y);
}

/****
 *
 * ProdutoInterno(): Calcula o produto interno de dois vetores
 *                   representados por estruturas do tipo tVetor
 *
 * Parmetros:
 *      *v1 (entrada): primeiro vetor
 *      *v2 (entrada): segundo vetor
 *
 * Retorno: O produto interno dos dois vetores
 *
 ****/
double ProdutoInterno(const tVetor *v1, const tVetor *v2)
{
   return v1->x*v2->x + v1->y*v2->y;
}

/****
 *
 * Orientacao(): Calcula a orientao de um vetor representado
 *               por uma estrutura do tipo tVetor
 *
 * Parmetros:
 *      *v (entrada): vetor cuja orientao ser calculada
 *
 * Retorno:
 *   - 0.0, se o vetor for nulo [i.e., (0, 0)].
 *   - Se o vetor no for nulo, o ngulo formado entre o vetor e o
 *     eixo horizontal medido no sentido anti-horrio em radianos.
 *
 ****/
double Orientacao(const tVetor *v)
{
   double angulo;

      /* Se o vetor for nulo, retorna 0.0 */
   if (!v->x && !v->y) {
      return 0.0;
   }

   /**********************************************************/
   /* A chamada de funo atan2(u, v) calcula o arco cuja    */
   /* tangente  v/u, levando em conta o quadrante em que se */
   /* encontra o vetor. A funo atan() tambm calcula arco  */
   /* tangente, mas  incapaz de considerar o quadrante do   */
   /* vetor. As duas funes so declaradas em <math.h>.     */
   /**********************************************************/

   angulo = atan2(v->y, v->x);

      /* Se o ngulo for negativo, ele foi medido no */
      /* sentido horrio. Nesse caso, soma-se 2*PI   */
      /* ao ngulo para obter o resultado desejado.  */
   return angulo < 0 ? 2*PI + angulo : angulo;
}

/****
 *
 * main(): Testa as funes de operaes sobre vetores
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   tVetor v1, v2, resultante;
   double resultado, c;
   int    op;

      /* A sada do lao ocorre quando */
      /* o usurio escolhe a opo '6' */
   while (1) {
         /* Apresenta menu */
      printf( "\n\t\t***** Opcoes *****\n"
              "\n\t1. Multiplica um vetor por uma constante"
              "\n\t2. Soma dois vetores"
              "\n\t3. Calcula a norma de um vetor"
              "\n\t4. Calcula o produto interno de dois vetores"
              "\n\t5. Calcula a orientacao de um vetor"
              "\n\t6. Encerra o programa\n"
              "\n\t>>> Escolha sua opcao: " );

      op = LeOpcao("123456"); /* Obtm a opo do usurio */

      if (op == '6') { /* Opo de encerramento */
         break;
      }

         /* Lida com as demais opes do programa */
      switch (op) {
         case '1': /* Multiplica vetor por constante */
            printf("\nIntroduza o vetor\n");
            LeVetor(&v1);

            printf("\nIntroduza a constante: ");
            c = LeReal();

            VetorVezesConstante(&resultante, &v1, c);

            printf("\n>>> Vetor resultante --> ");
            ExibeVetor(&resultante);

            break;

         case '2': /* Soma dois vetores */
            printf("\nIntroduza o primeiro vetor\n");
            LeVetor(&v1);

            printf("\nIntroduza o segundo vetor\n");
            LeVetor(&v2);

            SomaVetores(&resultante, &v1, &v2);

            printf("\n>>> Vetor resultante --> ");
            ExibeVetor(&resultante);

            break;

         case '3': /* Calcula a norma de um vetor */
            printf("\nIntroduza o vetor\n");
            LeVetor(&v1);

            resultado = Norma(&v1);

            printf( "\n>>> Norma do vetor --> %3.1f\n",
                    resultado );

            break;

         case '4': /* Produto interno de dois vetores */
            printf("\nIntroduza o primeiro vetor\n");
            LeVetor(&v1);

            printf("\nIntroduza o segundo vetor\n");
            LeVetor(&v2);

            resultado = ProdutoInterno(&v1, &v2);
            printf( "\n>>> Produto interno --> %3.1f\n",
                    resultado );

            break;

         case '5': /* Orientacao de um vetor */
            printf("\nIntroduza o vetor\n");
            LeVetor(&v1);

            resultado = Orientacao(&v1);

            printf( "\n>>> Orientacao --> %3.1f radianos "
                    "(%3.1f graus)\n", resultado,
                    resultado*GRAU_POR_RADIANO );

            break;

         default: /* O programa no deve chegar at aqui */
            printf("\nEste programa contem um erro\n");
            return 1;
      } /* switch */
   } /* while */

   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 1

Introduza o vetor
        Componente horizontal: -2
        Componente vertical: 3

Introduza a constante: 5

>>> Vetor resultante --> (-10.0, 15.0)

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 2

Introduza o primeiro vetor
        Componente horizontal: 2
        Componente vertical: -3

Introduza o segundo vetor
        Componente horizontal: -2
        Componente vertical: 3

>>> Vetor resultante --> (0.0, 0.0)

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 3

Introduza o vetor
        Componente horizontal: 3
        Componente vertical: -4

>>> Norma do vetor --> 5.0

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 4

Introduza o primeiro vetor
        Componente horizontal: 3
        Componente vertical: 4

Introduza o segundo vetor
        Componente horizontal: 4
        Componente vertical: 3

>>> Produto interno --> 24.0

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 5

Introduza o vetor
        Componente horizontal: -5
        Componente vertical: 5

>>> Orientacao --> 2.4 radianos (135.0 graus)

                ***** Opcoes *****

        1. Multiplica um vetor por uma constante
        2. Soma dois vetores
        3. Calcula a norma de um vetor
        4. Calcula o produto interno de dois vetores
        5. Calcula a orientacao de um vetor
        6. Encerra o programa

        >>> Escolha sua opcao: 6

        >>> Obrigado por usar este programa.

***********************************************************/
