/****
 *
 * Ttulo: Coordenadas Retangulares e Polares
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/06/2012
 * ltima modificao: 26/06/2012
 *
 * Descrio: Converte pontos entre coordenadas
 *            cartesianas e coordenadas polares
 *
 * Entrada: v. Exemplo de Execuo
 *
 * Sada: v. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()                    */
#include <math.h>    /* sqrt() e atan2()            */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */

/***************** Constantes Simblicas ******************/

#define PI               3.1416
#define NUM_PONTOS       3 /* Tamanho do array de pontos */
#define RADIANO_POR_GRAU (PI/180) /* Radianos em um grau */
#define GRAU_POR_RADIANO (180/PI) /* Graus em um radiano */

/****************** Definies de Tipos *******************/

    /* Tipos de sistemas */
typedef enum {POLAR, CARTESIANO} tSistema;

   /* Pontos em qualquer sistema */
typedef  struct {
            tSistema sistema; /* Sistema de coordenadas  */
            double   a, b;    /* As coordenadas do ponto */
         } tPonto;

/************************* Aluses ************************/

extern int LeCaractere(void);
extern double LeReal(void);
extern int LeOpcao(const char *opcoes);

extern tPonto *LePonto(tPonto *ponto);
extern void ExibePonto(const tPonto *ponto);
extern tPonto *CartesianoParaPolar( tPonto *polar,
                                    const tPonto *cart );
extern tPonto *PolarParaCartesiano( tPonto *cart,
                                    const tPonto *polar );

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>>Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeReal(): L um valor do tipo double via teclado
 *           deixando o buffer associado ao teclado
 *           sem caracteres remanescentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido e validado
 *
 ****/
double LeReal(void)
{
   int    teste, /* Armazena o retorno de scanf() */
          nResto = 0; /* Nmero de caracteres */
                      /* excedentes           */
   double valorLido; /* O valor digitado pelo usurio */

      /* O lao encerra quando o usurio se comporta bem */
   while (1) {
         /* Tenta ler um valor do tipo double */
      teste = scanf("%lf", &valorLido);

         /* Se no ocorreu erro de leitura ou de final */
         /* de arquivo, h caracteres remanescentes    */
         /* que precisam ser removidos                 */
      if (teste != EOF) {
         nResto = LimpaBuffer();
      }

         /* Se o valor retornado por scanf() */
         /* foi 1, a leitura foi OK          */
      if (teste == 1) {
         break; /* Leitura foi nota 10 */
      } else { /* Usurio foi mal comportado */
         printf( "\a\n\t>>> O valor digitado e' invalido. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* Retorna um valor vlido do tipo double */
   return valorLido;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

/****
 *
 * LePonto(): L valores para os campos de uma estrutura do tipo
 *            tPonto
 *
 * Parmetros:
 *     *ponto (sada) - estrutura que conter o resultado
 *
 * Retorno: Endereo da estrutura que contm o resultado
 *
 ****/
tPonto *LePonto(tPonto *ponto)
{
   int op; /* Opo de sistema de coordenadas */

      /* L a opo de sistema de coordenadas */
   printf("\nSistema de coordenadas "
          "(P = Polar, C = cartesiano)? ");
   op = LeOpcao("pPcC");

      /* L as coordenadas de acordo com  */
      /* o sistema escolhido pelo usurio */
   if (op == 'P' || op == 'p') { /* Sistema polar */
      ponto->sistema = POLAR;
      printf("\tCoordenada radial: ");
      ponto->a = LeReal();

         /* No sistema polar, este componente */
         /* no pode ser negativo             */
      while (ponto->a < 0) {
         printf( "\a\n\t>>> Esta coordenada nao pode "
                 "ser negativa <<<\n" );
         printf("\tCoordenada radial: ");
         ponto->a = LeReal();
      }
         /* L a coordenada angular em graus e armazena-a em  */
         /* radianos, pois parece mais natural para o usurio */
      printf("\tCoordenada angular (graus): ");
      ponto->b = RADIANO_POR_GRAU * LeReal();
   } else {  /* Sistema cartesiano */
      ponto->sistema = CARTESIANO;

      printf("\tCoordenada horizontal: ");
      ponto->a = LeReal();

      printf("\tCoordenada vertical: ");
      ponto->b = LeReal();
   }

   return ponto;
}

/****
 *
 * ExibePonto(): Escreve na tela uma estrutura do tipo tPonto
 *
 * Parmetros:
 *     *ponto (entrada): estrutura que ser escrita
 *
 * Retorno: Nada
 *
 ****/
void ExibePonto(const tPonto *ponto)
{
      /* Escreve o ponto de acordo com */
      /* o sistema de coordenadas      */
   if (ponto->sistema == POLAR) { /* Sistema polar */
      printf("\traio = %3.1f\n", ponto->a);

         /* A coordenada angular  escrita em graus. */
         /* Se o ngulo for negativo, soma-o a 2*PI  */
         /* antes de convert-lo em graus.           */
      if (ponto->b < 0) {
         printf( "\tteta (graus) = %3.1f\n",
                 GRAU_POR_RADIANO*(2*PI + ponto->b) );
      } else {
         printf( "\tteta (graus) = %3.1f\n",
                 GRAU_POR_RADIANO*ponto->b );
      }
   } else {  /* Sistema cartesiano */
      printf("\tx = %3.1f\n", ponto->a);
      printf("\ty = %3.1f\n", ponto->b);
   }
}

/****
 *
 * CartesianoParaPolar(): Converte um ponto de coordenadas
 *                        cartesianas para coordenadas polares
 *
 * Parmetros:
 *     *polar (sada): ponto em coordenadas polares
 *     *cart (entrada): ponto em coordenadas cartesianas
 *
 * Retorno: Endereo da estrutura que armazena o resultado
 *
 ****/
tPonto *CartesianoParaPolar(tPonto *polar, const tPonto *cart)
{
      /* Se o sistema do ponto j for polar,  preciso apenas  */
      /* copiar o parmetro de entrada para o parmetro de     */
      /* sada. Caso contrrio,  preciso efetuar a converso. */
   if (cart->sistema == POLAR) {
      *polar = *cart; /* Converso  desnecessria */
   } else { /* Efetua a converso do ponto */
      polar->sistema = POLAR;
      polar->a = sqrt(cart->a*cart->a + cart->b*cart->b);

         /* Se os dois componentes so iguais a 0,  */
         /* o ngulo tambm  0. Caso contrrio,    */
         /* ele ser calculado por meio de atan2(). */
      if (!cart->a && !cart->b) {
         polar->b = 0.0;
      } else {
         polar->b = atan2(cart->b, cart->a);
      }
   }

   return polar;
}

/****
 *
 * PolarParaCartesiano(): Converte um ponto de coordenadas
 *                        polares para coordenadas cartesianas
 *
 * Parmetros:
 *     *cart (sada): ponto em coordenadas cartesianas
 *     *polar (entrada): ponto em coordenadas polares
 *
 * Retorno: Endereo da estrutura que armazena o resultado
 *
 ****/
tPonto *PolarParaCartesiano( tPonto *cart,
                             const tPonto *polar )
{
      /* Se o sistema do ponto j for cartesiano,  */
      /*  necessrio apenas copiar o parmetro de */
      /* entrada para o parmetro de sada. Caso   */
      /* contrrio,  preciso efetuar a converso. */
   if (polar->sistema == CARTESIANO) {
      *cart = *polar; /* Converso no  necessria */
   } else { /* Efetua a converso do ponto */
      cart->sistema = CARTESIANO;
      cart->a = polar->a*cos(polar->b);
      cart->b = polar->a*sin(polar->b);
   }

   return cart;
}

/****
 *
 * main():
 *    1. L um array de estruturas do tipo tPonto
 *    2. Apresenta o array em coordenadas cartesianas
 *    3. Apresenta o array em coordenadas polares
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tPonto pontos[NUM_PONTOS], /* Array que armazena   */
                              /* os pontos do usurio */
          umPonto; /* Usado em converses entre os sistemas */
   int    i;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le %d pontos no sistema "
           "cartesiano ou\n\t>>> polar e exibe-os nos dois"
           "sistemas de coordenadas\n", NUM_PONTOS );

      /* L os pontos via teclado */
   printf("\n\t>>> Introduza os pontos <<<\n\n");
   for (i = 0; i < NUM_PONTOS; ++i) {
      printf("\n>>> Ponto no. %d <<<\n", i + 1);
      LePonto(pontos + i); /* pontos + i  o mesmo */
                           /* que &pontos[i]       */
   }

      /* Apresenta na tela os pontos */
      /* em coordenadas cartesianas  */
   printf( "\n\t>>> Pontos em coordenadas cartesianas <<<" );
   for (i = 0; i < NUM_PONTOS; ++i) {
         /* Note que pontos + i  o mesmo que &pontos[i] */
      PolarParaCartesiano( &umPonto, pontos + i);

      printf("\n>>> Ponto no. %d <<<\n", i + 1);
      ExibePonto(&umPonto);
   }

      /* Apresenta na tela os pontos em coordenadas polares */
   printf("\n\t>>> Pontos em coordenadas polares <<<");
   for (i = 0; i < NUM_PONTOS; ++i) {
         /* Note que pontos + i  o mesmo que &pontos[i] */
      CartesianoParaPolar( &umPonto, pontos + i);

      printf("\n>>> Ponto no. %d <<<\n", i + 1);
      ExibePonto(&umPonto);
   }

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa le 3 pontos no sistema cartesiano ou
        >>> polar e exibe-os nos dois sistemas de coordenadas.

        >>> Introduza os pontos <<<

>>> Ponto no. 1 <<<

Sistema de coordenadas (P = Polar, C = cartesiano)? c
        Coordenada horizontal: 2
        Coordenada vertical: -2

>>> Ponto no. 2 <<<

Sistema de coordenadas (P = Polar, C = cartesiano)? p
        Coordenada radial: -2

        >>> Esta coordenada nao pode ser negativa <<<
        Coordenada radial: 8
        Coordenada angular (graus): 45

>>> Ponto no. 3 <<<

Sistema de coordenadas (P = Polar, C = cartesiano)? c
        Coordenada horizontal: 3
        Coordenada vertical: 4

        >>> Pontos em coordenadas cartesianas <<<

>>> Ponto no. 1 <<<
        x = 2.0
        y = -2.0

>>> Ponto no. 2 <<<
        x = 5.7
        y = 5.7

>>> Ponto no. 3 <<<
        x = 3.0
        y = 4.0

        >>> Pontos em coordenadas polares <<<

>>> Ponto no. 1 <<<
        raio = 2.8
        teta (graus) = 315.0

>>> Ponto no. 2 <<<
        raio = 8.0
        teta (graus) = 45.0

>>> Ponto no. 3 <<<
        raio = 5.0
        teta (graus) = 53.1

***********************************************************/
