/****
 *
 * Ttulo: Problema das N Rainhas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/02/2016
 * ltima modificao: 16/02/2016
 *
 * Descrio: Resolve o problema das N Rainhas
 *
 * Entrada: No h
 *
 * Sada: Diagramas mostrando a posio de cada rainha no tabuleiro
 *
 ****/

#include <stdio.h> /* printf() */

#define N 4 /* Nmero de linhas, colunas ou rainhas */

   /* As constantes do tipo enumerao a seguir indicam se */
   /* uma posio num array de elementos desse tipo est   */
   /* disponvel (i.e., pode acomodar uma rainha) ou no   */
typedef enum {DISPONIVEL, INDISPONIVEL} tDisponibilidade;

/****
 *
 * ApresentaSolucao(): Apresenta na tela um tabuleiro que representa
 *                     uma soluo para o problema das N rainhas
 *
 * Parmetros:
 *      posEmLinha[] (entrada) - array contendo a posio das
 *                               rainhas em cada linha
 *      nSolucao (entrada) - nmero da soluo
 *
 * Retorno: Nada
 *
 ****/
void ApresentaSolucao(const int posEmLinha[], int nSolucao)
{
   int i, j;

      /* Apresenta o nmero da soluo */
   printf("\nSolucao %d:\n\n", nSolucao);

      /***********************/
      /* Desenha o tabuleiro */
      /***********************/

   for(i = 0; i < N; ++i) {  /* Desenha cada linha */
         /* Apresenta cada coluna vazia antes da rainha */
      for(j = 0; j < posEmLinha[i]; ++j)
         printf("\t-");

      printf("\tR"); /* Apresenta a rainha da linha */

         /* Apresenta cada coluna vazia depois da rainha */
      for(j = posEmLinha[i] + 1; j < N; ++j)
         printf("\t-");

      printf("\n"); /* Passa para a prxima linha */
   }
}

/****
 *
 * Posiciona(): Posiciona rainhas num tabuleiro N x N
 *
 * Parmetros:
 *      linha (entrada) - linha na qual ser efetuado o posicionamento
 *      colunas[] (entrada e sada) - array que armazena a disponibilidade
 *                                    em cada coluna
 *      diagonalE[] (entrada e sada) - array que armazena a disponibilidade
 *                                      em cada diagonal esquerda
 *      diagonalD[] (entrada e sada) - array que armazena a disponibilidade
 *                                      em cada diagonal direita
 *      pos[] (sada) - array que armazena a posio da rainha em cada linha
 *     *solucoes (entrada e sada) - nmero de solues encontradas
 *
 * Retorno: Nada
 *
 ****/
void Posiciona( int linha, tDisponibilidade colunas[],
                tDisponibilidade diagonalE[],
                tDisponibilidade diagonalD[], int pos[], int *solucoes )
{
   int coluna;

      /* Procura uma coluna para a rainha na */
      /* linha recebida como parmetro       */
   for (coluna = 0; coluna < N; coluna++) {
         /* Verifica se a coluna corrente est disponvel */
      if ( colunas[coluna] == DISPONIVEL &&
           diagonalE [linha + coluna] == DISPONIVEL &&
           diagonalD[linha - coluna + N - 1] == DISPONIVEL) {
            /* Encontrada uma coluna disponvel e */
            /* a rainha  colocada nessa coluna   */
         pos[linha] = coluna;

            /* Essa coluna passa a estar indisponvel e   */
            /* o mesmo ocorre com as diagonais que cruzam */
            /* a mesma coluna e a mesma linha             */
         colunas[coluna] = INDISPONIVEL;
         diagonalE[linha + coluna] = INDISPONIVEL;
         diagonalD[linha - coluna + N - 1] = INDISPONIVEL;

            /* Se a linha corrente no for a ltima,       */
            /* posiciona uma rainha na prxima linha. Caso */
            /* contrrio, apresenta a soluo corrente.    */
         if (linha < N - 1) {
               /* Posiciona a rainha na prxima linha */
            Posiciona( linha + 1, colunas, diagonalE,
                       diagonalD, pos, solucoes );
         } else { /* Essa era a ltima linha. Portanto, */
                  /* foi encontrada mais uma soluo.   */
               /* Incrementa o nmero de solues  */
               /* encontradas at aqui e apresenta */
               /* a soluo corrente               */
            (*solucoes)++;
            ApresentaSolucao(pos, *solucoes);
         }

            /* Torna a presente coluna e as diagonais  */
            /* que a cruzam disponveis novamente para */
            /* que ocorra retrocesso                   */
         colunas[coluna] = DISPONIVEL;
         diagonalE[linha + coluna] = DISPONIVEL;
         diagonalD[linha - coluna + N - 1] = DISPONIVEL;
      } /* if */
   } /*  for */
}

/****
 *
 * main(): Resolve o quebra-cabeas das N rainhas, delegando
 *         essa tarefa  funo Posiciona()
 *
 * Parmetros: Nenhum
 *
 * Retorno: zero
 *
 ****/
int main(void)
{
      /*                                                */
      /* Esta funo apenas inicia o nmero de solues */
      /* e os arrays que sero utilizados na resoluo  */
      /* do problema. Quem de fato realiza a tarefa    */
      /* a funo Posiciona()                           */
      /*                                                */

   tDisponibilidade colunas[N] = {DISPONIVEL},
                    diagEsq[2*N - 1] = {DISPONIVEL},
                    diagDir[2*N - 1] = {DISPONIVEL};
   int              pLinha[N], /* Posio de rainha numa linha */
                    ns = 0; /* Nmero de solues encontradas */

      /* Passa a bola para a funo Posiciona() */
   Posiciona( 0, colunas, diagEsq, diagDir, pLinha, &ns );

   return 0;
}

/****************** Resultado do Programa *****************

Solucao 1:

        -       R       -       -
        -       -       -       R
        R       -       -       -
        -       -       R       -

Solucao 2:

        -       -       R       -
        R       -       -       -
        -       -       -       R
        -       R       -       -

**********************************************************/
