/****
 *
 * Ttulo: Torres de Hani
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Resolve o problema das Torres de Hani
 *
 * Entrada: Nmero de discos do quebra-cabeas
 *
 * Sada: Instrues informando sobre movimentos dos discos
 *        de uma haste para outra
 *
 ****/

#include <stdio.h> /* printf() */

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeNaturalPositivo(): L um nmero natural positivo via
 *                      teclado e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNaturalPositivo(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * TorresDeHanoi(): Resolve o problema das Torres de Hani
 *
 * Parmetros:
 *      nDiscos (entrada) - nmero de discos
 *      hasteOrigem (entrada) - haste na qual os discos se
 *                              encontram inicialmente
 *      hasteDestino (entrada) - haste  qual os discos se
 *                               destinam
 *      hasteAuxiliar (entrada) - haste usada como auxiliar
 *
 * Retorno: Nada
 *
 ****/
void TorresDeHanoi( int nDiscos, int hasteOrigem,
                    int hasteDestino, int hasteAuxiliar )

{
   if (nDiscos == 1){
         /* Passo 1 do algoritmo: escreve e encerra */
      printf( "Mova o disco 1 da haste %c para a haste %c\n",
              hasteOrigem, hasteDestino );
       return;
   }

      /* Passo 2 do algoritmo: move os n-1 discos */
      /* de A para B usando C como auxiliar       */
   TorresDeHanoi( nDiscos - 1, hasteOrigem,
                  hasteAuxiliar, hasteDestino );

      /* Passo 3 do algoritmo: move ltimo disco de A para C */
   printf( "Mova o disco %d da haste %c para a haste %c\n",
           nDiscos, hasteOrigem, hasteDestino );

      /* Passo 4 do algoritmo: move os n-1 discos */
      /* de B para C usando A como auxiliar       */
   TorresDeHanoi( nDiscos - 1, hasteAuxiliar,
                  hasteDestino,hasteOrigem );
}

/****
 *
 * main(): Testa a funo TorresDeHanoi()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int nDiscos;

      /* Solicita dado ao usurio */
   printf("Introduza o numero de discos: ");
   nDiscos = LeNaturalPositivo();

   TorresDeHanoi(nDiscos, 'A', 'C', 'B');

   return 0;
}

/******************* Exemplo de Execuo ******************

Introduza o numero de discos: 3

Mova o disco 1 da haste A para a haste C
Mova o disco 2 da haste A para a haste B
Mova o disco 1 da haste C para a haste B
Mova o disco 3 da haste A para a haste C
Mova o disco 1 da haste B para a haste A
Mova o disco 2 da haste B para a haste C
Mova o disco 1 da haste A para a haste C

**********************************************************/

