/****
 *
 * Exemplo: Tempo estimado para execuo de operaes
 *
 * Seo: 6.5
 *
 * Descrio: Calcula o tempo estimado para execuo de algumas operaes
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf() */
#include <math.h>    /* powl() e log2() */
#include <limits.h>

/***************** Constantes Simblicas ******************/

#define NANO 1E-9
#define MAX  10000000
#define SEG_POR_DIA (24*3600)
#define SEG_POR_ANO (365*SEG_POR_DIA)

/********************* Funo main() **********************/

/****
 *
 * main(): Calcula o tempo estimado para execuo de algumas operaes
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main()
{
   printf("\n*** SEG_POR_DIA = %d ***", SEG_POR_DIA);
   printf("\n*** SEG_POR_ANO = %d ***\n\n", SEG_POR_ANO);

   for (int n = 10; n <= MAX; n *= 100) {
      printf("n = %d\n------------\n", n);
      printf("\nlog n = %G", NANO*log2(n));
      printf("\nn = %G", NANO*n);
      printf("\nnlog n = %G", n*NANO*log2(n));
      printf("\nn2 = %G", NANO*n*n);
      printf("\nn3 = %G\n\n", NANO*n*n*n);
   }

   printf("\n------------------------------------------\n");

   for (int n = 10; n <= MAX; n *= 100)
      printf( "\nn = %d\t\t2n = %LG", n, NANO*powl(2, n)/SEG_POR_ANO );

   printf("\n\n------------------------------------------\n");

   return 0;
}

/****************** Resultado do Programa *****************

*** SEG_POR_DIA = 86400 ***
*** SEG_POR_ANO = 31536000 (3.15E7) ***

n = 10
------------

log n = 3.3E-9
n = 1E-008
nlog n = 3.3E-8
n2 = 1E-7
n3 = 1E-6
2n = 1E-6

n = 1.000
------------

log n = 1E-8
n = 1E-6
nlog n = 1E-5
n2 = 0.001
n3 = 1
2n = 3.4E284 (anos)

n = 100.000
------------

log n = 1.7E-8
n = 0.0001
nlog n = 0.0017
n2 = 10
n3 = 1E6 (11,6 dias)

n = 10.000.000
------------

log n = 2.3E-8
n = 0.01
nlog n = 0.23
n2 = 100000 (27,8 minutos)
n3 = 1E12 (31.710 anos)

------------------------------------------

n = 10          2n = 1.024E-006
n = 1000        2n = 1.07151E+292
n = 100000      2n = 1.18973E+4923
n = 10000000    2n = 1.18973E+4923

------------------------------------------

            Em anos:

n = 1000        2n = 3.4E284 OK
n = 100000      2n = 3.77261E+4915
n = 10000000    2n = 3.77261E+4915

------------------------------------------

**********************************************************/
