/****
 *
 * Ttulo: Lista Esttica sem Ordenao
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada          */
#include <stdlib.h>   /* exit()                   */
#include "ASSEGURA.h" /* Interface do tipo lista  */
#include "Lista.h"    /* Interface do tipo lista  */

/***************** Definies de Funes ******************/

/****
 *
 * EstaCheia(): Verifica se uma lista sem ordenao est cheia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver cheia; 0, em caso contrario
 *
 ****/
static int EstaCheia(const tListaIdx *lista)
{
   return lista->nElementos >= MAX_ELEMENTOS;
}

/****
 *
 * IniciaLista(): Inicia uma lista sem ordenao
 *
 * Parmetros: lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void IniciaLista(tListaIdx *lista)
{
   lista->nElementos = 0;
}

/****
 *
 * Comprimento(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento ser calculado
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
int Comprimento(const tListaIdx *lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ObtemElemento(): Obtm o elemento de uma lista que se encontra numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o elemento
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
tElemento ObtemElemento(const tListaIdx *lista, int indice)
{
   ASSEGURA(indice >= 0 && indice < lista->nElementos, "Elemento inexistente");

   return lista->elementos[indice];
}

/****
 *
 * InsereElemento(): Insere um novo elemento numa lista numa posio especificada
 *
 * Parmetros:
 *      lista (entrada e sada) - lista na qual ser feita a insero
 *      item (entrada) - elemento que ser inserido
 *      indice (entrada) - posio de insero
 *
 * Retorno: Nada
 *
 ****/
int InsereElemento(tListaIdx *lista, tElemento item, int indice)
{
      /* Se a lista estiver cheia, no h */
      /* como inserir mais um elemento    */
   ASSEGURA (!EstaCheia(lista), "A lista esta cheia");

      /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice <= lista->nElementos,
             "Posicao de insercao inexistente" );

      /* Abre espao para o novo elemento */
   for (int i = lista->nElementos - 1; i >= indice; --i)
      lista->elementos[i + 1] = lista->elementos[i];

      /* Insere o novo elemento */
   lista->elementos[indice] = item;

   lista->nElementos++; /* O tamanho da lista aumentou */

   return 0;
}

/****
 *
 * RemoveElemento(): Remove um elemento numa dada posio
 *                   de uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a remoo
 *       indice (entrada) - posio do elemento que ser removido
 *
 * Retorno: O item removido
 *
 ****/
tElemento RemoveElemento(tListaIdx *lista, int indice)
{
   tElemento itemRemovido;

         /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Posicao de remocao inexistente" );

   itemRemovido = lista->elementos[indice];

      /* Remover um elemento significa mover cada elemento  */
      /* uma posio para trs a partir do prximo elemento */
      /* adiante daquele que ser  removido                 */
   for (int i = indice; i < lista->nElementos - 1; i++)
      lista->elementos[i] = lista->elementos[i + 1];

   --lista->nElementos; /* O tamanho da lista diminuiu */

   return itemRemovido;
}

/****
 *
 * EstaVazia(): Verifica se uma lista sem ordenao est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(const tListaIdx *lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * BuscaElemento(): Procura um elemento numa lista que
 *                  apresenta um determinado valor
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       elemento (entrada) - valor do elemento que ser
 *                            procurado
 *
 * Retorno: ndice do elemento se ele for encontrado;
 *          -1 em caso contrrio
 *
 ****/
int BuscaElemento(const tListaIdx *lista, tElemento elemento)
{
   for (int i = 0; i < lista->nElementos; ++i) {
      if (lista->elementos[i] == elemento) {
         return i; /* Elemento foi encontrado */
      }
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * AcrescentaElemento(): Acrescenta um novo elemento ao final
 *                       de uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (sada) - lista onde ser feita o acrscimo
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: 0, se a operao for bem sucedida;
 *          1, em caso contrrio
 *
 ****/
int AcrescentaElemento(tListaIdx *lista, tElemento elemento)
{
      /* Verifica se  possvel acrescentar */
      /* mais um elemento na lista          */
   ASSEGURA (!EstaCheia(lista), "A lista esta cheia");


      /* Acrescenta um novo elemento ao final da lista */
   lista->elementos[lista->nElementos] = elemento;

   ++lista->nElementos; /* O tamanho da lista aumentou */

   return 0;
}

/****
 *
 * AlteraElemento(): Altera o valor de um elemento que se
 *                   encontra num determinada posio de
 *                   uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada e sada) - lista na qual o elemento se encontra
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: 0, se a operao for bem sucedida;
 *          1, em caso contrrio
 *
 ****/
int AlteraElemento(tListaIdx *lista, int indice, tElemento novoValor)
{
         /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Posicao de alteracao inexistente" );

      /* Altera o valor do elemento */
   lista->elementos[indice] = novoValor;

   return 0;
}

