/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada         */
#include <stdlib.h>   /* exit()                  */
#include "ASSEGURA.h"    /* Interface do tipo lista */
#include "Lista.h"    /* Interface do tipo lista */

/***************** Definies de Funes ******************/

/****
 *
 * EstaCheia(): Verifica se uma lista sem ordenao est cheia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver cheia; 0, em caso contrario
 *
 ****/
static int EstaCheia(const tListaIdx *lista)
{
   return lista->nElementos >= MAX_ELEMENTOS;
}

/****
 *
 * IniciaLista(): Inicia uma lista sem ordenao
 *
 * Parmetros: lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void IniciaLista(tListaIdx *lista)
{
   lista->nElementos = 0;
}

/****
 *
 * Comprimento(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento
 *                                ser calculado
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
int Comprimento(const tListaIdx *lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ObtemElemento(): Obtm o elemento de uma lista que se encontra
 *              numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o elemento
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
tElemento ObtemElemento(const tListaIdx *lista, int indice)
{
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Elemento inexistente" );

   return lista->elementos[indice];
}

/****
 *
 * InsereEmOrdem(): Insere um novo elemento numa lista de
 *                  modo a preservar a ordenao da lista
 *
 * Parmetros:
 *      lista (entrada e sada) - lista na qual ser feita
 *                                a insero
 *      elemento (entrada) - elemento que ser inserido
 *
 * Retorno: 0, se o elemento for inserido;
 *          1, em caso contrrio
 *
 ****/
int InsereEmOrdem(tListaIdx *lista, tElemento elemento)
{
   int posicao, /* Posio de insero do elemento */
       i;

      /* Se a lista estiver cheia, no h */
      /* como inserir mais um elemento    */
   ASSEGURA (!EstaCheia(lista), "A lista esta cheia");

      /* Se a lista estiver vazia, o elemento  */
      /* inserido na primeira posio do array  */
   if (EstaVazia(lista)) {
         /* O elemento ser o primeiro da lista */
      lista->elementos[0] = elemento;

      ++lista->nElementos; /* A lista cresceu */

      return 0; /* Servio completo */
   }

      /* Encontra a posio no array onde */
      /* o elemento ser inserido         */
   for (posicao = 0; posicao < lista->nElementos; ++posicao) {
      /*                                           */
      /* Se for encontrado um elemento do array    */
      /* cujo valor  maior do que o valor do      */
      /* elemento a ser inserido,  na posio     */
      /* atual desse elemento que o elemento       */
      /* recebido como parmetro ser inserido.    */
      /* Se tal elemento no for encontrado, a     */
      /* insero ser efetuada ao final do array. */
      /*                                           */

      if (lista->elementos[posicao] > elemento) {
         break; /* Posio de insero encontrada */
      }
   }

   /************************************************/
   /* Neste ponto, a varivel 'posicao' armazena o */
   /* ndice no qual o novo elemento ser inserido */
   /************************************************/

      /* Abre espao para o novo elemento */
   for (i = lista->nElementos - 1; i >= posicao; --i) {
         /* Move cada elemento uma posio       */
         /* adiante a partir do elemento que ora */
         /* se encontra na posio de insero   */
      lista->elementos[i + 1] = lista->elementos[i];
   }

      /* Insere o novo elemento */
   lista->elementos[posicao] = elemento;

   ++lista->nElementos; /* A lista cresceu */

   return 0;
}

/****
 *
 * RemoveElemento(): Remove um elemento numa dada posio
 *                   de uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a
 *                               remoo
 *       indice (entrada) - posio do elemento que ser removido
 *
 * Retorno: O item removido
 *
 ****/
tElemento RemoveElemento(tListaIdx *lista, int indice)
{
   tElemento itemRemovido;

         /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Posicao de remocao inexistente" );

   itemRemovido = lista->elementos[indice];

      /* Remover um elemento significa mover cada elemento  */
      /* uma posio para trs a partir do prximo elemento */
      /* adiante daquele que ser  removido                 */
   for (int i = indice; i < lista->nElementos - 1; i++)
      lista->elementos[i] = lista->elementos[i + 1];

   --lista->nElementos; /* O tamanho da lista diminuiu */

   return itemRemovido;
}

/****
 *
 * EstaVazia(): Verifica se uma lista sem ordenao est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(const tListaIdx *lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * BuscaBinaria(): Efetua uma busca binria numa
 *                 lista de elementos do tipo int
 *                 implementada usando array esttico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       elemento (entrada) - valor do elemento que ser
 *                            procurado
 *
 * Retorno: ndice do primeiro elemento encontrado no array
 *          que casa com o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 * Observao: Na busca binria, quando um valor  encontrado
 *             e existe mais de uma ocorrncia desse valor no
 *             array, o ndice retornado no corresponde
 *             necessariamente ao menor ndice de elemento
 *             que contm o valor procurado.
 *
 ****/
int BuscaBinaria(const tListaIdx *lista, tElemento elemento)
{
   int inf, /* Limite inferior da busca */
       sup, /* Limite superior da busca */
       meio; /* Meio do intervalo */

      /* limites inferior e superior iniciais */
   inf = 0;
   sup = lista->nElementos - 1;

      /* Efetua a busca binria */
   while (inf <= sup) {
         /* Calcula o meio do intervalo */
      meio = inf + (sup - inf)/2;

         /* Verifica se o elemento se encontra */
         /* no meio do intervalo            */
      if (lista->elementos[meio] == elemento)
         return meio; /* Elemento encontrado */

         /* Ajusta o intervalo de busca */
      if (elemento  < lista->elementos[meio])
         sup = meio - 1;
      else
         inf = meio + 1;
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * BuscaBinariaRec(): Efetua uma busca binria recursiva
 *                    numa lista de elementos do tipo int
 *                    implementada usando array esttico
 *
 * Parmetros:
 *      ar (entrada) - lista na qual ser feita a busca
 *      inf (entrada) - ndice inferior da busca
 *      sup (entrada) - ndice superior da busca
 *      valor (entrada) - valor que ser procurado
 *
 * Retorno: ndice do primeiro elemento encontrado no array
 *          que casa com o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 ****/
static int BuscaBinariaRec(const tElemento elementos[],
                           int inf, int sup, tElemento valor)
{
   int meio;

   if (inf > sup)
      return -1;

   meio = inf + (sup - inf)/2;

   if (valor == elementos[meio])
      return meio;
   else if (valor < elementos[meio])
      return BuscaBinariaRec(elementos, inf, meio - 1, valor);

   return BuscaBinariaRec(elementos, meio + 1, sup, valor);
}

/****
 *
 * BuscaBinaria2(): Funo acionadora de BuscaBinariaRec()
 *
 * Parmetros:
 *      ar (entrada) - lista na qual ser feita a busca
 *      nElementos (entrada) - nmero de elementos da lista
 *      valor (entrada) - valor que ser procurado
 *
 * Retorno: O mesmo da funo BuscaBinariaRec().
 *
 ****/
int BuscaBinaria2(const tListaIdx *lista, tElemento elemento)
{
   return BuscaBinariaRec(lista->elementos, 0,
                          lista->nElementos - 1, elemento);
}
