/****
 *
 * Ttulo: Lista Indexada sem Ordenao
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Gerencia uma lista de compras permitindo
 *            desfazer e refazer operaes
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include <stdlib.h>   /* Funes de alocao dinmica e */
                      /* gerao de nmeros aleatrios  */
#include <time.h>     /* Funo time() para alimentar o */
                      /* gerador de nmeros aleatrios  */
#include "Lista.h"    /* Interface do tipo lista  */
#include "leitura.h"  /* Funes de leitura       */

/***************** Definies de Funes ******************/

/****
 *
 * ExibeDoInicio(): Exibe na tela do primeiro ao ltimo
 *                  elemento de uma lista indexada
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeDoInicio(tListaIdx *lista)
{
   int tamanho = Comprimento(lista);

   printf("\n\t*** Lista ***\n");

   for (int i = 0; i < tamanho; ++i) {
      printf("%d\t", ObtemElemento(lista, i));
   }

   putchar('\n');
}

/****
 *
 * ExibeDoFinal(): Exibe na tela do ltimo ao primeiro
 *                  elemento de uma lista indexada
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeDoFinal(tListaIdx *lista)
{
   int tamanho = Comprimento(lista);

      printf("\n\t*** Lista Invertida ***\n");

for (int i = tamanho - 1; i >= 0; --i) {
      printf("%d\t", ObtemElemento(lista, i));
   }

   putchar('\n');
}

/****
 *
 * ApresentaMenu(): Apresenta o menu do programa
 *
 * Parmetros: No h
 *
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(void)
{
   printf( "\n\t>>>>> Opcoes <<<<<\n"
           "\n\t(1) Acrescenta um elemento"
           "\n\t(2) Remove um elemento"
           "\n\t(3) Altera dados"
           "\n\t(4) Posicao de um elemento"
           "\n\t(5) Exibe a lista do inicio"
           "\n\t(6) Exibe a lista do final"
           "\n\t(7) Comprimento da lista"
           "\n\t(8) Encerra o programa"
           "\n\n\t>>> Escolha sua opcao: " );
}

/****
 *
 * main(): Testa as operaes sobre listas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaIdx lista;
   int    i, valor, indice, op;

   IniciaLista(&lista);

      /* Ser criada uma lista inicial com 10 elementos. */
      /* Os contedos destes elementos sero nmeros     */
      /* inteiros entre 0 e 100 gerados aleatoriamente   */

      /* Alimenta o gerador de nmeros aleatrios */
   srand((int) time(NULL));

      /* Cria a lista inicial */
   for (i = 1; i <= 10; ++i) {
      valor = (int) rand()%100 + 1;
      InsereEmOrdem(&lista, valor);
   }

   while (1) {
      ApresentaMenu();

      op = LeOpcao("12345678");

      if (op == '8') { /* Encerra o programa */
         printf("\nBye!\n");
         break; /* Sada do lao */
      }

      switch (op) {
         case '1': /* Acrescenta um elemento */
            printf("\n>>> Digite o valor a ser acrescentado: ");
            valor = LeInteiro();

            if (InsereEmOrdem(&lista, valor)) {
               printf("\n>>> Impossivel acrescentar o aluno\n");
            } else {
                  /* Acrscimo foi bem sucedido. Faz o ponteiro */
                  /* original apontar para turma redimensionada */
               printf("\n>>> Acrescimo bem sucedido\n");
            }
            break;

         case '2': /* Remove um aluno */
            printf("\n>>> Digite o valor a ser removido: ");
            valor = LeInteiro();

            indice = BuscaBinaria(&lista, valor);

            if (indice >= 0) {
               (void) RemoveElemento(&lista, indice);
               printf("\n>>> Remocao bem sucedida\n");
            } else {
               printf("\n>>> Valor nao encontrado\n");
            }

            break;

         case '3': /* Altera valor de um elemento */
            printf("\n>>> Digite o valor a ser alterado: ");
            valor = LeInteiro();

            indice = BuscaBinaria(&lista, valor);

            if (indice >= 0) {
               (void) RemoveElemento(&lista, indice);

               printf("\n>>> Digite o valor que substituira' %d: ", valor);
               valor = LeInteiro();

               InsereEmOrdem(&lista, valor);
               printf("\n>>> alteracao bem sucedida\n");
            } else {
               printf("\n>>> Valor nao encontrado\n");
            }

            break;

         case '4': /* Posio de um elemento */
            printf("\n>>> Digite o valor do elemento: ");
            valor = LeInteiro();

            indice = BuscaBinaria(&lista, valor);

            if (indice >= 0) {
               printf("\n>>> O elemento %d: se encontra na "
                      "posicao %d", valor, indice);
            } else {
               printf("\n>>> Valor nao encontrado\n");
            }

            break;

         case '5': /* Apresenta a lista a partir do incio */
            ExibeDoInicio(&lista);
            break;

         case '6': /* Apresenta a lista a partir do final */
            ExibeDoFinal(&lista);
            break;

         case '7': /* Apresenta o comprimento da lista */
            printf( "\n>>>Comprimento da lista: %d\n",
                    Comprimento(&lista) );
            break;

         default: /* O programa no deve chegar at aqui */
            printf("\nEste programa contem um erro\n");
            return 1;
      }
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

Lista a partir do inicio:
13      10      1       16      42      7       83      41      11      74

Lista a partir do final:
74      11      41      83      7       42      16      1       10      13
Comprimento da lista: 10


Elemento de indice 5: 7

Elemento de indice 5 apos incremento: 8

Lista a partir do inicio:
13      10      1       16      42      8       83      41      11      74

Um elemento de valor 99 sera' inserido na posicao 5

Comprimento da lista: 11

Lista a partir do inicio:
13      10      1       16      42      99      8       83      41      11      74

Elemento removido na posicao 5: 99

Comprimento da lista: 10

Lista a partir do inicio:
13      10      1       16      42      8       83      41      11      74

**********************************************************/
