/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada         */
#include <string.h>   /* Funes strXXX()        */
#include <stdlib.h>   /* malloc()                */
#include "ASSEGURA.h" /* Macro ASSEGURA()        */
#include "Lista.h"    /* Interface do tipo lista */

/***************** Definies de Funes ******************/

/****
 *
 * EstaCheia(): Verifica se uma lista est cheia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver cheia; 0, em caso contrario
 *
 ****/
int EstaCheia(const tListaAlunos *lista)
{
   return lista->nAlunos >= MAX_ELEMENTOS;
}

/****
 *
 * IniciaLista(): Inicia uma lista sem ordenao
 *
 * Parmetros: lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void IniciaLista(tListaAlunos *lista)
{
   lista->nAlunos = 0;
}

/****
 *
 * Comprimento(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento ser calculado
 *
 * Retorno: o nmero de alunos na lista
 *
 ****/
int Comprimento(const tListaAlunos *lista)
{
   return lista->nAlunos; /* No h o que calcular */
}

/****
 *
 * ObtemElemento(): Obtm o elemento de uma lista que se
 *                  encontra numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o ndice do elemento
 *
 * Retorno: o nmero de alunos na lista
 *
 ****/
tAluno ObtemElemento(const tListaAlunos *lista, int indice)
{
   ASSEGURA( indice >= 0 && indice < lista->nAlunos, "Elemento inexistente" );

   return lista->alunos[indice];
}

/****
 *
 * InsereEmOrdem(): Insere um novo elemento numa lista de
 *                  modo a preservar a ordenao da lista
 *
 * Parmetros:
 *      lista (entrada e sada) - lista na qual ser feita a insero
 *      aluno (entrada) - elemento que ser inserido
 *
 * Retorno: 0, se o elemento for inserido; 1, em caso contrrio
 *
 ****/
int InsereEmOrdem(tListaAlunos *lista, const tAluno *aluno)
{
   int posicao, /* Posio de insero do elemento */
       i;

      /* Se a lista estiver cheia, no h como inserir mais um elemento */
   if (EstaCheia(lista))
      return 1;

      /* Se a lista estiver vazia, o elemento  */
      /* inserido na primeira posio do array  */
   if (EstaVazia(lista)) {
      lista->alunos[0] = *aluno; /* O elemento ser o primeiro da lista */

      ++lista->nAlunos; /* A lista cresceu */

      return 0; /* Servio completo */
   }

      /* Encontra a posio na lista onde o elemento ser inserido */
   for (posicao = 0; posicao < lista->nAlunos; ++posicao) {
      /*                                                                     */
      /* Se for encontrado um elemento da lista cujo valor  maior do que o  */
      /* valor do elemento a ser inserido,  na posio atual desse elemento */
      /* que o elemento recebido como parmetro ser inserido. Se ele no    */
      /* for encontrado, a insero ser efetuada ao final da lista.         */
      /*                                                                     */

      if ( strcmp(lista->alunos[posicao].nome, aluno->nome) > 0 )
         break; /* Posio de insero encontrada */
   }

   /************************************************/
   /* Neste ponto, a varivel 'posicao' armazena o */
   /* ndice no qual o novo elemento ser inserido */
   /************************************************/

      /* Abre espao para o novo elemento */
   for (i = lista->nAlunos - 1; i >= posicao; --i)
         /* Move cada elemento uma posio adiante a partir do  */
         /* elemento que ora se encontra na posio de insero */
      lista->alunos[i + 1] = lista->alunos[i];

   lista->alunos[posicao] = *aluno; /* Insere o novo elemento */

   ++lista->nAlunos; /* A lista cresceu */

   return 0;
}

/****
 *
 * RemoveElemento(): Remove um elemento numa dada posio
 *                   de uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a remoo
 *       indice (entrada) - posio do elemento que ser removido
 *
 * Retorno: O item removido
 *
 ****/
tAluno RemoveElemento(tListaAlunos *lista, int indice)
{
   tAluno itemRemovido;

         /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nAlunos,
             "Posicao de remocao inexistente" );

   itemRemovido = lista->alunos[indice];

      /* Remover um elemento significa mover cada elemento uma posio para   */
      /* trs a partir do prximo elemento adiante daquele que ser  removido */
   for (int i = indice; i < lista->nAlunos; i++)
      lista->alunos[i] = lista->alunos[i + 1];

   --lista->nAlunos; /* O tamanho da lista diminuiu */

   return itemRemovido;
}

/****
 *
 * EstaVazia(): Verifica se uma lista sem ordenao est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(const tListaAlunos *lista)
{
   return lista->nAlunos == 0;
}

/****
 *
 * BuscaBinariaNome(): Efetua uma busca binria numa lista de elementos do
 *                 tipo int implementada usando array esttico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       elemento (entrada) - valor do elemento que ser procurado
 *
 * Retorno: ndice do primeiro elemento encontrado na lista
 *          que casa com o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 * Observao: Na busca binria, quando um valor  encontrado
 *             e existe mais de uma ocorrncia desse valor no
 *             array, o ndice retornado no corresponde
 *             necessariamente ao menor ndice de elemento
 *             que contm o valor procurado.
 *
 ****/
int BuscaBinariaNome(const tListaAlunos *lista, tNome nomeProcurado)
{
   int inf, /* Limite inferior da busca */
       sup, /* Limite superior da busca */
       meio; /* Meio do intervalo */

      /* limites inferior e superior iniciais */
   inf = 0;
   sup = lista->nAlunos - 1;

      /* Efetua a busca binria */
   while (inf <= sup) {
         /* Calcula o meio do intervalo */
      meio = inf + (sup - inf)/2;

         /* Verifica se o elemento se encontra no meio do intervalo */
      if (!strcmp(lista->alunos[meio].nome, nomeProcurado))
         return meio; /* Elemento encontrado */

         /* Ajusta o intervalo de busca */
      if (strcmp(nomeProcurado, lista->alunos[meio].nome) < 0)
         sup = meio - 1;
      else
         inf = meio + 1;
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * BuscaSequencialMatr(): Efetua uma busca sequencial numa lista de elementos
 *                    do tipo tAluno implementada usando array esttico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       matrProcurada (entrada) - valor da matrcula do elemento
 *                                  que ser procurado
 *
 * Retorno: ndice do primeiro elemento na lista que casa com
 *          o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 ****/
int BuscaSequencialMatr(const tListaAlunos *lista, tMatricula matrProcurada)
{
   for (int i = 0; i < lista->nAlunos; ++i)
      if (!strcmp(lista->alunos[i].matr, matrProcurada))
         return i; /* Elemento foi encontrado */

   return -1; /* Elemento no foi encontrado */
}
