/****
 *
 * Ttulo: Lista Ordenada de Alunos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: ???
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include <stdlib.h>   /* Funo strtod()          */
#include "Lista.h"    /* Interface do tipo lista  */
#include "Leitura.h"  /* Leitura via teclado      */
#include "ASSEGURA.h" /* Macro ASSEGURA()         */

/***************** Constantes Simblicas ******************/

   /* Nome do arquivo de texto (diretrio corrente) */
#define NOME_ARQUIVO  "Tudor.txt"

   /* Tamanho mximo de uma linha no arquivo */
#define TAMANHO_LINHA 60

   /* Nome do arquivo binrio (diretrio corrente) */
#define NOME_ARQ_BIN "Tudor.bin"

/***************** Definies de Funes ******************/

/****
 *
 * ExibeElemento(): Exibe na tela um elemento de uma lista ordenada
 *
 * Parmetros: *aluno (entrada) - elemento que ser exibido
 *
 * Retorno: Nada
 *
 ****/
static void ExibeElemento(const tAluno *aluno)
{
   printf("\n%s", aluno->nome);
   printf("\t%s", aluno->matr);
   printf("\t%3.1f", aluno->n1);
   printf("\t%3.1f", aluno->n2);
}

/****
 *
 * ExibeLista(): Apresenta na tela, em forma de tabela, os elementos de uma
 *               lista de estruturas do tipo tAluno armazenada num array
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeLista(const tListaAlunos *lista)
{
   tAluno aluno;
   int    tamanho = Comprimento(lista);

      /* Se a lista estiver vazia, no h o que exibir */
   if (!tamanho) {
      printf("\n*** Lista Vazia *** \n");
      return;
   }

      /* Cabealho de apresentao na tela */
   printf( "\n   Nome   \tMatr\tN1\tN2"
           "\n   ====   \t====\t==\t==\n");

      /* Apresenta cada estrutura numa linha separada */
   for (int i = 0; i < tamanho; ++i) {
      aluno = ObtemElemento(lista, i);
      ExibeElemento(&aluno);
   }

   putchar('\n'); /* Embelezamento apenas */
}

/****
 *
 * LeArquivo(): L arquivo de dados (texto) e o armazena
 *              numa lista ordenada por nome
 *
 * Parmetros:
 *      arq (entrada) - nome do arquivo
 *     *lista (sada) - lista que conter os registros lidos
 *
 * Retorno: Nada
 *
 ****/
void LeArquivo( const char *arq, tListaAlunos *lista )
{
   char    linha[TAMANHO_LINHA + 1];
   FILE   *stream;
   tAluno  umAluno;
   char   *str;

   stream = fopen(arq, "r"); /* Abre arquivo em formato texto para leitura */

      /* Se o arquivo no puder ser aberto, nada mais pode ser feito */
   ASSEGURA(stream, "Arquivo nao pode ser aberto");

      /* L cada linha do arquivo e a usa para construir um registro. */
      /* Ento, o registro  inserido na lista ordenada.              */
   while ( fgets(linha, TAMANHO_LINHA + 1, stream) ) {
         /* Obtm o nome e acrescenta-o ao registro*/
      str = strtok(linha, "\t\n");
      strcpy(umAluno.nome, str);

         /* Obtm a matrcula e acrescenta-a ao registro*/
      str = strtok(NULL, "\t\n");
      strcpy(umAluno.matr, str);

         /* Obtm a 1a. nota e acrescenta-a convertida em double ao registro */
      str = strtok(NULL, "\t\n");
      umAluno.n1 = strtod(str, NULL);

         /* Obtm a 2a. nota e acrescenta-a convertida em double ao registro */
      str = strtok(NULL, "\t\n");
      umAluno.n2 = strtod(str, NULL);

      ASSEGURA( !InsereEmOrdem(lista, &umAluno),
                "Nao foi possivel inserir um elemento na lista" );
   }

   fclose(stream); /* Processamento terminado. Fecha o arquivo. */
}

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros:
 *      opcoes[] (entrada) - array de strings contendo as opes do usurio
 *      nOpcoes (entrada) - o nmero de opes
 *
 * Retorno: Nada
 *
 ****/
static void ApresentaMenu(const char *opcoes[], int nOpcoes)
{
   int i;

   printf("\nEscolha uma das opcoes a seguir:\n");

   for (i = 0; i < nOpcoes; ++i)
      printf("\n\t(%d) %s", i + 1, opcoes[i]);

   printf("\n\n\t>>> ");
}

/****
 *
 * LeDadosAluno(): L os dados de um aluno
 *
 * Parmetros: *aluno (sada) - estrutura que receber os
 *                              dados introduzidos pelo usurio
 *             fMatr (entrada) - indica se o campo 'matr' ser lido ou no
 *
 * Retorno: O parmetro
 *
 * Observao: O parmetro 'fMatr'  uma flag que determina se a matrcula deve
 *             ser lida ('fMatr' diferente de zero) ou no ('fMatr' igual a zero).
 *             A justificativa para o uso deste parmetro  que, algumas vezes,
 *             quando esta funo  chamada, a matrcula j ter sido lida.
 *             Esta alternativa  melhor do que solicitar novamente a matrcula
 *             ao usurio (inconveniente) ou escrever duas funes que diferem
 *             apenas pelo fato de solicitarem matrcula ou no.
 *
 ****/
tAluno *LeDadosAluno(tAluno *aluno, int fMatr)
{
   LeNome(aluno->nome, MAX_NOME + 1); /* L o nome */

      /* Se indicado pelo parmetro 'fMatr', l a matrcula    */
   if (fMatr)
      LeMatricula(aluno->matr, TAM_MATR + 1);

      /* L a primeira nota */
   printf("\n\t>>> 1a. nota:\n\t    > ");
   aluno->n1 = LeNota();

      /* L a segunda nota */
   printf("\n\t>>> 2a. nota:\n\t    > ");
   aluno->n2 = LeNota();

   return aluno;
}

/****
 *
 * AtualizaArquivoBin(): Escreve cada estrutura do tipo tAluno
 *                       de uma lista num arquivo binrio
 *
 * Parmetros:
 *      nomeArquivo (entrada) - nome do arquivo binrio no qual
 *                              as estruturas sero escritas
 *     *lista (entrada) - lista contendo as estruturas que sero escritas no arquivo
 *
 * Retorno: 0, se ocorrer zero erro. 1, em caso contrrio.
 *
 ****/
void AtualizaArquivoBin( const char *nomeArquivo, const tListaAlunos *lista )
{
   FILE   *stream; /* Stream associado ao arquivo no qual ser feita a escrita */
   int     nElementosEscritos; /* Nmero de elementos do array */
                               /* que foram realmente escritos */

      /* Abre arquivo em formato binrio para escrita */
   stream = fopen(nomeArquivo, "wb");

      /* Se o arquivo no foi aberto, nada mais pode ser feito */
   ASSEGURA(stream, "Arquivo nao pode ser aberto");

      /* Escreve todos os elementos da lista no arquivo */
   nElementosEscritos = fwrite( lista->alunos, sizeof(lista->alunos[0]),
                                lista->nAlunos, stream );

   fclose(stream); /* Processamento terminado. Fecha o arquivo. */

      /* Se o nmero de elementos escritos for diferente do   */
      /* nmero de elementos do array, deve ter ocorrido erro */
   ASSEGURA( nElementosEscritos == lista->nAlunos,
             "Erro de escrita no arquivo binario" );
}

/****
 *
 * main(): Testa as operaes sobre listas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaAlunos lista;
   tAluno       aluno;
   const char  *opcoes[] = { "Acrescenta um aluno",
                             "Remove um aluno",
                             "Consulta dados de aluno",
                             "Altera dados de aluno",
                             "Exibe a turma na tela",
                             "Encerra o programa"
                           };
   int          indice,
                op,
                nOpcoes = sizeof(opcoes)/sizeof(opcoes[0]);
   char         umNome[MAX_NOME + 1],
                umaMatr[TAM_MATR + 1 ];

   IniciaLista(&lista);

      /* Armazena contedo do arquivo de dados e o armazena na lista */
   LeArquivo(NOME_ARQUIVO, &lista);

   while (1) {
      ApresentaMenu(opcoes, nOpcoes);

      op = LeOpcao("123456");

      if (op == '6') { /* Encerra o programa */
            /* Atualiza arquivo binrio */
         AtualizaArquivoBin(NOME_ARQ_BIN, &lista);

         break; /* Sada do lao */
      }

      switch (op) {
         case '1': /* Acrescenta um aluno */
               /* L dados do novo aluno, incluindo matrcula */
            LeDadosAluno(&aluno, 1);

            if (InsereEmOrdem(&lista, &aluno))
               printf("\n>>> Impossivel acrescentar o aluno\n");
            else
               printf("\n>>> Acrescimo bem sucedido\n");

            break;

         case '2': /* Remove um aluno */
               /* L a matrcula introduzida pelo usurio */
            LeMatricula(umaMatr, TAM_MATR + 1);

            indice = BuscaSequencialMatr(&lista, umaMatr);

            if (indice >= 0) {
               (void) RemoveElemento(&lista, indice);
               printf("\n>>> Remocao bem sucedida\n");
            } else {
               printf("\n>>> Matricula nao encontrada\n");
            }

            break;

         case '3': /* Consulta dados de um aluno */
            printf("\n>>> Deseja consultar por nome (N) ou por matricula (M)? ");
            op = LeOpcao("NnMm");

            if (op == 'N' || op == 'n') {
               LeNome(umNome, MAX_NOME + 1);
               indice = BuscaBinariaNome(&lista, umNome);
            } else {
               LeMatricula(umaMatr, TAM_MATR + 1);
               indice = BuscaSequencialMatr(&lista, umaMatr);
            }

            if (indice < 0) {
               printf("\n>>> Aluno nao foi encontrado\n");
            } else {
               aluno = ObtemElemento(&lista, indice);
               ExibeElemento(&aluno);
            }

            break;

         case '4': /* Altera dados de um aluno */
            printf( "\n>>> A implementacao dessa opcao fica "
                    "\n>>> como exercicio para o leitor\n" );
            break;

         case '5': /* Apresenta a turma na tela */
            ExibeLista(&lista);
            break;

         default: /* O programa no deve chegar at aqui */
            printf("\nEste programa contem um erro\n");
            return 1;
      }
   }

         /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 5

   Nome         Matr    N1      N2
   ====         ====    ==      ==

Ana Bolena      1012    7.8     8.0
Ana de Cleves   1022    4.5     6.0
Catarina Aragon 1014    5.5     6.5
Catarina Howard 1340    6.0     7.7
Catarina Parr   1440    4.0     6.0
Henrique VIII   1029    9.5     9.0
Joana Seymour   1017    7.7     8.7

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 2

Digite uma matricula com exatamente 4 digitos:
        > 1014

>>> Remocao bem sucedida

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 1

Digite o nome (max = 20 letras):
        > Mary Boleyn

Digite uma matricula com exatamente 4 digitos:
        > 0110

        >>> 1a. nota:
            > 7.2

        >>> 2a. nota:
            > 6.8

>>> Acrescimo bem sucedido

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 3

>>> Deseja consultar por nome (N) ou por matricula (M)? n

Digite o nome (max = 20 letras):
        > Catarina Parr

Catarina Parr   1440    4.0     6.0

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 3

>>> Deseja consultar por nome (N) ou por matricula (M)? m

Digite uma matricula com exatamente 4 digitos:
        > 1440

Catarina Parr   1440    4.0     6.0

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 5

   Nome         Matr    N1      N2
   ====         ====    ==      ==

Ana Bolena      1012    7.8     8.0
Ana de Cleves   1022    4.5     6.0
Catarina Howard 1340    6.0     7.7
Catarina Parr   1440    4.0     6.0
Henrique VIII   1029    9.5     9.0
Joana Seymour   1017    7.7     8.7
Mary Boleyn     0110    7.2     6.8

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 6

        >>> Obrigado por usar este programa.

**********************************************************/
