#include <stdio.h>      /* Funes de Entrada/Sada  */
#include <stdlib.h>     /* Funo exit()             */
#include "Pilha.h"      /* Declaraes do tipo pilha */
#include "ASSEGURA.h"   /* Macro ASSEGURA            */

/**************** Definies de funes ******************/

/****
 *
 * PilhaIdxCheia()
 *
 * Esta funo faz parte da implementao de pilha e no da
 * abstrao pilha. Por isso, ela deve ser definida como static.
 *
 ****/
static int PilhaIdxCheia(const tPilhaIdx *p)
{
   return p->topo >= MAX_ELEMENTOS - 1;
}

/****
 *
 * CriaPilhaIdx()
 *
 * Inicializa a pilha apontada por p.
 *
 ****/
void CriaPilhaIdx(tPilhaIdx *p)
{
   p->topo = -1;
}

/****
 *
 * PilhaIdxVazia()
 *
 * Retorna 1 se a pilha recebida como parmetro estiver vazia;
 * caso contrrio, retorna 0.
 *
 ****/
int PilhaIdxVazia(const tPilhaIdx *p)
{
   return p->topo == -1;
}

/****
 *
 * ElementoTopoIdx()
 *
 * Retorna o elemento do topo da pilha recebida como parmetro.
 * Se a pilha estiver vazia, causa o aborto do programa.
 *
 ****/
tItemPilha ElementoTopoIdx(const tPilhaIdx *p)
{
   ASSEGURA(!PilhaIdxVazia(p), "Erro: Pilha vazia.");

   return p->itens[p->topo];
}

/****
 *
 * EmpilhaIdx()
 *
 * EmpilhaIdx item na pilha apontada por p.
 * Se a pilha estiver cheia, causa o aborto do programa.
 *
 ****/
void EmpilhaIdx(const tItemPilha *item, tPilhaIdx *p)
{
   ASSEGURA(!PilhaIdxCheia(p), "Erro: Pilha cheia.");

   p->itens[++p->topo] = *item;
}

/****
 *
 * DesempilhaIdx()
 *
 * DesempilhaIdx e retorna o elemento do topo da pilha apontada por p.
 * Se a pilha estiver vazia, causa o aborto do programa.
 *
 ****/
tItemPilha DesempilhaIdx(tPilhaIdx *p)
{
   ASSEGURA(!PilhaIdxVazia(p), "Erro: Pilha vazia.");

   return p->itens[p->topo--];
}
