/****
 *
 * Ttulo: Casamento de Parnteses, Colchetes e Chaves
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Verifica se ocorre casamento de parnteses,
 *            colchetes e chaves usando pilha
 *
 * Entrada: Strings
 *
 * Sada: Informao sobre casamento de parnteses, colchetes
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada  */
#include <string.h>  /* Processamento de strings  */
#include "Pilha.h"   /* Interface do tipo pilha   */
                     /* implementada com array    */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo do string digitado pelo usurio */
#define MAX 20

/************************* Aluses ************************/

extern int LeString(char *ar, int nElementos);
extern char *EhAbertura(int c);
extern char *EhFechamento(int c);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array, incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a mais, sem incluir '\n'
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes no buffer */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * EhAbertura(): Verifica se um caractere  '(', '[' ou '{'
 *
 * Parmetros:
 *      c (entrada) - o caractere
 *
 * Retorno: 1, se o caractere for '(', '[' ou '{'; 0, em caso
 *          contrrio
 *
 ****/
char *EhAbertura(int c)
{
   static const char *const abre = "([{";

   return strchr(abre, c);
}

/****
 *
 * EhFechamento(): Verifica se um caractere  ')', ']' ou '}'
 *
 * Parmetros:
 *      c (entrada) - o caractere
 *
 * Retorno: 1, se o caractere for ')', ']' ou '}'; 0, em caso
 *          contrrio
 *
 ****/
char *EhFechamento(int c)
{
   static const char *const fecha = ")]}";

   return strchr(fecha, c);
}

/****
 *
 * Casam(): Verifica se dois caracteres casam;
 *          i.e., se um  '(' e o outro  ')' etc
 *
 * Parmetros:
 *      fecha (entrada) - um caractere de fechamento; i.e., ')', ']' ou '}'
 *      abre (entrada) - um caractere de abertura; i.e., '(', '[' ou '{'
 *
 ****/
int Casam(int fecha, int abre)
{
   if (fecha == ')' && abre == '(')
      return 1;
   else if (fecha == ']' && abre == '[')
      return 1;
   else if (fecha == '}' && abre == '{')
      return 1;

   return 0;
}

/****
 *
 * EscreveEspacos(): Escreve na tela o nmero de espaos
 *                   em branco determinado pelo parmetro
 *
 * Parmetros:
 *      n (entrada) - o nmero de espaos que sero escritos
 *
 * Retorno: Nada
 *
 ****/
void EscreveEspacos(int n)
{
      /* V. Exemplo de execuo para entender o que esta funo faz */
   for (int i = 1; i < n; ++i) {
      putchar(' ');
   }
}

/****
 *
 * MostraErro(): Apresenta na tela um erro de casamento
 *
 * Parmetros:
 *      posicao (entrada) - posio na qual o erro ocorres
 *
 * Retorno: Nada
 *
 ****/
void MostraErro(int posicao)
{
      /* V. Exemplo de execuo para entender o que esta funo faz */
   EscreveEspacos(posicao);
   printf("^\n");
   EscreveEspacos(posicao);
   printf("|\n");
   EscreveEspacos(posicao);
   printf("--- Nao casa\n");
}

/****
 *
 * main(): Verifica se ocorre casamento de parnteses,
 *         colchetes e chaves usando pilha
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro de casamento.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   tPilhaIdx pilha;
   char      string[MAX], *p = string;
   int       posErro = 1, /* Posio de um possvel erro */
             parentese;

   CriaPilhaIdx(&pilha);  /* Inicia pilha */

   printf( "\n>>> Digite uma sequencia de caracteres contendo"
           "\n>>> '()', '[]' ou '{}' para o programa verificar"
           "\n>>> se eles estao balanceados (max = %d caracteres):\n", MAX - 1 );
   LeString(p, MAX);

      /* Examina o string introduzido pelo usurio */
   for (; *p; ++p) {
         /* Aqui s interessam caracteres de abertura */
         /* (e.g., '(') ou fechamento (e.g., ')')     */
      if (EhAbertura(*p)) {
            /* Caractere de abertura  empilhado */
         EmpilhaIdx((tItemPilha) *p, &pilha);
      } else if (EhFechamento(*p)) {
            /* Caractere de fechamento causa desempilhamento e comparao */
         if (!PilhaIdxVazia(pilha)) {
            parentese = DesempilhaIdx(&pilha);

               /* Verifica se ocorre casamento entre o caractere */
               /* corrente e o caractere desempilhado            */
            if (!Casam(*p, parentese)) {
                  /* No houve casamento Mostra onde ocorreu o erro */
               MostraErro(posErro);
               return 1;
            }
         } else {
               /* A pilha est vazia; portanto, no h casamento */
            MostraErro(posErro);
            return 1;
         }
      }

         /* Ainda no foi encontrado erro. Ento, passa adiante. */
      ++posErro;
   }

   while (!PilhaIdxVazia(pilha))
      printf("\nNao foi encontrado casamento para %c\n", DesempilhaIdx(&pilha));

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Digite uma sequencia de caracteres contendo
>>> '()', '[]' ou '{}' para o programa verificar
>>> se eles estao balanceados (max = 19 caracteres):
a ( b [ (c]))
          ^
          |
          --- Nao casa

**********************************************************/
