/****
 *
 * Ttulo: Testando Palndromos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Verifica se strings so palndromos
 *
 * Entrada: Uma cadeia de caracteres introduzida pelo usurio
 *
 * Sada: A cadeia de caracteres invertida
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada */
#include "Pilha.h"   /* Interface do tipo pilha  */
                     /* implementada com array   */
#include <string.h>  /* Processamento de steings */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de um nome de soldado de Josephus */
#define TAM_ARRAY 31

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com exceo do caractere '\n'
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de caracteres lidos */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que sero
 *                            armazenados no array, incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a mais, sem incluir '\n'
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de caracteres excedentes no buffer */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * main(): Verifica se os caracteres digitados pelo usurio
 *         constituem um palndromo usando pilha
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tPilhaIdx pilha;
   char      string[TAM_ARRAY];
   int       tamString, metade, i;

   CriaPilhaIdx(&pilha);  /* Inicia pilha */

   printf( "\n>>> Digite uma sequencia de caracteres para"
           "\n>>> o programa verificar se e' palindromo"
           "\n>>> (max = %d caracteres): ", TAM_ARRAY - 1 );
   LeString(string, TAM_ARRAY);

   tamString = strlen(string);
   metade = tamString/2;

   for (i = 0; i < metade; ++i)
      EmpilhaIdx(string[i], &pilha);

   if (tamString%2)
      i = metade + 1; /* Tamanho do string  mpar */
   else
      i = metade; /* Tamanho do string  par */

   while (!PilhaIdxVazia(pilha)) {
      if ( string[i] != DesempilhaIdx(&pilha) ) {
         printf("\n>>> \"%s\" NAO e' palindromo\n", string);
         return 0;
      }
      ++i;
   }

   printf("\n>>> \"%s\" e' um palindromo\n", string);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Digite uma sequencia de caracteres para
>>> o programa verificar se e' palindromo
>>> (max = 30 caracteres): omississimo

>>> "omississimo" e' um palindromos

**********************************************************/
