#include <stdio.h>     /* Funes de Entrada/Sada  */
#include "Pilha.h"     /* Declaraes do tipo pilha */
#include "ASSEGURA.h"  /* Macro ASSEGURA            */

/**************** Definies de funes ******************/

/****
 *
 * PilhaCheia()
 *
 * Esta funo faz parte da implementao de pilha e no da
 * abstrao pilha. Por isso, ela deve ser definida como static.
 *
 ****/
static int PilhaCheia(const tPilha *p)
{
   return p->topo >= MAX_ELEMENTOS - 1;
}

/****
 *
 * CriaPilha(): Inicializa a pilha apontada por p.
 *
 ****/
void CriaPilha(tPilha *p)
{
   p->topo = -1;
}

/****
 *
 * PilhaVazia()
 *
 * Retorna 1 se a pilha recebida como parmetro estiver vazia;
 * caso contrrio, retorna 0.
 *
 ****/
int PilhaVazia(const tPilha *p)
{
   return p->topo == -1;
}

/****
 *
 * ElementoTopo()
 *
 * Retorna o elemento do topo da pilha recebida como parmetro.
 * Se a pilha estiver vazia, causa o aborto do programa.
 *
 ****/
tItemFila *ElementoTopo(const tPilha *p)
{
   ASSEGURA(!PilhaVazia(p), "Erro: Pilha vazia");

   return &p->itens[p->topo];
}

/****
 *
 * Empilha()
 *
 * Empilha item na pilha apontada por p.
 * Se a pilha estiver cheia, causa o aborto do programa.
 *
 ****/
void Empilha(const tItemFila *item, tPilha *p)
{
   ASSEGURA(!PilhaCheia(p), "Erro: Pilha cheia");

   p->itens[++p->topo] = *item;
}

/****
 *
 * Desempilha()
 *
 * Desempilha e retorna o elemento do topo da pilha apontada por p.
 * Se a pilha estiver vazia, causa o aborto do programa.
 *
 ****/
tItemFila Desempilha(tPilha  *p)
{
   ASSEGURA(!PilhaVazia(p), "Erro: Pilha vazia");

   return p->itens[p->topo--];
}
