/****
 *
 * Ttulo: Fazendo, Desfazendo e Refazendo
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Gerencia uma lista de compras permitindo
 *            desfazer e refazer operaes
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada */
#include <string.h>  /* Processamento de strings */
#include "Pilha.h"   /* Interface do tipo pilha  */
                     /* implementada com array   */
#include "Lista.h"   /* Interface do tipo lista  */
#include "Leitura.h" /* Funes de leitura       */
#include "ASSEGURA.h"  /* Macro ASSEGURA         */

/***************** Constantes Simblicas ******************/

#define MAX_OPCOES 20 /* Nmero mximo de caracteres que */
                      /* representam opes de menu      */

/***************** Definies de Funes ******************/

/****
 *
 * ApresentaMenu(): Apresenta o menu de opes do programa
 *                  de acordo com a lista e as pilhas
 *                  recebidas como parmetro
 *
 * Parmetros:
 *     *lista (entrada) - uma lista
 *     *desfaz (entrada) - pilha contendo informaes sobre
 *                         como desfazer a ltima operao
 *     *lista (entrada) - pilha contendo informaes sobre
 *                         como refazer a ltima operao
 *
 * Retorno: Endereo de um string cujos caracteres
 *          representam as opes que o usurio poder
 *          escolher
 *
 ****/
char *ApresentaMenu( const tLista *lista, const tPilha *desfaz,
                     const tPilha *refaz )
{
   tDesfazRefaz *desfazer;
   static char   opcoes[MAX_OPCOES + 1];

      /* O array opcoes[] armazenar um string        */
      /* cujos caracteres representam as opes       */
      /* que o usurio poder escolher. Aqui, ele     */
      /* recebe as opes de encerramento que so     */
      /* sempre disponveis. Isso no pode ser feito  */
      /* como iniciao pois o array tem durao fixa */
   strcpy(opcoes, "Ee");

      /* Apresenta o cabealho do menu */
   printf( "\n\n\t*** Opcoes ***" );

      /**********************************/
      /*                                */
      /* Apresenta o menu de acordo com */
      /* o estado da lista e das pilhas */
      /*                                */
      /**********************************/

   if (!EstaCheia(lista)) {
         /* Se a lista no est cheia, o usurio */
         /* pode acrescentar mais itens          */
      printf( "\n\t[A]crescenta" );
      strcat(opcoes, "Aa");

      if (Comprimento(lista) > 1) {
            /* Se a lista tiver mais de um elemento, */
            /* o usurio pode inserir um elemento    */
         printf( "\n\t[I]nsere" );
         strcat(opcoes, "Ii");
      }
   }

      /* Se a lista no estiver vazia, o usurio */
      /* pode remover e modificar um elemento    */
   if (!EstaVazia(lista)) {
      printf( "\n\t[R]emove" );
      printf( "\n\t[M]odifica" );

      strcat(opcoes, "RrMm");
   }

      /* Se a pilha de desfaz no estiver vazia, */
      /* o usurio pode desfazer a ltima ao   */
   if (!PilhaVazia(desfaz)) {
      printf( "\n\t[D]esfaz " );
      strcat(opcoes, "Dd");

         /* Obtm o elemento do topo da pilha 'desfaz' */
      desfazer = ElementoTopo(desfaz);

         /* Especifica a ao que pode ser desfeita */
      switch (desfazer->acao) {
         case ACRESCENTA:
            printf("acrescenta item");
            break;
         case INSERE:
            printf("insere item");
            break;
         case REMOVE:
            printf("remove item");
            break;
         case MODIFICA:
            printf("modifica item");
            break;
         default: /* O programa no deve chegar at aqui */
            ASSEGURA(0, "Erro que nao deveria ocorrer");
            break;
      }
   }

      /* Se a pilha de refaz no estiver vazia, */
      /* o usurio pode refazer a ltima ao   */
   if (!PilhaVazia(refaz)) {
      printf( "\n\tRe[F]az " );
      strcat(opcoes, "Ff");

         /* Obtm o elemento do topo da pilha 'refaz' */
      desfazer = ElementoTopo(refaz);

         /* Especifica a ao que pode ser desfeita */
      switch (desfazer->acao) {
         case ACRESCENTA:
            printf("acrescenta item");
            break;
         case INSERE:
            printf("insere item");
            break;
         case REMOVE:
            printf("remove item");
            break;
         case MODIFICA:
            printf("modifica item");
            break;
         default: /* O programa no deve chegar at aqui */
            ASSEGURA(0, "Erro que nao deveria ocorrer");
            break;
      }
   }

      /* Apresenta a ltima opo e o prompt */
   printf( "\n\t[E]ncerra o programa\n"
           "\nEscolha sua opcao >>> " );

   return opcoes;
}

/****
 *
 * Desfaz(): Desfaz uma operao efetuada sobre uma lista
 *
 * Parmetros:
 *      lista (sada) - a lista sobre a qual a ao ser
 *                      desfeita
 *      desfaz (entrada e sada) - pilha contendo informaes
 *                                 sobre como desfazer
 *      refaz (sada) - pilha contendo informaes sobre
 *                      como refazer
 *
 * Retorno: Nada
 *
 ****/
void Desfaz(tLista *lista, tPilha *desfaz, tPilha *refaz)
{
   tDesfazRefaz desfazer;

      /* Se a pilha 'desfaz' estiver vazia, */
      /* esta funo no deve ser chamada   */
   ASSEGURA(!PilhaVazia(desfaz), "Nao ha o que desfazer");

      /* Desempilha as informaes sobre */
      /* a operao a ser desfeita       */
   desfazer = Desempilha(desfaz);

      /* Empilha as informaes sobre a operao */
      /* a ser desfeita na pilha 'refaz'         */
   Empilha(&desfazer, refaz);

      /* Desfaz a ltima operao */
   switch (desfazer.acao) {
      case ACRESCENTA: /* Remove o elemento que havia   */
      case INSERE:     /* sido acrescentado ou inserido */
         RemoveElemento(lista, desfazer.posicao);
         break;
      case REMOVE:
            /* Insere o elemento que havia sido removido */
         InsereElemento( lista, desfazer.item,
                         desfazer.posicao );
         break;
      case MODIFICA:
            /* Retorna o valor anterior do item */
         AlteraValor(lista, desfazer.item, desfazer.posicao);
         break;
      default: /* O programa no deve chegar at aqui */
         ASSEGURA(0, "Erro que nao deveria ocorrer");
         break;
   }
}

/****
 *
 * Refaz(): Refaz uma operao efetuada sobre uma lista
 *
 * Parmetros:
 *      lista (sada) - a lista sobre a qual a ao ser
 *                      refeita
 *      refaz (entrada e sada) - pilha contendo informaes
 *                                sobre como refazer
 *      desfaz (sada) - pilha contendo informaes sobre
 *                       como desfazer
 *
 * Retorno: Nada
 *
 ****/
void Refaz(tLista *lista, tPilha *refaz, tPilha *desfaz)
{
   tDesfazRefaz refazer;

      /* Se a pilha 'refaz' estiver vazia, */
      /* esta funo no deve ser chamada  */
   ASSEGURA(!PilhaVazia(refaz), "Nao ha o que refazer");

      /* Desempilha as informaes sobre */
      /* a operao a ser refeita        */
   refazer = Desempilha(refaz);

      /* Empilha as informaes sobre a operao */
      /* a ser refeita na pilha 'desfaz'         */
   Empilha(&refazer, desfaz);

      /* Refaz a ltima operao desfeita */
   switch (refazer.acao) {
      case ACRESCENTA:
         AcrescentaElemento(lista, refazer.item);
         break;
      case INSERE:
         InsereElemento(lista, refazer.item, refazer.posicao);
         break;
      case REMOVE:
         (void) RemoveElemento(lista, refazer.posicao);
         break;
      case MODIFICA:
         AlteraValor(lista, refazer.item, refazer.posicao);
         break;
      default: /* O programa no deve chegar at aqui */
         ASSEGURA(0, "Erro que nao deveria ocorrer");
      break;
   }
}

/****
 *
 * EsvaziaPilha(): Esvazia uma pilha
 *
 * Parmetros:
 *      p (sada) - a pilha a ser esvaziada
 *
 * Retorno: Nada
 *
 ****/
void EsvaziaPilha(tPilha *p)
{
   while (!PilhaVazia(p)) {
      (void) Desempilha(p);
   }
}

/****
 *
 * main(): Gerencia uma lista de compras permitindo
 *         desfazer e refazer operaes
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int          opcao, /* Uma opo escolhida pelo usurio */
                pos, /* Uma posio na lista */
                nElementos; /* Nmero de elementos da lista */
   tLista       lista; /* A lista de compras */
   tItem        umItem; /* Um item da lista de compras */
   char        *removido; /* Um item removido da lista */
   tPilha       pDesfaz, /* Pilha que armazena informaes */
                         /* sobre como desfazer            */
                pRefaz; /* Pilha que armazena informaes */
                        /* sobre como refazer             */
   tDesfazRefaz desfazRefaz; /* Um item da pilha que     */
                             /* contm informaes sobre */
                             /* como desfazer ou refazer */
   char        *opcoes; /* Aponta para o string contendo as   */
                        /* opes que o usurio pode escolher */

   IniciaLista(&lista); /* Inicia a lista de compras  */
   CriaPilha(&pDesfaz); /* Inicia a pilha de desfazimento */
   CriaPilha(&pRefaz);  /* Inicia a pilha de refazimento  */

      /* Apresenta o programa ao usurio */
      /* e explica seu funcionamento     */
   printf( "\n\t>>> Este programa gerencia uma lista de"
           "\n\t>>> compras e permite desfazer e refazer\n" );

   while (1) {
         /* Apresenta a lista */
      ApresentaLista(&lista);

         /* Alerta o usurio quando a lista est cheia */
      if (EstaCheia(&lista)) {
         printf("\n*** A lista esta' cheia ***\n");
      }

         /* Apresenta o menu e obtm as opes disponveis */
      opcoes = ApresentaMenu(&lista, &pDesfaz, &pRefaz);

         /* L a opo do usurio          */
         /* (o prompt j foi apresentado ) */
      opcao = LeOpcao(opcoes);

         /*  melhor tratar a opo de encerramento aqui */
      if (opcao == 'E' || opcao == 'e') {
         break; /* Usurio quer encerrar o programa */
      }

         /* Processa as demais opes */
      switch (opcao) {
         case 'A': /* Acrescenta */
         case 'a':
            ASSEGURA(!EstaCheia(&lista), "Lista cheia");

               /* Acrescenta um novo intem na lista */
            printf("\nDigite o nome do item >>> ");
            LeString( (char *) umItem, MAX_ITEM );
            AcrescentaElemento(&lista, umItem);

               /* Preenche a estrutura com informaes */
               /* sobre desfazimento                   */
            desfazRefaz.acao = ACRESCENTA;
            strcpy((char *) desfazRefaz.item, (char *) umItem);
            desfazRefaz.posicao = Comprimento(&lista) - 1;

               /* Empilha as informaes sobre desfazimento */
            Empilha(&desfazRefaz, &pDesfaz);

               /* Quando uma ao  executada deixa de  */
               /* haver opo de refazimento. Portanto, */
               /* esvazia-se a pilha de refazimento.    */
            EsvaziaPilha(&pRefaz);
            break;
         case 'I': /* Insere */
         case 'i':
            ASSEGURA(!EstaCheia(&lista), "Lista cheia");

               /*                             */
               /* Insere um elemento na lista */
               /*                             */

            nElementos = Comprimento(&lista);

            if (nElementos > 1) {
               printf( "\nInforme a posicao entre 1 "
                       "e %d >>> ", nElementos );
               pos = LeNaturalPositivo();
            } else {
               pos = 1;
            }

            if (pos > nElementos) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' insercao.\n",
                       nElementos + 1 );
            } else {
                  /* A correo a seguir  necessria porque */
                  /* usurio normal conta a partir de 1, mas */
                  /* programador de C conta a partir de zero */
               --pos;

               printf("\nDigite o nome do item >>> ");
               LeString( (char *) umItem, MAX_ITEM );

               InsereElemento(&lista, umItem, pos);

                  /* Preenche a estrutura com informaes */
                  /* sobre desfazimento                   */
               desfazRefaz.acao = INSERE;
               strcpy((char *) desfazRefaz.item, (char *) umItem);
               desfazRefaz.posicao = pos;

                  /* Empilha as informaes sobre desfazimento */
               Empilha(&desfazRefaz, &pDesfaz);

                  /* Quando uma ao  executada deixa de  */
                  /* haver opo de refazimento. Portanto, */
                  /* esvazia-se a pilha de refazimento.    */
               EsvaziaPilha(&pRefaz);
            }
            break;
         case 'R': /* Remove */
         case 'r':
            ASSEGURA(!EstaVazia(&lista), "Lista vazia");

               /*                             */
               /* Remove um elemento da lista */
               /*                             */

            nElementos = Comprimento(&lista);
            printf( "\nInforme a posicao entre 1 e %d >>> ",
                    nElementos );
            pos = LeNaturalPositivo();

            if (pos > nElementos) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' remocao.\n",
                       nElementos + 1 );
            } else {
                  /* A correo a seguir  necessria     */
                  /* porque usurio normal conta a partir */
                  /* de 1, mas programador de C conta     */
                  /* a partir de zero                     */
               --pos;

               removido = RemoveElemento(&lista, pos);

               printf( "\n>>> \"%s\" foi removido da "
                       "lista\n", removido );

                  /* Preenche a estrutura com informaes */
                  /* sobre desfazimento                   */
               desfazRefaz.acao = REMOVE;
               strcpy( (char *) desfazRefaz.item,
                       (char *) removido );
               desfazRefaz.posicao = pos;

                  /* Empilha as informaes sobre desfazimento */
               Empilha(&desfazRefaz, &pDesfaz);

                  /* Esvazia a pilha de refazimento */
               EsvaziaPilha(&pRefaz);
            }
            break;
         case 'M': /* Modifica */
         case 'm':
            ASSEGURA(!EstaVazia(&lista), "Lista vazia");

               /*                               */
               /* Modifica um elemento da lista */
               /*                               */

            nElementos = Comprimento(&lista);
            printf( "\nInforme a posicao entre 1 e %d >>> ",
                    nElementos );
            pos = LeNaturalPositivo();

            if (pos > nElementos) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' modificacao.\n",
                       nElementos + 1 );
            } else {
                  /* A correo a seguir  necessria     */
                  /* porque usurio normal conta a partir */
                  /* de 1, mas programador de C conta     */
                  /* a partir de zero                     */
               --pos;

               printf( "\nDigite o novo nome do item: "
                       "%s >>> ",
                       ConsultaElemento(&lista, pos) );
               LeString( (char *) umItem, MAX_ITEM );

               AlteraValor(&lista, umItem, pos);

                  /* Preenche a estrutura com informaes */
                  /* sobre desfazimento                   */
               desfazRefaz.acao = MODIFICA;
               strcpy( (char *) desfazRefaz.item,
                       (char *) umItem );
               desfazRefaz.posicao = pos;

                  /* Empilha as informaes sobre desfazimento */
               Empilha(&desfazRefaz, &pDesfaz);

                  /* Esvazia a pilha de refazimento */
               EsvaziaPilha(&pRefaz);
            }
            break;
         case 'D': /* Desfaz */
         case 'd':
            ASSEGURA(!PilhaVazia(&pDesfaz), "Pilha vazia");

            Desfaz(&lista, &pDesfaz, &pRefaz);
            break;
         case 'F': /* Refaz */
         case 'f':
            ASSEGURA(!PilhaVazia(&pRefaz), "Pilha vazia");

            Refaz(&lista, &pRefaz, &pDesfaz);
            break;
         default:
            printf("\a\n\t>>> Opcao invalida");
            break;
      }
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa. "
           "Boas compras!\n" );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa gerencia uma lista de
        >>> compras  e permite desfazer e refazer

        *** Lista vazia ***


        *** Opcoes ***
        [A]crescenta
        [E]ncerra o programa


Escolha sua opcao >>> a

Digite o nome do item >>> Arroz

        *** Lista ***

            1. Arroz

        *** Opcoes ***
        [A]crescenta
        [R]emove
        [M]odifica
        [D]esfaz
        [E]ncerra o programa

Escolha sua opcao >>> a

Digite o nome do item >>> Acucar

        *** Lista ***

            1. Arroz
            2. Acucar

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [D]esfaz
        [E]ncerra o programa

Escolha sua opcao >>> i

Informe a posicao entre 1 e 2 >>> 2

Digite o nome do item >>> Feijao

        *** Lista ***

            1. Arroz
            2. Feijao
            3. Acucar

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [D]esfaz
        [E]ncerra o programa

Escolha sua opcao >>> d

        *** Lista ***

            1. Arroz
            2. Acucar

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [D]esfaz
        Re[F]az
        [E]ncerra o programa

Escolha sua opcao >>> f

        *** Lista ***

            1. Arroz
            2. Feijao
            3. Acucar

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [D]esfaz
        [E]ncerra o programa

Escolha sua opcao >>> e

        >>> Obrigado por usar este programa. Boas compras!

**********************************************************/
