/****
 *
 * Arquivo: Fila1.c
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Implementao de fila usando array esttico linear
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>      /* Funes de Entrada/Sada  */
#include <stdlib.h>     /* Funo exit()             */
#include "Fila.h"       /* Declaraes do tipo fila  */
#include "ASSEGURA.h"   /* Macro ASSEGURA            */

/***************** Definies de Funes ******************/

/****
 *
 * FilaIdxCheia(): Verifica se uma fila est repleta e, se for
 *              o caso, promove o deslocamento de elementos
 *              de modo a criar espao para insero.
 *
 * Parmetros:
 *      f (entrada/sada) - a fila
 *
 * Retorno: 1, se a fila estiver repleta
 *          0, em caso contrrio
 *
 * Observao: Esta funo faz parte da implementao de fila e
 *             no da abstrao fila. Por isso, ela deve ser
 *             definida como static.
 *
 ****/
static int FilaIdxCheia(tFilaIdx  *f)
{
   int i, deslocamento;

   if (f->fundo < MAX_ELEMENTOS - 1)
      return 0; /* H espao no final da fila */

   if (f->frente == -1 && f->fundo == MAX_ELEMENTOS - 1)
      return 1; /* No h mais espao na fila */

      /* Neste ponto, sabe-se que h espao, mas  necessrio    */
      /* mover os elementos da fila para o incio do array.      */
      /* O deslocamento a ser feito em cada elemento  dado por: */
   deslocamento = MAX_ELEMENTOS - (f->fundo - f->frente);

      /* O lao for a seguir desloca cada elemento */
      /* da fila para o incio do array            */
   for (i = f->frente + 1; i <= f->fundo; ++i)
      f->itens[i - deslocamento] = f->itens[i];

      /* A frente e o fundo da fila tambm precisam ser deslocados */
   f->frente -= deslocamento;
   f->fundo -= deslocamento;

      /* A fila foi deslocada e agora     */
      /* podem-se acrescentar novos itens */
   return 0;
}

/****
 *
 * CriaFilaIdx(): Inicia a fila recebida como parmetro.
 *
 * Parmetros:
 *      f (entrada/sada) - a fila
 *
 * Retorno: Nada
 *
 ****/
void  CriaFilaIdx(tFilaIdx  *f)
{
   f->frente = f->fundo = -1;
}

/****
 *
 * FilaIdxVazia(): Verifica se a fila recebida como parmetro est vazia.
 *
 * Parmetros:
 *      f (entrada) - a fila
 *
 * Retorno: 1, se a fila estiver vazia
 *          0, em caso contrrio
 *
 ****/
int  FilaIdxVazia(tFilaIdx  f)
{
   return f.frente == f.fundo;
}

/****
 *
 * ElementoFrenteIdx(): Retorna o elemento que se encontra
 *                   na frente de uma fila.
 *
 * Parmetros:
 *      f (entrada) - a fila
 *
 * Retorno: O elemento que se encontra na frente da fila.
 *
 * Observao: Se a fila estiver vazia, causa o aborto do programa.
 *
 ****/
tItemFila  ElementoFrenteIdx(tFilaIdx  f)
{
   ASSEGURA(!FilaIdxVazia(f), "Erro: Fila vazia");

   return f.itens[f.frente + 1];
}

/****
 *
 * EnfileiraIdx(): EnfileiraIdx um item numa fila.
 *
 * Parmetros:
 *      item (entrada) - o item que ser acrescentado
 *      f (entrada/sada) - a fila na qual ser feita o acrscimo
 *
 * Retorno: Nada.
 *
 * Observao: Se a fila estiver cheia, causa o aborto do programa.
 *
 ****/
void EnfileiraIdx(tItemFila item, tFilaIdx  *f)
{
   ASSEGURA(!FilaIdxCheia(f), "Erro: Fila cheia.");

   ++f->fundo;

   f->itens[f->fundo] = item;
}

/****
 *
 * DesenfileiraIdx(): DesenfileiraIdx o elemento que se encontra na frente de uma fila
 *
 * Parmetros:
 *      f (entrada/sada) - a fila na qual ser feita a retirada
 *
 * Retorno: O elemento que se encontra na frente da fila
 *
 * Observao: Se a fila estiver vazia, causa o aborto do programa.
 *
 ****/
tItemFila DesenfileiraIdx(tFilaIdx  *f)
{
   ASSEGURA(!FilaIdxVazia(*f), "Erro: Fila vazia.");

   ++f->frente;

   return f->itens[f->frente];
}

/****
 *
 * NItensFilaIdx(): Calcula o nmero de elementos de uma fila
 *
 * Parmetros:
 *     f (entrada/sada) - a fila cujos elementos sero contados
 *
 * Retorno: O nmero de elementos da fila
 *
 ****/
int NItensFilaIdx(tFilaIdx f)
{
   return f.fundo - f.frente;
}

