/****
 *
 * Ttulo: Simulao de Fila de Banco
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2016
 * ltima modificao: 16/04/2016
 *
 * Descrio: Simula o atendimento de clientes numa fila de
 *            banco usando uma fila de elementos do tipo int
 *            em que cada item na fila representa o instante
 *            em que o respectivo cliente  adicionado  fila
 *
 * Entrada: O tempo (fictcio) de durao da simulao
 *
 * Sada: O nmero de clientes atendidos, o maior tempo de
 *        espera e o tempo mdio de espera
 *
 ****/

#include <stdio.h>   /* printf()         */
#include <stdlib.h>  /* srand() e rand() */
#include <time.h>    /* time()           */
#include "leitura.h" /* LeInteiro()      */
#include "Fila.h"    /* Operaes sobre filas  */

   /* Nmeros mnimo e mximo de clientes */
   /* que entram na fila simultaneamente  */
#define MIN_CLIENTES 0
#define MAX_CLIENTES 3

/****
 *
 * ExibeFila(): Apresenta um diagrama na tela mostrando o estado de uma fila
 *
 * Parmetros:
 *      f (entrada) - a fila
 *
 * Retorno: Nada
 *
 ****/
void ExibeFila(tFilaIdx f)
{
   int        nElementos = NItensFilaIdx(f);
   static int primeiraChamada = 1;

   if (primeiraChamada) {
      primeiraChamada = 0;
      printf("\n\t  >>> Fila <<<\n\n");
   }

   if (!nElementos) {
      printf("\tNinguem na fila\n");
      return;
   }

   putchar('\t');

   for (int i = 0; i < nElementos; ++i)
      putchar('*');

   putchar('\n');
}

/****
 *
 * main(): Simula uma fila de banco
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main (void)
{
   tFilaIdx fila; /* Fila de clientes contendo o instante */
               /* em que cada cliente chegou na fila   */
   int duracao,         /* durao da simulao (em minutos) */
       minuto,          /* Minuto corrente */
       instanteChegada, /* Instante em que o cliente */
                        /* entrou na fila            */
       tempoEspera, /* Tempo de espera de um cliente na fila */
       totalAtendidos = 0, /* Total clientes atendidos */
       esperaTotal = 0, /* Tempo de espera total dos clientes */
       maiorEspera = 0,   /* Tempo de espera mais longo */
       clientesChegando = 0, /* Nmero de clientes que   */
                             /* acabam de chegar na fila */
       naoAtendidos = 0; /* Nmero de clientes que no */
                         /* foram atendidos porque a   */
                         /* simulao acabou antes     */

      /* Inicia o gerador de nmeros aleatrios */
   srand(time(NULL));

      /* L o tempo de durao da simulao */
   printf("\n>>> Digite a duracao da simulacao (inteiro): ");
   duracao = LeInteiro();

   if (duracao <= 0) {
      printf("\nValor de duracao invalido. Bye.\n");
      return 1;
   }

      /* Inicia a fila */
   CriaFilaIdx(&fila);

      /* Executa a simulao durante o tempo estipulado */
   for (minuto = 1; minuto <= duracao; ++minuto) {
         /* Exibe o estado da fila */
      ExibeFila(fila);

         /* Se a fila no estiver vazia, atende mais */
         /* um cliente; i.e., remove-o da fila       */
      if (!FilaIdxVazia(fila)) {
            /* Cada cliente  representado por seu instante de chegada */
         instanteChegada = DesenfileiraIdx(&fila);

         totalAtendidos++; /* Mais um cliente foi atendido */

            /* Calcula o tempo de espera deste cliente */
         tempoEspera = minuto - instanteChegada;

         /* EnfileiraIdx o tempo de espera deste */
         /* cliente ao tempo total de espera   */
         esperaTotal = esperaTotal + tempoEspera;

         /* Atualiza o valor do maior tempo de espera */
         if (maiorEspera < tempoEspera)
            maiorEspera = tempoEspera;
      }

         /* Sorteia o nmero de clientes  */
         /* que acabam de chegar ao banco */
      clientesChegando = rand()%(MAX_CLIENTES-MIN_CLIENTES+1) + MIN_CLIENTES;

         /* Coloca os clientes na fila */
      for ( int j = 0; j < clientesChegando ; j++ )
         EnfileiraIdx(minuto, &fila);
   }

      /* Exibe o estado final da fila */
   ExibeFila(fila);

      /* Apresenta o resultado da simulao */
   if (!totalAtendidos) {
      printf("\n\t>>> Nenhum cliente foi atendido\n");
   } else {
      printf( "\n\t>>> Numero de clientes atendidos: %d",
              totalAtendidos );
      printf( "\n\t>>> Maior tempo de espera: %d min",
              maiorEspera );
      printf( "\n\t>>> Tempo medio de espera: %3.2f min",
              (double) esperaTotal/totalAtendidos );
   }

      /* Conta o nmero de clientes que no foram atendidos */
   while (!FilaIdxVazia(fila)) {
      (void) DesenfileiraIdx(&fila);
      ++naoAtendidos;
   }

         /* Apresenta o nmero de clientes */
         /* que no foram atendidos        */
if (naoAtendidos) {
      printf( "\n\t>>> Numero de clientes nao atendidos: %d\n",
              naoAtendidos );
   } else {
      printf("\n\t>>> Todos os clientes foram atendidos\n");
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Digite a duracao da simulacao (inteiro): 15

          >>> Fila <<<

        Ninguem na fila
        ***
        *****
        ****
        *****
        ******
        ******
        *****
        ******
        *******
        *******
        ********
        *******
        ******
        ********
        **********

        >>> Numero de clientes atendidos: 14
        >>> Maior tempo de espera: 6 min
        >>> Tempo medio de espera: 4.36 min
        >>> Numero de clientes nao atendidos: 10

**********************************************************/

