/****
 *
 * Ttulo: Pilha Dinmica
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <string.h>   /* Funes strXXX()        */
#include <stdlib.h>   /* malloc()                */
#include "ASSEGURA.h" /* Macro ASSEGURA()        */
#include "pilha.h"    /* Interface do tipo pilha */

/***************** Constantes Simblicas ******************/

   /* Nmero de elementos que sero acrescentados ao */
   /* array a cada alocao ou realocao de memria */
#define ACRESCIMO 10

/****************** Complemento de Tipo *******************/

struct rotPilhaIdxD {
   tItemPilha *itens; /* Ponteiro para o array que conter os itens */
   int        topo; /* Indicar o topo de uma pilha */
   int        tamanhoArray; /* Nmero de elementos do array */
};

/***************** Definies de Funes ******************/

/****
 *
 * CriaPilhaIdxD(): Cria uma pilha
 *
 * Parmetros: No h
 *
 * Retorno: Nada
 *
 ****/
void CriaPilhaIdxD(tPilhaIdxD *umaPilha)
{
      /* Aloca espao para a estrutura que armazena a pilha */
   *umaPilha = malloc(sizeof(struct rotPilhaIdxD));

      /* Garante que a alocao realmente ocorreu */
   ASSEGURA(*umaPilha, "Nao foi possivel alocar pilha");

   (*umaPilha)->topo = -1; /* Inicia o topo da pilha */

      /* Aloca um array inicial */
   (*umaPilha)->itens = calloc(ACRESCIMO, sizeof(tItemPilha));

      /* Garante que o array foi realmente alocado */
   ASSEGURA( (*umaPilha)->itens, "Nao foi possivel alocar "
             "array para conter elementos da pilha" );

      /* Atualiza a varivel que armazena o tamanho do array */
   (*umaPilha)->tamanhoArray = ACRESCIMO;
}

/****
 *
 * DestroiPilhaIdxD(): Libera o espao ocupado pela pilha
 *
 * Parmetros: pilha (entrada e sada) - a pilha
 *
 * Retorno: Nada
 *
 ****/
void DestroiPilhaIdxD(tPilhaIdxD *pilha)
{
      /* Se a pilha no estiver vazia, libera o espao */
      /* ocupado pelo array que contm os elementos    */
   if ((*pilha)->topo) {
      free((*pilha)->itens);
   }

      /* Libera a estrutura que representa a pilha */
   free((*pilha));
}

/****
 *
 * EstaVaziaPilhaIdxD(): Verfica se uma pilha est vazia
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1 se a pilha recebida como parmetro estiver vazia;
 *          0, em caso contrrio
 *
 ****/
int EstaVaziaPilhaIdxD(const tPilhaIdxD *p)
{
   return (*p)->topo == -1;
}

/****
 *
 * ElementoTopoIdxD(): Obtm o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: O elemento do topo da pilha recebida como parmetro
 *
 * Observao: Se a pilha estiver vazia, causa aborto de programa
 *
 ****/
tItemPilha ElementoTopoIdxD(const tPilhaIdxD *p)
{
   ASSEGURA(!EstaVaziaPilhaIdxD(p), "Erro: Pilha vazia.");

   return (*p)->itens[(*p)->topo];
}

/****
 *
 * EmpilhaIdxD(): Empilha um item numa pilha
 *
 * Parmetros:
 *      p (entrada e sada) - ponteiro para a pilha na qual
 *                             ocorrer o empilhamento
 *      item (entrada) - ponteiro para o contedo
 *                       que ser empilhado
 *
 * Retorno: Nada
 *
 * Observao: Se a pilha estiver cheia, causa aborto
 *
 ****/
void EmpilhaIdxD(tPilhaIdxD *p, const tItemPilha *item)
{
   tItemPilha *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */

      /* Se o array estiver repleto, tenta redimension-lo */
   if ((*p)->topo >= (*p)->tamanhoArray) {
         /* Calcula o novo tamanho do array */
      (*p)->tamanhoArray += ACRESCIMO;

         /* Tenta redimensionar o array */
      novoArray = realloc((*p)->itens,
                          (*p)->tamanhoArray*sizeof(tItemPilha));

         /* Checa se houve realocao de memria */
      ASSEGURA(novoArray, "Nao houve redimensionamento");

         /* O ponteiro que apontava para o incio */
         /* do array pode no ser mais vlido.    */
         /* Portanto,  preciso atualiz-lo.      */
      (*p)->itens = novoArray;
   }

   (*p)->itens[++(*p)->topo] = *item;  /* Efetua o empilhamento */
}

/****
 *
 * DesempilhaIdxD(): Desempilha o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada e sada ) - ponteiro para a pilha na qual
 *                              ocorrer o empilhamento
 *
 * Retorno:  elemento do topo da pilha apontada por p
 *
 * Observao: Se a pilha estiver vazia, causa aborto
 *
 ****/
tItemPilha DesempilhaIdxD(tPilhaIdxD *p)
{
   ASSEGURA(!EstaVaziaPilhaIdxD(p), "Erro: Pilha vazia.");

   return (*p)->itens[(*p)->topo--];
}
