/****
 *
 * Ttulo: Lista Indexada sem Ordenao
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Gerencia uma lista de compras permitindo
 *            desfazer e refazer operaes
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include <stdlib.h>   /* Funes de alocao dinmica e */
                      /* gerao de nmeros aleatrios  */
#include <time.h>     /* Funo time() para alimentar o */
                      /* gerador de nmeros aleatrios  */
#include "Lista.h"    /* Interface do tipo lista  */
#include "leitura.h"  /* Funes de leitura       */

/***************** Constantes Simblicas ******************/

#define NOME_ARQUIVO "ArquivoLista.bin"

/***************** Definies de Funes ******************/

/****
 *
 * ExibeDoInicio(): Exibe na tela do primeiro ao ltimo
 *                  elemento de uma lista simplesmente encadeada
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeDoInicio(tListaSE lista)
{
   tConteudo *pConteudo = NULL;

   while ((pConteudo = ProximoListaSE(lista)))
      printf("%d\t", *pConteudo);
}

/****
 * AtualizaArquivoBinLSE(): Escreve o contedo efetivo de cada
 *                          n de uma lista simplesmente
 *                          encadeada num arquivo binrio
 * Parmetros:
 *    nomeArq (entrada) - nome do arquivo binrio no qual os
 *                        contedos dos ns sero escritos
 *    lista (entrada) - ponteiro para o primeiro n da lista
 *                      encadeada contendo as estruturas que
 *                      sero escritas no arquivo
 *
 * Retorno: 0, se ocorrer zero erro; 1, em caso contrrio.
 ****/
int AtualizaArquivoBinLSE(const char *nomeArq, tListaSE lista)
{
   FILE      *stream; /* Stream associado ao arquivo no qual ocorrer a escrita */
   tConteudo *pConteudo = NULL;

      /* Abre arquivo em formato binrio para escrita */
   stream = fopen(nomeArq, "wb");

      /* Se o arquivo no foi aberto, nada mais pode ser feito */
   if (!stream)
      return 1; /* Arquivo no foi aberto */

      /* Garante que o acesso comea do incio da lista */
   while (ProximoListaSE(lista)) {
      ; /* Instruo vazia */
   }

      /* Acessa cada n da lista e escreve seu */
      /* contedo efetivo no arquivo binrio   */
   while ((pConteudo = ProximoListaSE(lista))) {
         /* Tenta escrever o contedo do n corrente no arquivo */
      fwrite(&lista->conteudo, sizeof(*pConteudo), 1, stream);

         /* Se ocorreu erro de escrita, fecha o arquivo */
         /* e retorna um valor indicativo do fato       */
      if (ferror(stream)) {
         fclose(stream);
         return 1;
      }
   }
      /* Processamento terminado. Fecha o arquivo. */
   fclose(stream);

   return 0; /* Se ocorreu erro, j houve retorno */
}

/****
 *
 * ApresentaMenu(): Apresenta o menu do programa
 *
 * Parmetros: No h
 *
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(void)
{
   printf( "\n\t>>>>> Opcoes <<<<<\n"
           "\n\t(1) Acrescenta um elemento"
           "\n\t(2) Remove um elemento"
           "\n\t(3) Altera dados"
           "\n\t(4) Verifica se valor faz parte da lista"
           "\n\t(5) Exibe a lista do inicio"
           "\n\t(6) Comprimento da lista"
           "\n\t(7) Atualiza arquivo binrio"
           "\n\t(8) Encerra o programa"
           "\n\n\t>>> Escolha sua opcao: " );
}

/****
 *
 * main(): Testa as operaes sobre listas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaSE lista;
   int      i, valor, valor2, op, *pConteudo;

   IniciaListaSE(&lista);

      /* Ser criada uma lista inicial com 10 elementos. */
      /* Os contedos destes elementos sero nmeros     */
      /* inteiros entre 0 e 100 gerados aleatoriamente   */

      /* Alimenta o gerador de nmeros aleatrios */
   srand((int) time(NULL));

      /* Cria a lista inicial */
   for (i = 1; i <= 10; ++i) {
      valor = (int) rand()%100 + 1;
      InsereListaSE(&lista, valor);
   }

   while (1) {
      ApresentaMenu();

      op = LeOpcao("12345678");

      if (op == '8') { /* Encerra o programa */
         printf("\nBye!\n");
         break; /* Sada do lao */
      }

      switch (op) {
         case '1': /* Acrescenta um elemento */
            printf("\n>>> Digite o valor a ser acrescentado: ");
            valor = LeInteiro();

            InsereListaSE(&lista, valor);
            printf("\n>>> Acrescimo efetuado\n");

            break;

         case '2': /* Remove um aluno */
            printf("\n>>> Digite o valor a ser removido: ");
            valor = LeInteiro();

            if (!RemoveListaSE(&lista, valor)) {
               printf("\n>>> Remocao bem sucedida\n");
            } else {
               printf("\n>>> Valor nao encontrado\n");
            }

            break;

         case '3': /* Altera valor de um elemento */
            printf("\n>>> Digite o valor a ser alterado: ");
            valor = LeInteiro();

            printf("\n>>> Digite o valor que substituira' %d: ", valor);
            valor2 = LeInteiro();

            if (!AlteraConteudoListaSE(lista, valor, valor2)) {
               printf("\n>>> Remocao bem sucedida\n");
            } else {
               printf("\n>>> Valor nao encontrado\n");
            }

            break;

         case '4': /* Verifica se valor faz parte da lista */
            printf("\n>>> Digite o valor do elemento: ");
            valor = LeInteiro();

            pConteudo = BuscaListaSE(lista, valor);

            if (pConteudo) {
               printf("\n>>> O elemento %d: se encontra na lista",
                      valor);
            } else {
               printf("\n>>> O valor nao foi encontrado na lista\n");
            }

            break;

         case '5': /* Apresenta a lista a partir do incio */
            ExibeDoInicio(lista);
            break;

         case '6': /* Apresenta o comprimento da lista */
            printf( "\n>>>Comprimento da lista: %d\n",
                    ComprimentoListaSE(lista) );
            break;

         case '7': /* Atualiza arquivo binrio */
            if (!AtualizaArquivoBinLSE(NOME_ARQUIVO, lista)) {
               printf("\n>>> Atualizacao bem sucedida\n");
            } else {
               printf("\n>>> Ocorreu erro na atualizacao\n");
            }

            break;

         default: /* O programa no deve chegar at aqui */
            printf("\nEste programa contem um erro\n");
            return 1;
      }
   }

   return 0;
}

/******************* Exemplo de Execuo ******************



**********************************************************/
