/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada   */
#include <stdlib.h>  /* Alocao dinmica */
#include "ASSEGURA.h" /* Macro ASSEGURA */
#include "Lista.h"    /* Interface deste arquivo */

/***************** Definies de Funes ******************/

/****
 *
 * IniciaListaDE(): Inicia uma lista duplamente encadeada
 *
 * Parmetros: *lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
void IniciaListaDE(tListaDE *lista)
{
   *lista = NULL;
}

/****
 *
 * ComprimentoListaDE(): Calcula o nmero de ns de uma lista
 *                       duplamente encadeada
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para o primeiro n da lista
 *                       cujo comprimento ser calculado
 *
 * Retorno: O nmero de ns da referida lista
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
int ComprimentoListaDE(tListaDE lista)
{
   int tamanho = 0; /* Armazena o nmero de ns da lista */

      /* Acessa cada n da lista e conta */
      /* quantos ns so acessados       */
   while (lista) {
      ++tamanho; /* Mais um n foi encontrado */
      lista = lista->proximo; /* Passa para o prximo n */
   }

   return tamanho;
}

/****
 *
 * InsereEmOrdemLDE(): Insere um novo n numa lista
 *                     duplamente encadeada ordenada
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista na qual
 *                             ser feita a insero
 *     conteudo (entrada) - contedo do n que ser inserido
 *
 * Retorno: Nada
 *
 * Observao: Como a lista no obedece nenhuma ordem, pode-se
 *             inserir um n em qualquer posio da lista.
 *             Escolheu-se insero no incio da lista porque
 *              mais simples e eficiente.
 *
 ****/
void InsereEmOrdemLDE(tListaDE *lista, tConteudo conteudo)
{
   tNoListaDE    *ptrNovo; /* Apontar para o novo n alocado */
   tListaDE  p = *lista, /* p aponta para o n corrente  e  */
             q = NULL;   /* q aponta para o n anterior a p */

      /* Tenta alocar um novo n */
   ASSEGURA( ptrNovo = malloc(sizeof(tNoListaDE)),
             "Nao foi possivel alocar no'" );

      /* Armazena no novo n os dados recebidos como parmetro */
   ptrNovo->conteudo = conteudo;

      /* Se a lista estiver vazia, basta adicionar */
      /* o n ao incio da lista e retornar        */
   if (!*lista) {
         /* O novo n ser o nico da lista. Portanto, */
         /* ele no ter antecessor nem sucessor.      */
      ptrNovo->proximo = NULL;
      ptrNovo->anterior = NULL;

         /* O incio da lista apontar para novo n */
      *lista = ptrNovo;

      return;
   }

      /*************************************************/
      /* A lista tem pelo menos um n. Nesse caso, a   */
      /* posio de insero  aquela do primeiro n   */
      /* cujo contedo seja maior do que aquele do n  */
      /* a ser inserido. Se tal n no for encontrado, */
      /* o novo n ser o ltimo da lista.             */
      /*************************************************/

      /* Procura o local de insero */
   while ( p && p->conteudo <= conteudo ) {
      q = p;
      p = p->proximo; /* Faz p apontar para o prximo n */
   }

      /***********************************************/
      /* Neste instante, ou q aponta para o n       */
      /* aps o qual ocorrer a insero ou q  NULL */
      /***********************************************/

      /* Em qualquer caso, as seguintes */
      /* atribuies so vlidas        */
   ptrNovo->anterior = q;
   ptrNovo->proximo = p;

      /* Verifica se p aponta para o primeiro n da lista */
      /* pois insero no incio  um caso especial      */
   if (p == *lista) { /* Insero ser no incio da lista */
      (*lista)->anterior = ptrNovo;
      *lista = ptrNovo;
   } else {
         /* Se q no estiver apontando para o ltimo n, */
         /* o ponteiro anterior do prximo n apontar   */
         /* para o n novo                               */
      if (q->proximo) {
         q->proximo->anterior = ptrNovo;
      }
         /* O n para o qual q aponta */
         /* apontar para o novo n   */
      q->proximo = ptrNovo;
   }
}

/****
 *
 * RemoveListaDE(): Remove de uma lista duplamente encadeada
 *                um n que contm um determinado valor
 *                especificado como parmetro
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro que representa
 *                             a lista na qual ser feita a remoo
 *     conteudo (entrada) - contedo do n a ser removido
 *
 * Retorno: O contedo do n removido
 *
 ****/
tConteudo RemoveListaDE(tListaDE *lista, tConteudo conteudo)
{
   tConteudo itemRemovido;
   tListaDE  p = *lista; /* p apontar para o n */
                         /* que ser removido    */

      /*                                                     */
      /* O lao while procura um n que cujo contedo case   */
      /* com o parmetro 'conteudo' e tem duas condies de  */
      /* parada: (1) todos os ns so acessados sem que o    */
      /* referido n tenha sido encontrado (neste  caso, p   */
      /* assume NULL); (2) o n que contm o valor procurado */
      /*  encontrado.                                       */
      /*                                                     */
   while (p && p->conteudo != conteudo) {
      p = p->proximo; /* p passa a apontar para o prximo n */
   }

      /* Se p for NULL, a lista foi totalmente examinada   */
      /* sem que o n a ser removido tenha sido encontrado */
   ASSEGURA (p, "Elemento nao foi encontrado");

      /**************************************************/
      /* Neste ponto, p aponta para o n a ser removido */
      /**************************************************/

      /* Guarda o contedo do n a ser removido */
   itemRemovido = p->conteudo;

      /* Se p no estiver apontando para o ltimo n, o  */
      /* ponteiro 'anterior' do prximo n apontar para */
      /* o n que antecede o n sendo removido           */
   if (p->proximo) {
      p->proximo->anterior = p->anterior;
   }

      /* Verifica se p aponta para o primeiro */
      /* n da lista porque remoo no incio */
      /* deve ser tratada separadamente       */
   if (p == *lista) { /* Remoo ser no incio da lista */
      (*lista) = p->proximo; /* Altera incio da lista */
   } else {/* N a ser removido NO  o primeiro da lista */
      p->anterior->proximo = p->proximo; /* Desvia o n anterior */
                                         /* do n a ser removido */
   }

      /* Neste ponto, o n desejado no faz mais parte da */
      /* lista, mas ele ainda ocupa espao em memria.    */
      /* Portanto,  necessrio liberar este espao.      */
   free(p); /* Libera o n removido */

   return itemRemovido;
}

/****
 *
 * EstaVaziaListaDE(): Verifica se uma lista duplamente
 *                 encadeada est vazia
 *
 * Parmetros:
 *     lista (entrada) - a lista que ser verificada
 *
 * Retorno: 1, quando a lista est vazia, ou 0, em caso contrrio
 *
 ****/
int EstaVaziaListaDE(tListaDE lista)
{
   return lista == NULL;
}

/****
 *
 * BuscaListaDE(): Retorna o endereo do contedo efetivo do
 *            n que possui um determinado contedo numa
 *            lista duplamente encadeada
 *
 * Parmetros:
 *      lista (entrada) - lista que ser pesquisada
 *      valor (entrada) - contedo do n procurado
 *
 * Retorno: Endereo dos dados que contm o referido contedo,
 *          se ele for encontrado. Caso contrrio, NULL.
 *
 ****/
tConteudo *BuscaListaDE(tListaDE lista, tConteudo valor)
{
      /* Enquanto o ponteiro 'lista' no assume NULL ou o   */
      /* campo 'conteudo' de um n no casa com o parmetro */
      /* 'conteudo', a busca prossegue                      */
   while (lista && lista->conteudo != valor) {
      lista = lista->proximo;
   }

      /* Se 'lista' assume NULL  porque o contedo     */
      /* especificado como parmetro no foi encontrado */
   if(!lista) {
      return NULL; /* N no foi encontrado */
   }

      /* O n foi encontrado. Ento, retorna-se */
      /* o endereo de seu campo 'conteudo'.    */
   return &lista->conteudo;
}

/****
 *
 * AlteraConteudoLDE(): Altera o contedo efetivo de um n
 *                      de uma lista duplamente encadeada
 *
 * Parmetros:
 *     lista (entrada) - lista que contm o n que
 *                       ser alterado
 *     antigo (entrada) - antigo valor
 *     novo (entrada) - novo valor
 *
 * Retorno: 0, se a atualizao for bem sucedida, ou 1, se a
 *          atualizao no for possvel, porque o n no foi
 *          encontrado
 *
 ****/
int AlteraConteudoLDE(tListaDE lista, tConteudo antigo, tConteudo novo)
{
   tConteudo *pConteudo; /* Ponteiro para o contedo */
                         /* que ser alterado        */

      /* Obtm o endereo do contedo efetivo do */
      /* n que casa com o parmetro 'antigo'    */
   pConteudo = BuscaListaDE(lista, antigo);

      /* Se 'pConteudo' for NULL o n no foi encontrado */
   if (!pConteudo) {
      return 1; /* N no foi encontrado */
   }

         /*                                 */
         /* O n foi encontrado. Altera-se  */
         /* seu contedo e retorna-se zero. */
         /*                                 */

   *pConteudo = novo; /* Altera o contedo do n */

   return 0; /* Alterao completa */
}

/****
 *
 * DestroiListaDE(): Libera o espao ocupado pelos ns de
 *                   uma lista encadeada, tornando-a vazia
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a
 *                             lista duplamente encadeada
 *                             que ser destruda
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaDE(tListaDE *lista)
{
   tListaDE p; /* Aponta para o prximo n a ser liberado */

   if (!*lista) { /* Verifica se a lista est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *lista; /* Faz p apontar para o incio da lista */

      /* Visita cada n da lista liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /* o incio da lista seja alterado. Afinal, este   */
         /* ponteiro ser mesmo anulado.                    */
      *lista = (*lista)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *lista    */
      p = *lista;
   } while (p);

      /* Neste ponto, a lista ficou vazia, pois o  */
      /* ltimo valor assumido por *lista foi NULL */
}

/****
 *
 * ProximoListaDE(): Encontra o prximo n de uma lista
 *                   duplamente encadeada
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *                       duplamente encadeada
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *ProximoListaDE(tListaDE lista)
{
   static tNoListaDE *proximoNo = NULL;

   if (lista) {
      if (!proximoNo) {
         proximoNo = lista;
      } else {
         proximoNo = proximoNo->proximo;
      }
      return proximoNo ? &proximoNo->conteudo : NULL;
   } else {
      return NULL;
   }
}

static tListaDE UltimoNo(tListaDE lista)
{
   tListaDE p = lista,
            q = NULL;

   while (p) {
      q = p;
      p = p->proximo;
   }

   return q;
}

/****
 *
 * AnteriorListaDE(): Encontra o n anterior de uma lista
 *                    duplamente encadeada
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *                       duplamente encadeada
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *AnteriorListaDE(tListaDE lista)
{
   static tNoListaDE *noAnterior = NULL;

   if (lista) {
      if (!noAnterior) {
         noAnterior = UltimoNo(lista);
      } else {
         noAnterior = noAnterior->anterior;
      }
      return noAnterior ? &noAnterior->conteudo : NULL;
   } else {
      return NULL;
   }
}

