/******
 *
 * Arquivo Pilha2.c: Implementao do mdulo Pilha
 *
 *****/

#include <stdio.h>      /* Funes de Entrada/Sada  */
#include "Pilha.h"      /* Declaraes do tipo pilha */
#include "ASSEGURA.h"   /* Macro ASSEGURA            */

/****************** Complemento de Tipo *******************/

struct rotPilhaSE {
   tItemPilha       conteudo;
   struct rotPilhaSE *proximo;
};

typedef struct rotPilhaSE tNoPilhaSE;

/**************** Definies de funes ******************/

/****
 *
 * CriaPilhaSE(): Inicializa uma pilha
 *
 * Parmetros:
 *      *p (sada) - a pilha que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void CriaPilhaSE(tPilhaSE *p)
{
      /* Torna nulo o ponteiro para o topo da pilha, que  o   */
      /* primeiro elemento de uma lista simplesmente encadeada */
   *p = NULL;
}

/****
 *
 * DestroiPilhaSE(): Libera o espao ocupado pela pilha
 *
 * Parmetros: pilha (entrada e sada) - a pilha
 *
 * Retorno: Nada
 *
 ****/
void DestroiPilhaSE(tPilhaSE *pilha)
{
   tPilhaSE p; /* Aponta para o prximo n a ser liberado */

   if (!*pilha) { /* Verifica se a pilha est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *pilha; /* Faz p apontar para o incio da pilha */

      /* Visita cada n da pilha liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /*o incio da pilha seja alterado. Afinal, este    */
         /* ponteiro ser mesmo anulado.                    */
      *pilha = (*pilha)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *pilha    */
      p = *pilha;
   } while (p);

      /* Neste ponto, a pilha ficou vazia, pois o  */
      /* ltimo valor assumido por *pilha foi NULL */
}

/****
 *
 * EstaVaziaPilhaSE(): Verfica se uma pilha est vazia
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1 se a pilha recebida como parmetro estiver vazia;
 *          0, em caso contrrio
 *
 ****/
int EstaVaziaPilhaSE(const tPilhaSE *p)
{
      /* A pilha est vazia quando o ponteiro */
      /* que a representa  NULL              */
   return !*p;
}

/****
 *
 * ElementoTopoSE(): Obtm o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: O elemento do topo da pilha recebida como parmetro
 *
 * Observao: Se a pilha estiver vazia, causa aborto de programa
 *
 ****/
tItemPilha ElementoTopoSE(const tPilhaSE *p)
{
      /* Se a pilha estiver vazia, aborta o programa */
   ASSEGURA(!EstaVaziaPilhaSE(p), "A pilha esta' vazia");

   return (*p)->conteudo;
}

/****
 *
 * EmpilhaSE(): Empilha um item numa pilha
 *
 * Parmetros:
 *      p (entrada e sada) - ponteiro para a pilha na qual
 *                            ocorrer o empilhamento
 *      item (entrada) - ponteiro para o contedo que ser empilhado
 *
 * Retorno: Nada
 *
 * Observao: Se a pilha estiver cheia, causa aborto
 *
 ****/
void EmpilhaSE(tPilhaSE *p, const tItemPilha *item)
{
   tPilhaSE pNovoNo;

      /* Tenta alocar um n da pilha */
   pNovoNo = malloc(sizeof(struct rotPilhaSE));

      /* Se a alocao no foi possvel aborta o programa */
   ASSEGURA(pNovoNo, "Nao houve alocacao");

   /*                                                    */
   /* O topo da pilha  o primeiro elemento de uma lista */
   /* encadeada. Logo, empilhar  o mesmo que inserir    */
   /* um elemento no incio de uma lista encadeada.      */
   /*                                                    */

      /* Armazena o contedo do novo n */
   pNovoNo->conteudo = *item;

      /* Faz o novo n apontar para */
      /* o topo corrente da pilha   */
   pNovoNo->proximo = *p;

      /* O novo n passa a ser o topo */
   *p = pNovoNo;
}

/****
 *
 * DesempilhaSE(): Desempilha o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada e sada ) - ponteiro para a pilha na qual
 *                              ocorrer o empilhamento
 *
 * Retorno:  elemento do topo da pilha apontada por p
 *
 * Observao: Se a pilha estiver vazia, causa aborto
 *
 ****/
tItemPilha DesempilhaSE(tPilhaSE *p)
{
   tPilhaSE    aux; /* Ponteiro que auxilia a operao */
   tItemPilha conteudo;

      /* Se a pilha estiver vazia, aborta o programa */
   ASSEGURA(!EstaVaziaPilhaSE(p), "A pilha esta' vazia");

   /*                                                    */
   /* O topo da pilha  o primeiro elemento de uma lista */
   /* encadeada. Logo, desempilhar  o mesmo que remover */
   /* o elemento inicial de uma lista encadeada.         */
   /*                                                    */

      /* Armazena o contedo do elemento do topo */
   conteudo = (*p)->conteudo;

      /* Faz 'aux' apontar para o topo da pilha */
   aux = *p;

      /* Faz com que o topo passe a ser */
      /* o segundo elemento da lista    */
   *p = (*p)->proximo;

      /* Libera o n que constitua o antigo topo */
   free(aux);

      /* Retorna o valor do segundo parmetro */
   return conteudo;
}
