/****
 *
 * Ttulo: Pilha Encadedada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Implementao de pilha dinmica
 *
 * Entrada: Caracteres (letras)
 *
 * Sada: Resultados de operaes sobre uma pilha
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>    /* Entrada e sada           */
#include "Pilha.h"    /* Declaraes do tipo pilha */
#include "leitura.h"  /* Leitura via teclado       */

/****************** Constantes Simblicas *****************/

#define LETRAS "ABCDEFGHIJKLMNOPQRSTUVWXYZ"\
               "abcdefghijklmnopqrstuvwxyz"

   /* Nmero de opes do programa */
#define N_OPCOES 5

/****************** Definies de Funes *****************/

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros:
 *      opcoes[] (entrada) - array de strings contendo as
 *                           opes do usurio
 *      nOpcoes (entrada) - o nmero de opes
 *
 * Retorno: Nada
 *
 ****/
static void ApresentaMenu(char *opcoes[], int nOpcoes)
{
   int i;

   printf("\n\nEscolha uma das opcoes a seguir:");

   for (i = 0; i < nOpcoes; ++i)
      printf("\n%d - %s", i + 1, opcoes[i]);

   putchar('\n');
}

/****
 *
 * main():
 *         1. Apresenta um menu com as seguintes opes
 *            para o usurio:
 *               1. Empilha
 *               2. Desempilha
 *               3. Elemento do topo
 *               4. Esvazia pilha
 *               5. Encerra o programa
 *         2. Enquanto o usurio no escolher a opo de
 *            sada do programa, l a opo escolhida pelo
 *            usurio e executa a operao correspondente
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tPilhaSE     pilha;
   tItemPilha  letra;
   int op;
   char *menu[N_OPCOES] = { "Empilha",
                            "Desempilha",
                            "Elemento do topo",
                            "Esvazia pilha",
                            "Encerra o programa"
                          };

   CriaPilhaSE(&pilha);  /* Inicia a pilha */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa testa operacoes"
           "\n\t>>> sobre uma pilha de letras.\n" );

      /* Lao principal do programa. O programa */
      /* acaba quando o usurio escolhe a opo */
      /* de encerramento.                       */
   while (1) {
         /* Apresenta menu e obtm a opo */
         /* escolhida pelo usurio         */

      ApresentaMenu(menu, N_OPCOES);
      op = LeOpcao("12345");

      if (op == '5') { /* Encerra o programa */
            /* Despede-se do usurio */
         printf("Obrigado por usar este este programa");
         return 0; /* Sada do lao */
      }

         /* Lida com demais opes */
      switch (op) {
         case '1': /* Empilha */
            printf("\nDigite a letra que sera' empilhada:\n\t> ");
            letra = LeOpcao(LETRAS); /* L uma letra */

            EmpilhaSE(&pilha, &letra);
            printf("A letra foi empilhada");

            break;
         case '2': /* Desempilha */
            if (EstaVaziaPilhaSE(&pilha)) {
               printf("A pilha esta' vazia\n");
            } else {
               printf( "\n\t>>> Valor desempilhado: %c\n",
                       DesempilhaSE(&pilha) );
            }

            break;
         case '3': /* Elemento do topo */
            if (EstaVaziaPilhaSE(&pilha)) {
               printf("A pilha esta' vazia");
            } else {
               printf( "\n\t>>> Elemento no topo: %c\n", ElementoTopoSE(&pilha) );
            }

            break;
         case '4': /* Esvazia pilha */
            if (EstaVaziaPilhaSE(&pilha)) {
               printf("A pilha ja' esta' vazia");
            } else {
               while ( !EstaVaziaPilhaSE(&pilha) ) {
                  printf( "\n\t>>> Desempilhado: %c\n",
                          DesempilhaSE(&pilha) );
               }
            }

            break;
         default: /* Esta opo no deve ser executada */
            printf("\nEste programa contem um erro\n");
            break;
      } /* switch */
   } /* while */

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa testa operacoes
        >>> sobre uma pilha de letras.

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 1

Digite a letra que sera' empilhada:
        > a

        >>> A letra foi empilhada

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 1

Digite a letra que sera' empilhada:
        > b

        >>> A letra foi empilhada

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 2

        >>> Valor desempilhado: b

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 3

        >>> Elemento no topo: a

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 4

        >>> Desempilhado: a

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 3

        >>> A pilha esta' vazia

        ********** Opcoes *********

        1. Empilha
        2. Desempilha
        3. Elemento do topo
        4. Esvazia pilha
        5. Encerra o programa

        >>> Escolha uma opcao: 5

        >>> Obrigado por usar este este programa.

**********************************************************/
