/****
 *
 * Ttulo: Nmeros Felizes
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 28/03/2013
 * ltima modificao: 28/03/2013
 *
 * Descrio: Verifica se um nmero inteiro positivo  feliz
 *
 * Entrada: Valor inteiro positivos maior do que 1
 *
 * Sada: Mensagem indicando se o nmero  feliz ou no
 *
 * Observao: Considere uma sequncia de nmeros inteiros positivos
 *             na qual cada elemento consiste na soma dos quadrados
 *             dos dgitos do elemento que o antecede. Ento, um
 *             nmero  feliz quando o ltimo nmero desta sequncia
 *             for igual a 1. Por exemplo, 13  um nmero feliz, pois:
 *
 *                 1^2 + 3^2 = 10
 *                 1^2 + 0^2 = 1
 *
 *             Quando um nmero no  feliz, a sequncia descrita
 *             acima torna-se cclica em algum ponto. Por exemplo,
 *             3 no  um nmero feliz:
 *
 *                 3^2 = 9
 *                 9^2 = 81
 *                 8^2 + 1^2 = 65 <---
 *                 6^2 + 5^2 = 57    |
 *                 5^2 + 7^2 = 74    |
 *                 7^2 + 4^2 = 65 <---
 *                 ... [A sequncia ir se repetir]
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada            */
#include <stdlib.h>   /* Alocao dinmica e exit() */
#include "ASSEGURA.h" /* Macro ASSEGURA             */
#include "leitura.h"  /* Leitura via teclado        */
#include "Lista.h"    /* Interface do tipo tListaSE */

/***************** Definies de Funes ******************/

/****
 *
 * SomaDigitosQuadrados(): Soma os quadrados dos dgitos de um
 *                         nmero inteiro positivo
 *
 * Parmetros:
 *      numero (entrada) - o referido nmero
 *
 * Retorno: A soma dos quadrados dos dgitos do nmero se
 *          ele no for negativo.
 *
 ****/
int SomaDigitosQuadrados(int numero)
{
   int digito ,/* Armazenar cada dgito do nmero */
       soma = 0; /* Armazenar a soma dos quadrados dos dgitos */

      /* Verifica se o nmero  vlido */
   ASSEGURA(numero >= 0, "Numero invalido");

      /* O lao encerra quando o nmero cujos dgitos */
      /* sero somados assume zero como valor         */
   do {
      digito = numero % 10; /* Obtm o prximo dgito do nmero */

         /* Atualiza o valor do nmero que */
         /* ainda falta levar em conta     */
      numero = numero / 10;

         /* Atualiza a soma dos quadrados dos dgitos */
      soma = soma + digito*digito;
   } while (numero);

   return soma;
}

/****
 *
 * EhFeliz(): Verifica se um nmero inteiro maior do que 1  feliz
 *
 * Parmetros:
 *      num (entrada) - o nmero a ser examinado
 *
 * Retorno: 1, se o nmero for feliz; 0, em caso contrrio
 *
 * Observao: Esta funo aborta o programa se ocorrer
 *             erro de alocao dinmica de memria
 *
 ****/
int EhFeliz(int num)
{
   tListaSE lista; /* Lista encadeada que armazena os termos */
                   /* da sequncia de nmeros gerada         */

   IniciaListaSE(&lista);

      /* Nmeros felizes devem ser maiores do que 1 */
   if (num <= 1) {
      return 0; /* Esse nmero  um infeliz */
   }

      /* O lao termina quando for concludo */
      /* se o nmero  feliz ou no          */
   while (1) {
         /* Insere o termo corrente da     */
         /* sequncia numa lista encadeada */
      InsereListaSE(&lista, num);

         /* Obtm o termo seguinte da sequncia somando */
         /* os quadrados dos dgitos do termo corrente  */
      num = SomaDigitosQuadrados(num);

         /* Se o novo termo for 1, o nmero  feliz. Se   */
         /* j apareceu na sequncia, ele no  feliz.    */
         /* Em outro caso, continua gerando novos termos. */
      if (num == 1) { /* O nmero  feliz */
            /* Libera o espao ocupado pela */
            /* lista antes de retornar      */
         DestroiListaSE(&lista);

         return 1; /* Felicssimo! */
      } else if ( BuscaListaSE(lista, num) ) {
            /* O novo termo j foi gerado e, portanto, */
            /* o nmero no  feliz. Libera o espao   */
            /* ocupado pela lista e retorna.           */
         DestroiListaSE(&lista);
         return 0; /* O coitado  infeliz */
      }
   }

      /* O fluxo de execuo no deve chegar at aqui */
   printf("\nEste programa nao e' feliz\n");

   return 0;
}

/****
 *
 * main(): Verifica se nmeros introduzidos via teclado so felizes
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int numero;

   printf( "\n\t>>> Este programa determina se os numeros "
           "introduzidos\n\t>>> sao felizes ou nao. Um valor "
           "menor do que 2 encerra\n\t>>> o programa.\n" );

   while (1) {
      printf("\n\t>>> Digite o numero que sera' checado: ");

         /* Se o usurio digitar um valor  */
         /* menor do que 2, encerra o lao */
      if ((numero = LeInteiro()) < 2) {
         break;
      }

         /* Apresenta o resultado do exame do nmero */
      printf( "\n\t>>> O numero %d %se' feliz\n", numero,
              EhFeliz(numero) ? "" : "nao " );
   }

   printf( "\n\t>>> Obrigado por usar este programa. "
           "Felicidades!\n" );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa determina se os numeros introduzidos
        >>> sao felizes ou nao. Um valor menor do que 2 encerra
        >>> o programa.

        >>> Digite o numero que sera' checado: 5

        >>> O numero 5 nao e' feliz

        >>> Digite o numero que sera' checado: 70

        >>> O numero 70 e' feliz

        >>> Digite o numero que sera' checado: 0

        >>> Obrigado por usar este programa. Felicidades.

**********************************************************/

