/****
 *
 * Ttulo: Representao de Polinmios Usando Listas Encadeadas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/


/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada          */
#include <string.h>  /* Processamento de strings */
#include <stdlib.h>  /* Alocao dinmica        */
#include <math.h>    /* Operaes de ponto flutuante */

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              fprintf(stderr, "\n%s", msg); \
                              exit(1); \
                           }

   /* Determina o valor absoluto de um nmero */
#define ABS(x) (((x) > 0) ? (x) : (-(x)))

/****************** Definies de Tipos *******************/

   /* Tipo de um termo de polinmio */
typedef struct {
           int coef; /* Coeficiente do termo */
           int exp;  /* Expoente do termo    */
        } tTermo;

   /* Tipo de um polinmio */
typedef struct no {
           tTermo     termo;   /* Termo         */
           struct no *proximo; /* Prximo termo */
        } tPoli, *tPoliPtr;

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeCaractere(void);
extern int LeInteiro(void);
extern int LeNatural(void);
extern double LeReal(void);
extern int LeOpcao(const char *opcoes);

extern int InsereTermo(tPoliPtr *poli, const tTermo *oTermo);
extern tPoliPtr LePolinomio(void);
extern tPoliPtr SomaPolinomios(tPoli *poli1,  tPoli *poli2);
extern tPoliPtr SubtraiPolinomios(tPoli *poli1,  tPoli *poli2);
extern double ValorNumerico(const tPoli *poli, double a);
extern int Grau(const tPoli *poli);
extern void ExibeTermo(const tTermo *termo, int ordem);
extern void ExibePolinomio(const tPoli *poli);
extern void DestroiPolinomio(tPoliPtr *poli);
extern void ApresentaMenu(void);

/***************** Definies de Funes ******************/

/****************** Funes de Leitura ********************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>>Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeNatural(): L um nmero natural via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 0) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeReal(): L um valor do tipo double via teclado
 *           deixando o buffer associado ao teclado
 *           sem caracteres remanescentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido e validado
 *
 ****/
double LeReal(void)
{
   int    teste, /* Armazena o retorno de scanf() */
          nResto = 0; /* Nmero de caracteres */
                      /* excedentes           */
   double valorLido; /* O valor digitado pelo usurio */

      /* O lao encerra quando o usurio se comporta bem */
   while (1) {
         /* Tenta ler um valor do tipo double */
      teste = scanf("%lf", &valorLido);

         /* Se no ocorreu erro de leitura ou de final */
         /* de arquivo, h caracteres remanescentes    */
         /* que precisam ser removidos                 */
      if (teste != EOF) {
         nResto = LimpaBuffer();
      }

         /* Se o valor retornado por scanf() */
         /* foi 1, a leitura foi OK          */
      if (teste == 1) {
         break; /* Leitura foi nota 10 */
      } else { /* Usurio foi mal comportado */
         printf( "\a\n\t>>> O valor digitado e' invalido. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* Retorna um valor vlido do tipo double */
   return valorLido;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

/********* Funes de Processamento de Polinmios *********/

/****
 *
 * InsereTermo(): Insere um novo termo num polinmio
 *                representado usando lista simplesmente
 *                encadeada
 *
 * Parmetros:
 *     *poli (entrada e sada) - o polinmio
 *      oTermo (entrada) - o novo termo
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
int InsereTermo(tPoliPtr *poli, const tTermo *oTermo)
{
   tPoli    *pNovoNo;  /* Apontar para o novo n alocado */
   tPoliPtr  p = *poli, /* p aponta para o n corrente     */
             q = NULL;   /* q aponta para o n anterior a p */

   ASSEGURA( pNovoNo = malloc(sizeof(*pNovoNo)),
             "Impossivel alocar um termo" );

      /* Armazena no novo n o termo recebido como parmetro */
   pNovoNo->termo = *oTermo;

      /* Se a lista estiver vazia, basta adicionar */
      /* o n ao incio da lista e retornar        */
   if (!*poli) {
      *poli = pNovoNo; /* Incio da lista apontar */
                          /* para novo n             */
      (*poli)->proximo = NULL; /* Novo n  o nico da lista */
      return 0; /* Servio completo */
   }

         /***************************************************/
         /* A lista tem pelo menos um n. Nesse caso, a     */
         /* posio de insero  aquela do primeiro n     */
         /* cujo campo 'exp' seja menor do que aquele do n */
         /* a ser inserido. Se tal n no for encontrado, o */
         /* novo n ser o ltimo da lista.                 */
         /***************************************************/

      /* Procura o local de insero */
   while (p && oTermo->exp < p->termo.exp) {
      q = p;    /* q passa a apontar para o n corrente */
      p = p->proximo; /* p passa a apontar */
                      /* para o prximo n */
   }

      /* Verifica se o termo j existe */
   if (p && oTermo->exp == p->termo.exp) {
      free(pNovoNo);
      return 1; /* Termo j existe */
   }

      /********************************************************/
      /* Neste ponto, p aponta para o local da insero e q   */
      /* aponta para o n imediatamente antes desse local. Se */
      /* p for NULL, o novo n ser o ltimo da lista, mas    */
      /* isso no  um caso especial a ser tratado  parte.   */
      /********************************************************/

      /* O novo n apontar para o n apontado por p */
   pNovoNo->proximo = p;

      /* Verifica se p aponta para o primeiro n da lista pois */
      /* insero no incio deve ser tratada separadamente     */
   if (p == *poli) { /* Insero ser no incio da lista */
      *poli = pNovoNo;/* Insero no incio  especial */
   } else {
         /* O n anterior quele apontado */
         /* por p apontar para o novo n */
      q->proximo = pNovoNo;
   }

   return 0; /* Servio completo */
}

/****
 *
 * LePolinomio(): Le termos de um polinmio introduzidos via
 *                teclado e representa-os utilizando uma lista
 *                simplesmente encadeada
 *
 * Parmetros: Nenhum
 *
 * Retorno: O polinmio lido e representado em lista encadeada
 *
 ****/
tPoliPtr LePolinomio(void)
{
   tPoliPtr umPolinomio = NULL; /* Ponteiro para a lista que */
                                /* armazenar o polinmio    */
   tTermo   umTermo; /* Armazenar um termo do polinmio */

      /* O lao a seguir encerra quando o usurio */
      /* digitar 0 como valor de coeficiente      */
   while (1) {
         /* O coeficiente deveria ser real, mas, */
         /* como fazem os livros de matemtica,  */
         /* por comodidade, ele ser inteiro     */
      printf("\n\tCoeficiente: ");
      umTermo.coef = LeInteiro();

         /* Se o coeficiente for zero, encerra o lao */
      if (!umTermo.coef) {
         break;
      }

         /* O expoente deve ser um inteiro no negativo */
      printf("\tExpoente: ");
      umTermo.exp = LeNatural();

         /* Tenta inserir o novo termo na lista que   */
         /* representa o polinmio. Se a funo       */
         /* InsereTermo() retornar um valor diferente */
         /* de zero, o termo j existe e o programa   */
         /* ser abortado.                            */
      ASSEGURA( !InsereTermo(&umPolinomio, &umTermo),
                "\nEste termos ja' existe" );
   }

      /* Se 'umPolinomio' for NULL, nenhum termo foi inserido */
      /* no polinmio. Nesse caso, aborta o programa.         */
   ASSEGURA(umPolinomio, "Polinomio nao possui nenhum termo");

   return umPolinomio; /* Done */
}

/****
 *
 * SomaPolinomios(): Soma dois polinmios representados como
 *                   listas encadeadas
 *
 * Parmetros:
 *     *poli1 (entrada) - primeiro polinmio a ser somado
 *     *poli1 (entrada) - segundo polinmio a ser somado
 *
 * Retorno: O resultado da soma
 *
 ****/
tPoliPtr SomaPolinomios(tPoli *poli1, tPoli *poli2)
{
   tPoliPtr resultado = NULL; /* Ponteiro para a lista que */
                              /* armazenar o resultado    */
   tTermo novoTermo; /* Armazenar um termo resultante da soma */
                     /* de dois termos dos polinmios somados  */

      /* Acessa cada termo de cada polinmio e encerra */
      /* o lao quando todos os termos de um dos       */
      /* polinmios tiverem sido acessados             */
   while (poli1 && poli2) {
         /* Se os termos correntes tiverem o mesmo     */
         /* expoente, o coeficiente do novo termo ser */
         /* a soma dos coeficentes desses termos       */
      if (poli1->termo.exp == poli2->termo.exp) {
         novoTermo.coef = poli1->termo.coef +
                          poli2->termo.coef;

            /* O expoente do novo termo  o mesmo    */
            /* expoente dos termos que foram somados */
         novoTermo.exp = poli1->termo.exp;

         poli1 = poli1->proximo;
         poli2 = poli2->proximo;
      } else if (poli1->termo.exp > poli2->termo.exp) {
            /* Armazena no resultado o termo do polinmio */
            /* com maior expoente e passa para o prximo  */
            /* termo desse polinmio                      */
         novoTermo = poli1->termo;
         poli1 = poli1->proximo;
      } else {
            /* Armazena no resultado o termo do polinmio */
            /* com maior expoente e passa para o prximo  */
            /* termo desse polinmio                      */
         novoTermo = poli2->termo;
         poli2 = poli2->proximo;
      }

         /* Armazena um novo termo na lista que  */
         /* representa o resultado apenas quando */
         /* o coeficiente for diferente de 0     */
      if (novoTermo.coef) {
            /* Insere o novo termo. O programa no */
            /* deve ser abortado aqui, a no ser   */
            /* que ele esteja incorreto.           */
         ASSEGURA( !InsereTermo(&resultado, &novoTermo),
                   "Ocorreu um erro inesperado!" );
      }
   }

      /* Acrescenta eventuais termos remanescentes */
      /* do primeiro polinmio ao resultado       */
   while (poli1) {
      ASSEGURA( !InsereTermo(&resultado, &poli1->termo),
                "Ocorreu um erro inesperado!" );
      poli1 = poli1->proximo;
   }

      /* Acrescenta eventuais termos remanescentes */
      /* do segundo polinmio ao resultado         */
   while (poli2) {
      ASSEGURA( !InsereTermo(&resultado, &poli2->termo),
                "Ocorreu um erro inesperado!" );
      poli2 = poli2->proximo;
   }

      /* Se nenhum termo foi acrescentado  lista que         */
      /* representa o resultado, o resultado s pode ser zero */
   if (!resultado) {
      novoTermo.coef = novoTermo.exp = 0;
      ASSEGURA( !InsereTermo(&resultado, &novoTermo),
                "Ocorreu um erro inesperado!" );
   }

   return resultado;
}

/****
 *
 * SubtraiPolinomios(): Subtrai dois polinmios representados
 *                      como listas encadeadas
 *
 * Parmetros:
 *     *poli1 (entrada) - primeiro polinmio a ser subtrado
 *     *poli1 (entrada) - segundo polinmio a ser subtrado
 *
 * Retorno: O resultado da subtrao
 *
 ****/
tPoliPtr SubtraiPolinomios(tPoli *poli1, tPoli *poli2)
{
   tPoliPtr resultado = NULL; /* Ponteiro para a lista que */
                              /* armazenar o resultado    */
   tTermo novoTermo; /* Armazenar um termo resultante da soma */
                     /* de dois termos dos polinmios somados  */

      /* Acessa cada termo de cada polinmio e encerra */
      /* o lao quando todos os termos de um dos       */
      /* polinmios tiverem sido acessados             */
   while (poli1 && poli2) {
         /* Se os termos correntes tiverem o mesmo     */
         /* expoente, o coeficiente do novo termo ser */
         /* a soma dos coeficentes desses termos       */
      if (poli1->termo.exp == poli2->termo.exp) {
         novoTermo.coef = poli1->termo.coef -
                          poli2->termo.coef;

            /* O expoente do novo termo  o mesmo    */
            /* expoente dos termos que foram somados */
         novoTermo.exp = poli1->termo.exp;

         poli1 = poli1->proximo;
         poli2 = poli2->proximo;
      } else if (poli1->termo.exp > poli2->termo.exp) {
            /* Armazena no resultado o termo do polinmio */
            /* com maior expoente e passa para o prximo  */
            /* termo desse polinmio                      */
         novoTermo = poli1->termo;
         poli1 = poli1->proximo;
      } else {
            /* Armazena no resultado o termo do polinmio */
            /* com maior expoente e passa para o prximo  */
            /* termo desse polinmio                      */
         novoTermo = poli2->termo;
         poli2 = poli2->proximo;
      }

         /* Armazena um novo termo na lista que  */
         /* representa o resultado apenas quando */
         /* o coeficiente for diferente de 0     */
      if (novoTermo.coef) {
            /* Insere o novo termo. O programa no */
            /* deve ser abortado aqui, a no ser   */
            /* que ele esteja incorreto.           */
         ASSEGURA( !InsereTermo(&resultado, &novoTermo),
                   "Ocorreu um erro inesperado!" );
      }
   }

      /* Acrescenta eventuais termos remanescentes */
      /* do primeiro polinmio ao resultado       */
   while (poli1) {
      ASSEGURA( !InsereTermo(&resultado, &poli1->termo),
                "Ocorreu um erro inesperado!" );
      poli1 = poli1->proximo;
   }

      /* Acrescenta eventuais termos remanescentes */
      /* do segundo polinmio ao resultado         */
   while (poli2) {
      ASSEGURA( !InsereTermo(&resultado, &poli2->termo),
                "Ocorreu um erro inesperado!" );
      poli2 = poli2->proximo;
   }

      /* Se nenhum termo foi acrescentado  lista que         */
      /* representa o resultado, o resultado s pode ser zero */
   if (!resultado) {
      novoTermo.coef = novoTermo.exp = 0;
      ASSEGURA( !InsereTermo(&resultado, &novoTermo),
                "Ocorreu um erro inesperado!" );
   }

   return resultado;
}

/****
 *
 * ValorNumerico(): Calcula o valor numrico de um polinmio
 *                  representado como lista encadeada quando
 *                  ele  aplicado sobre um valor real
 *
 * Parmetros:
 *     *poli1 (entrada) - primeiro polinmio a ser subtrado
 *     *poli1 (entrada) - segundo polinmio a ser subtrado
 *
 * Retorno: O resultado da subtrao
 *
 ****/
double ValorNumerico(const tPoli *poli, double a)
{
   double resultado = 0.0;

   while (poli) {
      resultado += poli->termo.coef*pow(a, poli->termo.exp);
      poli = poli->proximo;
   }

   return resultado;
}

/****
 *
 * Grau(): Calcula o grau de um polinmio representado como
 *         lista encadeada quando
 *
 * Parmetros:
 *     *poli1 (entrada) - o polinmio cujo grau ser determinado
 *
 * Retorno: O grau do polinmio
 *
 ****/
int Grau(const tPoli *poli)
{
      /* Como os polinmios so armazenados em ordem      */
      /* decrescente de expoentes, o grau de um polinmio */
      /* corresponde ao expoente do primeiro termo        */
   return poli->termo.exp;
}

/****
 *
 * ExibeTermo(): Exibe um termo de um polinmio na tela
 *
 * Parmetros:
 *      termo (entrada) - o termo que ser exibido
 *      ordem (entrada) - a ordem do termo
 *
 * Retorno: Nada
 *
 ****/
void ExibeTermo(const tTermo *termo, int ordem)
{
      /* Se o termo no for o primeiro, */
      /* apresenta o sinal do termo     */
   if (ordem > 1) {
      if (termo->coef > 0) {
         printf(" + ");
      } else {
         printf(" - ");
      }
   }

      /* Escreve o termo independente */
   if (termo->exp == 0) {
      printf("%d", ordem > 1 ? ABS(termo->coef) : termo->coef);
      return;
   }

      /* Escreve o termo linear */
   if (termo->exp == 1) {
      if (termo->coef == 1) {
         putchar('x');
      } else if (termo->coef == -1) {
         printf("%sx", ordem > 1 ? "" : "-");
      } else {
         printf( "%dx", ordem > 1 ? ABS(termo->coef)
                                 : termo->coef );
      }
      return;
   }

      /* Neste ponto, o expoente deve ser maior do 1 */
   ASSEGURA( termo->exp > 1,
             "Encontrado um expoente estranho" );

      /* Escreve os demais termos */
   if (termo->coef == 1) {
      printf("x^%d", termo->exp);
   } else if (termo->coef == -1) {
      printf("%sx^%d", ordem > 1 ? "" : "-", termo->exp);
   } else {
      printf( "%dx^%d", ordem > 1 ? ABS(termo->coef)
                                 : termo->coef, termo->exp );
   }
}

/****
 *
 * ExibePolinomio(): Exibe um polinmio na tela
 *
 * Parmetros:
 *      poli (entrada) - o polinmio que ser exibido
 *
 * Retorno: Nada
 *
 ****/
void ExibePolinomio(const tPoli *poli)
{
   int i = 1; /* Indica a ordem do termo */

   ASSEGURA(poli, "\nO polinomio est vazio");

      /* Exibe os termos */
   while (poli) {
      ExibeTermo(&poli->termo, i);

      ++i;

      poli = poli->proximo;
   }
}

/****
 *
 * DestroiPolinomio(): Destroi um polinmio representado como
 *                     uma lista simplesmente encadeada
 *
 * Parmetros:
 *     *poli (entrada) - o polinmio que ser destrudo
 *
 * Retorno: Nada
 *
 ****/
void DestroiPolinomio(tPoliPtr *poli)
{
   tPoliPtr p; /* Aponta para o prximo n a ser liberado */

   if (!*poli) { /* Verifica se a lista est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *poli; /* Faz p apontar para o incio da lista */

      /* Visita cada n da lista liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /*o incio da lista seja alterado. Afinal, este    */
         /* ponteiro ser mesmo anulado.                    */
      *poli = (*poli)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *lista    */
      p = *poli;
   } while (p);

      /* Neste ponto, a lista ficou vazia, pois o  */
      /* ltimo valor assumido por *lista foi NULL */
}

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros: Nenhum
 *
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(void)
{
   printf( "\n\t[S]oma"
           "\n\tSu[b]tracao"
           "\n\t[V]alor numerico [P(a)]"
           "\n\t[G]rau"
           "\n\t[R]ecomeca"
           "\n\t[E]ncerra programa\n" );
}

/********************* Funo main() **********************/

/****
 *
 * main(): Testa operaes sobre polinmios representados
 *         com listas encadeadas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int      op; /* Opo escolhida pelo usurio */
   tPoliPtr poli1, poli2, /* Dois nmeros polinmios */
            resultado;  /* Resultado de uma operao */
                        /* com polinmios            */
   double a; /* 'a' em tratamento de polinmios em Matemtica! */

      /* Apresenta o programa ao usurio */
      /* e explica seu funcionamento     */
   printf( "\n\t>>> Este programa le polinomios e efetua "
           "\n\t>>> operacoes sobre eles. Digite zero como"
           "\n\t>>> coeficiente para encerrar a introducao"
           "\n\t>>> de cada polinomio.\n" );

inicio: /* Retorna para c se o usurio desejar recomear */

      /* L o primeiro polinmio */
   printf("\n\t>>> Digite o primeiro polinomio <<<\n");
   poli1 = LePolinomio();

      /* L o segundo polinmio */
   printf("\n\t>>> Digite o segundo polinomio <<<\n");
   poli2 = LePolinomio();

      /* Lao principal do programa */
   while (1) {
         /*                         */
         /* Apresenta os polinmios */
         /*                         */

      printf("\n\n>>> P(x) = ");
      ExibePolinomio(poli1);

      printf("\n>>> Q(x) = ");
      ExibePolinomio(poli2);
      putchar('\n');

         /*                                          */
         /* Apresenta o menu e l a opo do usurio */
         /*                                          */

      ApresentaMenu();

      printf("\n>>> Escolha sua opcao: ");
      op = LeOpcao("SBVGREsbvgre");

         /* Usurio quer encerrar o programa */
      if (op == 'E' || op == 'e') {
         break;
      }

         /* Processa as demais opes */
      switch (op) {
      case 'S': /* Soma */
      case 's':
            /* Obtm o resultado da soma */
         resultado = SomaPolinomios(poli1, poli2);

            /* Apresenta o resultado */
         printf("\n\t>>> P(x) + Q(x) =  ");
         ExibePolinomio(resultado);

            /* Libera o espao ocupado pelo resultado */
         DestroiPolinomio(&resultado);
         break;

      case 'B': /* Subtrao */
      case 'b':
            /* Obtm o resultado da subtrao */
         resultado = SubtraiPolinomios(poli1, poli2);

            /* Apresenta o resultado */
         printf("\n\t>>>  P(x) - Q(x) = ");
         ExibePolinomio(resultado);

            /* Libera o espao ocupado pelo resultado */
         DestroiPolinomio(&resultado);
         break;

      case 'V': /* Valor numrico */
      case 'v':
            /* Obtm o valor de 'a' */
         printf("\n>>> Introduza o valor de 'a': ");
         a = LeReal();

            /* Apresenta o resultado */
         printf( "\n\t>>> P(%3.2f) = %3.2f", a,
                 ValorNumerico(poli1, a) );
         printf( "\n\t>>> Q(%3.2f) = %3.2f", a,
                 ValorNumerico(poli2, a) );
         break;

      case 'G': /* Graus dos polinmios */
      case 'g':
            /* Apresenta o resultado */
         printf( "\n\t>>> Grau(P(x)) = %d", Grau(poli1) );
         printf( "\n\t>>> Grau(Q(x)) = %d", Grau(poli2) );

         break;

      case 'R': /* Usurio deseja recomear */
      case 'r':
            /* Libera o espao ocupado pelos polinmios  */
            /* antigos, pois o usurio deseja introduzir */
            /* novos polinmios                          */
         DestroiPolinomio(&poli1);
         DestroiPolinomio(&poli2);

         goto inicio; /* Sem espaguete */
         break;

      default:
         printf("\nIsto nao deveria ser exibido.\n");
         break;
      }
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa le polinomios e efetua
        >>> operacoes sobre eles. Digite zero como
        >>> coeficiente para encerrar a introducao
        >>> de cada polinomio.

        >>> Digite o primeiro polinomio <<<

        Coeficiente: 1
        Expoente: 4

        Coeficiente: -2
        Expoente: 3

        Coeficiente: 1
        Expoente: 1

        Coeficiente: -1
        Expoente: 0

        Coeficiente: 0

        >>> Digite o segundo polinomio <<<

        Coeficiente: 1
        Expoente: 2

        Coeficiente: -1
        Expoente: 1

        Coeficiente: 1
        Expoente: 0

        Coeficiente: 0


>>> P(x) = x^4 - 2x^3 + x - 1
>>> Q(x) = x^2 - x + 1

        [S]oma
        Su[b]tracao
        [V]alor numerico [P(a)]
        [G]rau
        [R]ecomeca
        [E]ncerra programa

>>> Escolha sua opcao: s

        >>> P(x) + Q(x) =  x^4 - 2x^3 + x^2

>>> P(x) = x^4 - 2x^3 + x - 1
>>> Q(x) = x^2 - x + 1

        [S]oma
        Su[b]tracao
        [V]alor numerico [P(a)]
        [G]rau
        [R]ecomeca
        [E]ncerra programa

>>> Escolha sua opcao: b

        >>>  P(x) - Q(x) = x^4 - 2x^3 + x^2 + 2x - 2

>>> P(x) = x^4 - 2x^3 + x - 1
>>> Q(x) = x^2 - x + 1

        [S]oma
        Su[b]tracao
        [V]alor numerico [P(a)]
        [G]rau
        [R]ecomeca
        [E]ncerra programa

>>> Escolha sua opcao: v

>>> Introduza o valor de 'a': 2

        >>> P(2.00) = 1.00
        >>> Q(2.00) = 3.00

>>> P(x) = x^4 - 2x^3 + x - 1
>>> Q(x) = x^2 - x + 1

        [S]oma
        Su[b]tracao
        [V]alor numerico [P(a)]
        [G]rau
        [R]ecomeca
        [E]ncerra programa

>>> Escolha sua opcao: g

        >>> Grau(P(x)) = 4
        >>> Grau(Q(x)) = 2

>>> P(x) = x^4 - 2x^3 + x - 1
>>> Q(x) = x^2 - x + 1

        [S]oma
        Su[b]tracao
        [V]alor numerico [P(a)]
        [G]rau
        [R]ecomeca
        [E]ncerra programa

>>> Escolha sua opcao: e

        >>> Obrigado por usar este programa.

**********************************************************/
