/****
 *
 * Ttulo: Nmeros Inteiros de Largura Ilimitada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 11/08/2015
 * ltima modificao: 11/08/2015
 *
 * Descrio: Implementa nmeros inteiros de usando listas
 *            circulares duplamente encadeadas
 *
 * Entrada: Nmeros inteiros e opes de menu
 *
 * Sada: v. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada             */
#include <stdlib.h>  /* Alocao dinmica           */
#include <ctype.h>   /* Classificao de caracteres */
#include <string.h>  /* Funes strXXX() e memXXX() */

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

   /* Tamanho do acrscimo do bloco usado para conter */
   /* uma linha a cada chamada de realloc()           */
#define TAMANHO_BLOCO 256

/****************** Definies de Tipos *******************/

   /* Tipo do sinal de um nmero */
typedef enum {MAIS, MENOS} tSinal;

   /* Tipo do resultado da comparao de */
   /* dois nmeros inteiros ilimitados   */
typedef enum {IGUAL, MENOR, MAIOR} tComparacao;

   /* Tipo de n e ponteiro para n da lista encadeada */
   /* que armazena um nmero inteiro ilimitado         */
typedef struct no {
           char       digito;
           struct no *anterior, *proximo;
        } tDigitos, *tDigitosPtr;

   /* Tipo de varivel que representa */
   /* um nmero inteiro ilimitado     */
typedef struct {
           tSinal      sinal;
           tDigitosPtr valor;
           long long   nDigitos;
        } tNumero, *tNumeroPtr;

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeCaractere(void);
extern int LeOpcao(const char *opcoes);
extern char *LeLinhaIlimitada(int *tam, FILE *stream);

extern void AcrescentaDigitoInicio(tNumero *numero, int digito);
extern void AcrescentaDigitoFim(tNumero *numero, int digito);
extern void DestroiNumero(tNumero *numero);
extern void DestroiListaDEC(tDigitosPtr *lista);
extern tNumeroPtr CriaInteiro(const char *str);
extern int SomaDigitos(int digito1, int digito2, int *sobra);
extern tNumeroPtr SomaInteiros( const tNumero *num1,
                                const tNumero *num2 );
extern void RemoveZerosIniciais(tNumero *num);
extern int SubtraiDigitos( int digito1, int digito2,
                           int *sobra );
extern tNumeroPtr SubtraiMenor( const tNumero *num1,
                                const tNumero *num2 );
extern tNumeroPtr SubtraiInteiros(tNumero *num1, tNumero *num2);
extern tComparacao ComparaModulos( const tNumero *num1,
                                   const tNumero *num2 );
extern tComparacao ComparaInteiros( const tNumero *num1,
                                    const tNumero *num2 );
extern void ExibeNumero(const tNumero *numero);
extern void ApresentaMenu(void);


/***************** Definies de Funes ******************/

/****************** Funes de Leitura ********************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>> Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

/****
 *
 * LeLinhaIlimitada(): L uma linha de tamanho arbitrrio num
 *                     stream de texto e armazena os caracteres
 *                     lidos num array alocado dinamicamente
 *
 * Parmetros:
 *      tam (sada) - se no for NULL, apontar para uma varivel
 *                    que armazenar o tamanho do string
 *                    constitudo pelos caracteres da linha
 *      stream (entrada) - stream de texto no qual ser feita a
 *                         leitura
 *
 * Retorno: Endereo do array contendo a linha lida. NULL, se
 *          ocorrer erro ou o final do arquivo for atingido
 *          antes da leitura de qualquer caractere
 *
 * Observaes:
 *       1. O stream deve estar associado a um arquivo de texto
 *          aberto em modo de texto que permita leitura
 *       2. O caractere '\n' no  includo no string resultante
 *          da leitura
 *       3. O primeiro parmetro pode ser NULL. Nesse caso, o
 *          tamanho do string no ser armazenado
 *
 ****/
char *LeLinhaIlimitada(int *tam, FILE *stream)
{
   char *ar = NULL,   /* Ponteiro para um array alocado */
                      /* dinamicamente que conter os   */
                      /* caracteres lidos               */
        *p; /* Usado em chamada de realloc() */
   int   tamanho = 0, /* Tamanho do array alocado */
         c,  /* Armazenar cada caractere lido */
         i; /* ndice do prximo caractere */
            /* a ser inserido no array     */

      /* L caracteres a partir da posio corrente do      */
      /* indicador de posio do arquivo e armazena-os num  */
      /* array. A leitura encerra quando '\n'  encontrado, */
      /* o final do arquivo  atingido ou ocorre erro.      */
   for (i = 0; ; ++i) {
         /* L o prximo caractere no arquivo */
      c = fgetc(stream);

         /* Se ocorreu erro de leitura, libera o  */
         /* bloco eventualmente alocado e retorna */
      if (ferror(stream)) {
         free(ar); /* Libera o bloco apontado por 'ar' */
         return NULL; /* Ocorreu erro de leitura */
      }

         /* Verifica se array est completo. O maior valor que i*/
         /* poderia assumir deveria ser tamanho - 1. Mas, como  */
         /* ao final, o caractere '\0' dever ser inserido,     */
         /* limita-se o valor de i a tamanho - 2.               */
      if (i > tamanho - 2) { /* Limite atingido */
            /* Tenta redimensionar o array */
         p = realloc(ar, tamanho + TAMANHO_BLOCO);

            /* Se o redimensionamento no foi     */
            /* possvel, libera o bloco e retorna */
         if (!p) {
            free(ar); /* Libera o bloco apontado por 'ar' */
            return NULL; /* Ocorreu erro de alocao */
         }

            /* Redimensionamento foi OK. Ento, faz-se */
            /* 'ar' apontar para o novo bloco.         */
         ar = p;

            /* O array aumentou de tamanho */
         tamanho = tamanho + TAMANHO_BLOCO;
      }

         /* Se final do arquivo for atingido ou o caractere */
         /* '\n' for lido,  encerra-se a leitura            */
      if (feof(stream) || c == '\n') {
         break; /* Leitura encerrada */
      }

      ar[i] = c; /* Acrescenta o ltimo caractere lido ao array */
   }

      /* Se nenhum caractere foi lido, libera */
      /* o espao alocado e retorna NULL      */
   if (feof(stream) && !i) {
      free(ar); /* Libera o bloco apontado por 'ar' */
      return NULL; /* Nenhum caractere foi armazenado no array */
   }

      /* Insere o caractere terminal de string. Neste    */
      /* instante, deve haver espao para ele porque o   */
      /* array foi sempre redimensionado deixando um     */
      /* espao a mais para o onipresente caractere '\0' */
   ar[i] = '\0';

      /* Atualiza o valor apontando pelo parmetro */
      /* 'tam', se ele no for NULL                */
   if (tam) {
         /* i  o ndice do caractere terminal do */
         /* string e corresponde ao seu tamanho   */
      *tam = i;
   }

            /*                                     */
            /* >>> NB: O tamanho do string no <<< */
            /* >>>     inclui o caractere '\0' <<< */
            /*                                     */

      /* Tenta ajustar o tamanho do array para no */
      /* haver desperdcio de memria. Como i  o  */
      /* tamanho do string, o tamanho do array que */
      /* o contm deve ser i + 1.                  */
   p = realloc(ar, i + 1);

      /*                                                 */
      /* Se a realocao foi bem sucedida, retorna-se p. */
      /* Caso contrrio, 'ar' ainda aponta para um bloco */
      /* vlido. Talvez, haja desperdcio de memria,    */
      /* mas, aqui,  melhor retornar 'ar' do que NULL.  */
      /*                                                 */
   return p ? p : ar;
}

/****** Implementao de Nmeros Inteiros Ilimitados ******/

/****
 *
 * AcrescentaDigitoInicio(): Acrescenta um dgito no incio da
 *                           lista encadeada que representa um
 *                           nmero inteiro ilimitado
 *
 * Parmetros:
 *     *numero (entrada e sada) - o nmero que ter um dgito
 *                                 acrescentado
 *      digito (entrada) - o dgito que ser acrescentado ao
 *                         nmero
 *
 * Retorno: Nada
 *
 * Observao: Se ocorrer falha de alocao, o programa
 *             ser abortado
 *
 ****/
void AcrescentaDigitoInicio(tNumero *numero, int digito)
{
   tDigitos *ptrNoDigito;

      /* Tenta alocar o novo n */
   ASSEGURA( ptrNoDigito = malloc(sizeof(tDigitos)),
             "Nao foi possivel alocar no' da lista" );

      /* Armazena o dgito no novo n */
   ptrNoDigito->digito = digito;

      /* Insere novo n no incio da lista */
   if (!numero->valor) { /* Primeiro dgito da lista */
         /* O n anterior e o prximo n */
         /* do novo n sero ele prprio */
      ptrNoDigito->anterior = ptrNoDigito;
      ptrNoDigito->proximo = ptrNoDigito;
   } else { /* A lista no est vazia */
         /* O n adiante do novo n ser p */
      ptrNoDigito->proximo = numero->valor;

         /* O n anterior ao novo n ser o ltimo n */
      ptrNoDigito->anterior = numero->valor->anterior;

         /* O prximo n do ltimo n  o primeiro da */
         /* lista, que  o n sendo inserido          */
      numero->valor->anterior->proximo = ptrNoDigito;

         /* O novo n  anterior ao n apontado por p */
      numero->valor->anterior = ptrNoDigito;
   }

   ++numero->nDigitos; /* Nmero de dgitos aumentou */

      /* O novo n ser o primeiro da lista */
   numero->valor = ptrNoDigito;
}

/****
 *
 * AcrescentaDigitoFim(): Acrescenta um dgito no final da
 *                        lista encadeada que representa um
 *                        nmero inteiro ilimitado
 *
 * Parmetros:
 *     *numero (entrada e sada) - o nmero que ter um dgito
 *                                 acrescentado
 *      digito (entrada) - o dgito que ser acrescentado ao
 *                         nmero
 *
 * Retorno: Nada
 *
 * Observao: Se ocorrer falha de alocao, o programa
 *             ser abortado
 *
 ****/
void AcrescentaDigitoFim(tNumero *numero, int digito)
{
   tDigitos *ptrNoDigito;

      /* Tenta alocar o novo n */
   ASSEGURA( ptrNoDigito = malloc(sizeof(tDigitos)),
             "Nao foi possivel alocar no' da lista" );

      /* Armazena o dgito no novo n */
   ptrNoDigito->digito = digito;

      /* Insere novo n no final da lista */
   if (!numero->valor) { /* Primeiro dgito da lista */
         /* O n anterior e o prximo n */
         /* do novo n sero ele prprio */
      ptrNoDigito->anterior = ptrNoDigito;
      ptrNoDigito->proximo = ptrNoDigito;

         /* Ponteiro que representa a lista */
         /* apontar para o novo n         */
      numero->valor = ptrNoDigito;
   } else { /* A lista no est vazia */
         /* O n adiante do novo n ser o primeiro da lista */
      ptrNoDigito->proximo = numero->valor;

         /* O n anterior ao novo n ser o ltimo n atual */
      ptrNoDigito->anterior = numero->valor->anterior;

         /* O n adiante do ltimo n atual ser o novo n */
      numero->valor->anterior->proximo = ptrNoDigito;

         /* O n anterior do primeiro n ser o novo n */
      numero->valor->anterior = ptrNoDigito;
   }

   ++numero->nDigitos; /* Nmero de dgitos aumentou */
}

/****
 *
 * DestroiListaDEC(): Libera o espao ocupado pelos ns de
 *                    uma lista circular duplamente encadeada
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a lista
 *                             encadeada que ser destruda
 * Retorno: Nada
 *
 ****/
void DestroiListaDEC(tDigitosPtr *lista)
{
   tDigitosPtr p, /* Aponta para o prximo n a ser liberado */
               ultimo; /* Aponta para o ltimo n da lista */

   if (!*lista) { /* Verifica se a lista est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *lista; /* Faz p apontar para o incio da lista */

      /* Faz 'ultimo' apontar para o ltimo n da lista */
   ultimo = (*lista)->anterior;


      /* Visita cada n da lista liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /*o incio da lista seja alterado. Afinal, este    */
         /* ponteiro ser mesmo anulado.                    */
      *lista = (*lista)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *lista    */
      p = *lista;
   } while (p != ultimo);

   free(p); /* Faltou liberar o ltimo n */

   *lista = NULL; /* A lista ficou vazia */
}

/****
 *
 * DestroiNumero(): Libera o espao ocupado por um
 *                  nmero inteiro ilimitado
 *
 * Parmetros:
 *     *numero (entrada e sada) - o nmero cujo espao
 *                                 ser liberado
 *
 * Retorno: Nada
 *
 ****/
void DestroiNumero(tNumero *numero)
{
   DestroiListaDEC(&numero->valor);

   free(numero);
}

/****
 *
 * CriaInteiro(): Cria um nmero inteiro ilimitado
 *
 * Parmetros:
 *      str (entrada) - string que dever conter apenas espaos
 *                      em branco no incio, um sinal e dgitos
 *
 * Retorno: O nmero ilimitado criado se no for encontrado
 *          nenhum caractere esprio no string; NULL, em caso
 *          contrrio
 *
 * Observao: Se houver falha de alocao, o programa
 *             ser abortado
 *
 ****/
tNumeroPtr CriaInteiro(const char *str)
{
   tNumero *ptrNumero;

      /* Verifica se o string  vazio */
   if (!*str) {
      return NULL; /* O string  vazio */
   }

      /* Tenta alocar espao para a estrutura que */
      /* contm as informaes sobre o nmero     */
   ASSEGURA( ptrNumero = malloc(sizeof(tNumero)),
             "Nao foi possivel alocar numero" );

      /* A lista que contm os dgitos ainda est vazia */
   ptrNumero->valor = NULL;
   ptrNumero->nDigitos = 0;

      /* Salta espaos em branco iniciais */
   while (*str && isspace(*str)) {
      ++str;
   }

      /* Obtm o sinal do nmero */
   if (*str == '-') {
      ptrNumero->sinal = MENOS;
      str++;
   } else if (*str == '+') {
      ptrNumero->sinal = MAIS;
      str++;
   } else {
      ptrNumero->sinal = MAIS;
   }

      /* Salta zeros iniciais */
   while (*str && *str == '0') {
      ++str;
   }

      /* S foram encontrados zeros */
   if (!*str && str[-1] == '0') {
      str--;
      ptrNumero->sinal = MAIS;
   }

      /* Acessa cada caractere do string e armazena os    */
      /* dgitos na lista que representar o nmero. Se   */
      /* for encontrado algum caractere que no  dgito, */
      /* libera o espao alocado e retorna NULL.          */
   while (*str) {
         /* Verifica se o caractere corrente  dgito */
      if (isdigit(*str)) {
            /* Acrescenta novo n contendo o dgito ao */
            /* final da lista que representa o nmero  */
         AcrescentaDigitoFim(ptrNumero, *str);

         ++str; /* Passa para o prximo caractere */
      } else { /* Encontrado um caractere que no  dgito */
            /* Libera o espao alocado at ento */
         DestroiNumero(ptrNumero);

         return NULL; /* Criao do nmero falhou */
      }
   }

   return ptrNumero;  /* Nmero foi criado com sucesso */
}

/****
 *
 * SomaDigitos(): Soma dois dgitos que fazem parte
 *                de um nmero inteiro ilimitado
 *
 * Parmetros:
 *      digito1 (entrada) - o primeiro digito
 *      digito2 (entrada) - o segundo digito
 *      *sobra (entrada e sada) - ponteiro para um valor a
 *                                 ser acrescido  soma e que
 *                                 passar a ser acrescido 
 *                                 prxima soma
 *
 * Retorno: O resultado da soma
 *
 ****/
int SomaDigitos(int digito1, int digito2, int *sobra)
{
      /* A expresso dgito - '0' resulta no nmero  */
      /* inteiro que o dgito representa em qualquer */
      /* cdigo de caracteres                        */
   int valor1 = digito1 - '0',
       valor2 = digito2 - '0',
       resultado;

      /* A varivel 'sobra' aponta para */
      /* o que sobrou da ltima soma    */
   resultado = valor1 + valor2 + *sobra;

      /* Se o resultado for maior do que */
      /* 9 sobra 1 para a prxima soma   */
   *sobra = resultado > 9 ? 1 : 0;

      /* Se 0 =< nmero <= 9, nmero + '0' resulta */
      /* no dgito que representa o nmero         */
   return resultado%10 + '0';
}

/****
 *
 * SomaInteiros(): Soma dois nmeros inteiros ilimitados
 *
 * Parmetros:
 *     *num1 (entrada) - primeira parcela
 *     *num2 (entrada) - segunda parcela
 *
 * Retorno: O resultado da soma
 *
 ****/
tNumeroPtr SomaInteiros( const tNumero *num1,
                         const tNumero *num2 )
{
   tSinal      oSinal = MAIS; /* Sinal do resultado */
   tDigitosPtr pDigitos1, /* Aponta para um n de num1 */
               pDigitos2; /* Aponta para um n de num2 */
   tNumero    *resultado; /* Ponteiro para o resultado */
   int         sobra = 0, /* Armazena o 'vai um' */
               digitoParcial, /* A soma de dois dgitos */
               resultadoZero = 0; /* Informa se o     */
                                  /* resultado  zero */

      /********************************************/
      /*                                          */
      /* A operao a ser efetuada depende do     */
      /* sinal e do valor absoluto de cada nmero */
      /*                                          */
      /********************************************/

   if (num1->sinal == MAIS && num2->sinal == MENOS) {
         /* O primeiro nmero  positivo  */
         /* e o segundo nmero  negativo */
      if (ComparaModulos(num1, num2) == MENOR) {
            /* O primeiro nmero  menor do que o segundo */
            /* em valores absolutos. Subtrai o primeiro   */
            /* nmero do segundo e atribui sinal negativo */
            /* ao resultado.                              */
         resultado = SubtraiMenor(num2, num1);
         resultado->sinal = MENOS;
         return resultado;
      } else if (ComparaModulos(num1, num2) == MAIOR) {
            /* O primeiro nmero  maior do que o     */
            /* segundo em valores absolutos. A funo */
            /* SubtraiMenor() resolve esse problema.  */
         return SubtraiMenor(num1, num2);
      } else {
            /* Os dois nmeros so iguais em valores  */
            /* valores absolutos e tm sinais opostos */
         resultadoZero = 1;
      }
   } else if (num1->sinal == MENOS && num2->sinal == MAIS) {
         /* O primeiro nmero  negativo  */
         /* e o segundo nmero  positivo */
      if (ComparaModulos(num1, num2) == MENOR) {
            /* O primeiro nmero  menor do que o     */
            /* segundo em valores absolutos. A funo */
            /* SubtraiMenor() resolve esse problema.  */
         return SubtraiMenor(num2, num1);
      } else if (ComparaModulos(num1, num2) == MAIOR) {
            /* O primeiro nmero  maior do que o segundo  */
            /* em valores absolutos. Subtrai o segundo     */
            /* nmero do primeiro e atribui sinal negativo */
            /* ao resultado.                               */
         resultado = SubtraiMenor(num1, num2);
         resultado->sinal = MENOS;
         return resultado;
      } else {
            /* Os dois nmeros so iguais em valores  */
            /* valores absolutos e tm sinais opostos */
         resultadoZero = 1;
      }
   } else if (num1->sinal == MENOS && num2->sinal == MENOS) {
         /* Os dois nmeros so negativos. Portanto, eles */
         /* sero somados e o resultado ser negativo     */
      oSinal = MENOS;
   }

      /*****************************************/
      /*                                       */
      /* >>> A verdadeira soma comea aqui <<< */
      /*                                       */
      /*****************************************/

      /* Tenta alocar espao para a estrutura que */
      /* contm as informaes sobre o resultado  */
   ASSEGURA( resultado = malloc(sizeof(tNumero)),
             "Nao foi possivel alocar numero" );

      /* O sinal do resultado foi obtido acima */
   resultado->sinal = oSinal;

      /* Inicialmente, o resultado no contm digito nenhum */
   resultado->valor = NULL;
   resultado->nDigitos = 0;

      /* Se o resultado for 0, no h o que calcular */
   if (resultadoZero) {
      AcrescentaDigitoInicio(resultado, '0');
      return resultado;
   }

      /* Faz os ponteiros apontarem para os dgitos menos */
      /* significativos, que so os ltimos das listas    */
   pDigitos1 = num1->valor->anterior;
   pDigitos2 = num2->valor->anterior;

      /* Percorre as listas a partir de seus finais at */
      /* atingir o incio de alguma delas ou ambas      */
   do {
         /* Soma os dgitos correspondentes */
      digitoParcial = SomaDigitos( pDigitos1->digito,
                                   pDigitos2->digito, &sobra );

         /* Acrescenta o resultado da ltima */
         /* soma de dgitos ao resultado     */
      AcrescentaDigitoInicio(resultado, digitoParcial);

         /* Passa para os prximos dgitos */
      pDigitos1 = pDigitos1->anterior;
      pDigitos2 = pDigitos2->anterior;
   } while (pDigitos1 != num1->valor->anterior &&
                         pDigitos2 != num2->valor->anterior);

      /* Talvez alguns dgitos do primeiro nmero no tenham */
      /* sido levados em considerao no lao acima          */
   while (pDigitos1 != num1->valor->anterior) {
         /* Soma um dgito que sobrou com zero */
      digitoParcial = SomaDigitos( pDigitos1->digito, '0',
                                   &sobra );

         /* Acrescenta o resultado da ltima */
         /* soma de dgitos ao resultado     */
      AcrescentaDigitoInicio(resultado, digitoParcial);

         /* Passa para o prximo dgito do primeiro nmero */
      pDigitos1 = pDigitos1->anterior;
   }

      /* Talvez alguns dgitos do segundo nmero no tenham */
      /* sido levados em considerao no lao acima          */
   while (pDigitos2 != num2->valor->anterior) {
         /* Soma um dgito que sobrou com zero */
      digitoParcial = SomaDigitos( '0', pDigitos2->digito,
                                   &sobra );

         /* Acrescenta o resultado da ltima */
         /* soma de dgitos ao resultado     */
      AcrescentaDigitoInicio(resultado, digitoParcial);

         /* Passa para o prximo dgito do segundo nmero */
      pDigitos2 = pDigitos2->anterior;
   }

      /* Se houver 'mais um' para acrescentar, acrescenta-o */
   if (sobra) {
      AcrescentaDigitoInicio(resultado, '1');
   }

   return resultado; /* Misso concluda */
}

/****
 *
 * RemoveZerosIniciais(): Remove ns contendo o dgito '0'
 *                        que aparecem no incio de uma lista
 *                        que representa um inteiro ilimitado
 *
 * Parmetros:
 *    *num (entrada e sada) - nmero inteiro ilimitado que ter
 *                             seus zeros iniciais removidos
 *
 * Retorno: Nada
 *
 ****/
void RemoveZerosIniciais(tNumero *num)
{
   tDigitos *p, /* Apontar para o n ora examinado */
            *ultimo; /* Apontar para o ltimo n da lista */

   p = num->valor; /* Faz p apontar para o primeiro n */
   ultimo = num->valor->anterior; /* Faz 'ultimo' apontar */
                                  /* para o ltimo n     */

      /* Enquanto p no estiver apontando para o ltimo n */
      /* e o n corrente contiver '0', remove esse n      */
   while (p != ultimo && p->digito == '0') {
         /* Segundo n passa a ser o primeiro */
      num->valor = num->valor->proximo;
      num->valor->anterior = ultimo;
      ultimo->proximo = num->valor;

      --num->nDigitos; /* A lista ter um n a menos */

      free(p); /* Libera espao ocupado pelo primeiro n */

      p = p->proximo; /* Passa para o prximo n */
   }
}

/****
 *
 * SubtraiDigitos(): Subtrai dois dgitos que fazem parte
 *                   de um nmero inteiro ilimitado
 *
 * Parmetros:
 *      digito1 (entrada) - o primeiro digito
 *      digito2 (entrada) - o segundo digito
 *      *sobra (entrada e sada) - ponteiro para um valor a
 *                                 ser subtrado do resultado
 *                                 e que passar a ser subtrado
 *                                 da prxima subtrao
 *
 * Retorno: O resultado de digito1 - digito2
 *
 ****/
int SubtraiDigitos(int digito1, int digito2, int *sobra)
{
      /* A expresso dgito - '0' resulta no nmero  */
      /* inteiro que o dgito representa em qualquer */
      /* cdigo de caracteres                        */
   int valor1 = digito1 - '0', /* Inteiros representados */
       valor2 = digito2 - '0', /* por digito1 e digito2  */
       resultado;

      /* A operao depende de qual dos dgitos  maior */
   if (valor1 >= valor2) {
      resultado = valor1 - valor2 - *sobra;
      *sobra = 0;
   } else {
        /* Quando o segundo valor  maior do que o primeiro,  */
        /* soma-se 10 ao primeiro antes de subtrair e sobra 1 */
      resultado = 10 + valor1 - valor2 - *sobra;
      *sobra = 1;
   }

      /* Se 0 =< nmero <= 9, nmero + '0' resulta */
      /* no dgito que representa o nmero         */
   return resultado%10 + '0';
}

/****
 *
 * SubtraiMenor(): Subtrai um nmero inteiro ilimitado menor
 *                 de um nmero inteiro ilimitado maior
 *
 * Parmetros:
 *      num1 (entrada) - o minuendo
 *      num2 (entrada) - o sustraendo
 *
 * Retorno: O resultado da subtrao
 *
 * Observao: Assume-se que segundo nmero  sempre menor
 *             do que o primeiro nmero
 *
 ****/
tNumeroPtr SubtraiMenor( const tNumero *num1,
                         const tNumero *num2 )
{
   tDigitosPtr pDigitos1, /* Aponta para um n de num1 */
               pDigitos2; /* Aponta para um n de num2 */
   tNumero    *resultado; /* Ponteiro para o resultado */
   int         sobra = 0, /* Armazena o 'vai um' */
               digitoParcial; /* A diferena de dois dgitos */

      /* Tenta alocar espao para a estrutura que */
      /* contm as informaes sobre o resultado  */
   ASSEGURA( resultado = malloc(sizeof(tNumero)),
             "Nao foi possivel alocar numero" );

      /* Como o segundo nmero  sempre menor do que o   */
      /* primeiro o sinal do resultado  sempre positivo */
   resultado->sinal = MAIS;

      /* O resultado ainda no possui nenhum dgito */
   resultado->valor = NULL;
   resultado->nDigitos = 0;

      /* Faz os ponteiros apontarem para os dgitos menos */
      /* significativos, que so os ltimos das listas    */
   pDigitos1 = num1->valor->anterior;
   pDigitos2 = num2->valor->anterior;

      /* Percorre as listas a partir de seus finais at */
      /* atingir o incio de alguma delas ou ambas      */
   do {
         /* Subtrai os dgitos correspondentes */
      digitoParcial = SubtraiDigitos(pDigitos1->digito,
                                     pDigitos2->digito,
                                     &sobra);

         /* Acrescenta o resultado da ltima  */
         /* subtrao de dgitos ao resultado */
      AcrescentaDigitoInicio(resultado, digitoParcial);

         /* Passa para os prximos dgitos */
      pDigitos1 = pDigitos1->anterior;
      pDigitos2 = pDigitos2->anterior;
   } while (pDigitos1 != num1->valor->anterior &&
                         pDigitos2 != num2->valor->anterior);

      /* Se o nmero de dgitos do primeiro nmero for */
      /* maior do que o nmero de dgitos do segundo   */
      /* nmero, alguns dgitos no primeiro nmero no */
      /* foram ainda levados em considerao           */
   while (pDigitos1 != num1->valor->anterior) {
         /* Subtrai zero de um dgito que sobrou */
      digitoParcial = SubtraiDigitos( pDigitos1->digito, '0',
                                      &sobra );

         /* Acrescenta o resultado da ltima  */
         /* subtrao de dgitos ao resultado */
      AcrescentaDigitoInicio(resultado, digitoParcial);

         /* Passa para o prximo dgito do primeiro nmero */
      pDigitos1 = pDigitos1->anterior;
   }

      /* Remove eventuais zeros iniciais do resultado */
   RemoveZerosIniciais(resultado);

   return resultado; /* Done */
}

/****
 *
 * SubtraiInteiros(): Subtrai dois nmeros inteiros ilimitados
 *
 * Parmetros:
 *      num1 (entrada e sada) - o minuendo
 *      num2 (entrada e sada) - o sustraendo
 *
 * Retorno: O resultado da subtrao
 *
 ****/
tNumeroPtr SubtraiInteiros(tNumero *num1, tNumero *num2)
{
   tNumeroPtr resultado; /* Resultado da operao */

      /* Troca o sinal do segundo nmero e soma */
   num2->sinal = num2->sinal == MAIS ? MENOS : MAIS;
   resultado = SomaInteiros(num1, num2);

      /* Destroca o sinal do segundo nmero */
   num2->sinal = num2->sinal == MAIS ? MENOS : MAIS;

   return resultado;
}

/****
 *
 * ComparaModulos(): Compara os valores absolutos de dois
 *                   nmeros inteiros ilimitados
 *
 * Parmetros:
 *      num1 (entrada) - o primeiro nmero
 *      num2 (entrada) - o segundo nmero
 *
 * Retorno: Uma das constantes a seguir:
 *    * IGUAL - se os nmeros forem iguais
 *    * MENOR - se o primeiro nmero for menor do que o segundo
 *    * MAIOR - se o primeiro nmero for maior do que o segundo
 *
 * Observao: Assume-se que os nmeros a ser comparados no
 *             possuem zeros  esquerda
 *
 ****/
tComparacao ComparaModulos( const tNumero *num1,
                            const tNumero *num2 )
{
   tDigitosPtr p1, p2;

      /*****************************************/
      /*                                       */
      /* Essa funo s funciona se os nmeros */
      /* no tiverem zeros  esquerda          */
      /*                                       */
      /*****************************************/

      /* O nmero que possuir mais dgitos  o maior */
   if (num1->nDigitos > num2->nDigitos) {
      return MAIOR;
   } else if (num1->nDigitos < num2->nDigitos) {
      return MENOR;
   }

      /*                                                  */
      /* Os nmeros possuem a mesma quantidade de dgitos */
      /*                                                  */

      /* p1 e p2 apontaro para ns das listas */
      /* que representam os nmeros            */
   p1 = num1->valor; /* p1 passa a apontar para o */
                     /* primeiro dgito de num1   */
   p2 = num2->valor; /* p2 passa a apontar para o */
                     /* primeiro dgito de num2   */

      /* Compara dgitos corresondentes nas duas listas */
   do {
         /* Se for encontrado um dgito maior do que o */
         /* outro, o nmero correspondente ser maior; */
         /* se for encontrado um dgito menor do que o */
         /* outro, o nmero correspondente ser menor; */
         /* caso contrrio, as comparaes continuam   */
      if (p1->digito > p2->digito) {
         return MAIOR;
      } else if (p1->digito < p2->digito) {
         return MENOR;
      }

         /* Passa para os prximos dgitos */
      p1 = p1->proximo;
      p2 = p2->proximo;
   } while (p1 != num1->valor && p2 != num2->valor);

      /* Se no ocorreu retorno no corpo */
      /* do lao, os nmeros so iguais  */
   return IGUAL;
}

/****
 *
 * ComparaInteiros(): Compara dois nmeros inteiros ilimitados
 *
 * Parmetros:
 *      num1 (entrada) - o primeiro nmero
 *      num2 (entrada) - o segundo nmero
 *
 * Retorno: Uma das constantes a seguir:
 *    * IGUAL - se os nmeros forem iguais
 *    * MENOR - se o primeiro nmero for menor do que o segundo
 *    * MAIOR - se o primeiro nmero for maior do que o segundo
 *
 ****/
tComparacao ComparaInteiros(const tNumero *num1, const tNumero *num2)
{
   tComparacao compModulos;

      /* O resultado da comparao depende dos sinais */
      /* dos nmeros e de seus valores absolutos      */
   if (num1->sinal == MAIS) { /* O primeiro nmero  positivo */
      if (num2->sinal == MAIS) {
            /* Os dois nmeros so positivos. O resultado */
            /* da comparao  obtido comparando-se seus  */
            /* valores absolutos.                         */
         return ComparaModulos(num1, num2);
      } else {
            /* O primeiro nmero  positivo e o segundo  */
            /* negativo. Logo, o primeiro  o maior.      */
         return MAIOR;
      }
   } else { /* O primeiro nmero  negativo */
      if (num2->sinal == MAIS) {
            /* O primeiro nmero  negativo e o segundo  */
            /* positivo. Logo, o primeiro  o menor.      */
         return MENOR;
      } else { /* Os dois nmeros so negativos */
            /* Compara os valores absolutos dos nmeros */
         compModulos = ComparaModulos(num1, num2);

            /* O nmero que tem o menor valor  */
            /* absoluto  o maior e vice versa */
         if (compModulos == MENOR) {
            return MAIOR;
         } else if (compModulos == MAIOR) {
            return MENOR;
         }

         return IGUAL; /* Os nmeros so iguais */
      }
   }

   ASSEGURA(0, "O programa nao deveria ter chegado ate' aqui");

   return IGUAL;
}

/****
 *
 * ExibeNumero(): Exibe na tela um nmero inteiro ilimitado
 *
 * Parmetros:
 *     *numero (entrada) - o nmero que ser exibido
 *
 * Retorno: Nada
 *
 * Observao: Se a lista que representa o nmero no contiver
 *             nenhum dgito, o programa ser abortado
 *
 ****/
void ExibeNumero(const tNumero *numero)
{
   tDigitosPtr p;

      /* Se o nmero de dgitos do nmero for 0, */
      /* o programa ser abortado                */
   ASSEGURA( numero->nDigitos, "Numero nao contem digitos" );

      /* Se o sinal do nmero for indefinido, */
      /* o programa ser abortado             */
   ASSEGURA( numero->sinal == MENOS || numero->sinal == MAIS,
             "Sinal do numero e' indefinido" );

      /* Se o nmero for negativo, escreve '-' na tela */
   if (numero->sinal == MENOS) {
      putchar('-');
   }

      /* Faz p apontar para o primeiro n */
      /* da lista que representa o nmero */
   p = numero->valor;

      /* No exibe zeros iniciais */
//   while (p->digito == '0') {
//      p = p->proximo;
//
//      if (p == numero->valor) {
//         putchar(p->digito);
//         return;
//      }
//   }

      /* Visita cada n da lista que representa o */
      /* nmero, escrevendo seu contedo na tela  */
   do {
      putchar(p->digito); /* Escreve dgito corrente */
      p = p->proximo; /* Passa para o prximo dgito */
   } while (p != numero->valor);
}

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros: Nenhum
 *
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(void)
{
   printf( "\n\t[S]oma"
           "\n\tSu[b]tracao"
           "\n\t[M]ultiplicacao"
           "\n\t[D]ivisao"
           "\n\t[C]ompara"
           "\n\t[R]ecomeca"
           "\n\t[E]ncerra programa\n" );
}

/****
 *
 * main(): Testa operaes sobre nmeros inteiros ilimitados
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char      *linha; /* Apontar para cada linha lida */
   int        op; /* Opo escolhida pelo usurio */
   tNumeroPtr num1, num2, /* Dois nmeros inteiros ilimitados */
              resultado; /* Resultado de uma operao com */
                         /* nmeros inteiros ilimitados   */
   tComparacao compara; /* Resultado de uma comparao de */
                         /* nmeros inteiros ilimitados   */

      /* Apresenta o programa ao usurio */
      /* e explica seu funcionamento     */
   printf( "\n\t>>> Este programa le um numeros inteiros de "
           "\n\t>>> tamanho arbtrario e efetua operacoes"
           "\n\t>>> sobre eles. Se voce nao digitar um"
           "\n\t>>> numero inteiro quando solicitado"
           "\n\t>>> o programa sera' abortado.\n" );

inicio: /* Retorna para c se o usurio desejar recomear */

      /* L o primeiro nmero */
   printf("\n\t>>> Digite o primeiro inteiro: ");
   linha = LeLinhaIlimitada(NULL, stdin);

      /* Tenta representar o primeiro nmero e no h */
      /* segunda chance se o nmero no for correto   */
   ASSEGURA( num1 = CriaInteiro(linha),
             "Numero nao pode ser representado" );

   free(linha); /* Libera espao ocupado pelo string lido */

      /* L o segundo nmero */
   printf("\n\t>>> Digite o segundo inteiro: ");
   linha = LeLinhaIlimitada(NULL, stdin);

      /* Tenta representar o segundo nmero e no h */
      /* segunda chance se o nmero no for correto  */
   ASSEGURA( num2 = CriaInteiro(linha),
             "Numero nao pode ser representado" );

   free(linha); /* Libera espao ocupado pelo string lido */

      /* Lao principal do programa */
   while (1) {
         /*                      */
         /* Apresenta os nmeros */
         /*                      */

      printf("\n\n\t>>> Numero 1: ");
      ExibeNumero(num1);

      printf("\n\t>>> Numero 2: ");
      ExibeNumero(num2);
      putchar('\n');

         /*                                          */
         /* Apresenta o menu e l a opo do usurio */
         /*                                          */

      ApresentaMenu();

      printf("\n>>> Escolha sua opcao: ");
      op = LeOpcao("SBMDCREsbmdcre");

         /* Usurio quer encerrar o programa */
      if (op == 'E' || op == 'e') {
         break;
      }

         /* Processa as demais opes */
      switch (op) {
      case 'S': /* Soma */
      case 's':
            /* Obtm o resultado da soma */
         resultado = SomaInteiros(num1, num2);

            /* Apresenta o resultado */
         printf("\n\t>>> Soma: ");
         ExibeNumero(resultado);

            /* Libera o espao ocupado pelo resultado */
         DestroiNumero(resultado);
         break;

      case 'B': /* Subtrao */
      case 'b':
            /* Obtm o resultado da subtrao */
         resultado = SubtraiInteiros(num1, num2);

            /* Apresenta o resultado */
         printf("\n\t>>> Subtracao: ");
         ExibeNumero(resultado);

            /* Libera o espao ocupado pelo resultado */
         DestroiNumero(resultado);
         break;

      case 'M': /* Multiplicao */
      case 'm':
            /* Esta opo fica como exerccio */
         printf("\nVoce implementa\n");
         break;

      case 'D': /* Diviso */
      case 'd':
            /* Esta opo fica como exerccio */
         printf("\nVoce implementa\n");
         break;

      case 'C': /* Comparao */
      case 'c':
            /* Obtm o resultado da comparao */
         compara = ComparaInteiros(num1, num2);

            /* Apresenta o resultado da comparao */
         if (compara == IGUAL) {
            printf("\n>>> ");
            ExibeNumero(num1);
            printf(" e' igual a ");
            ExibeNumero(num2);
         } else if (compara == MAIOR) {
            printf("\n>>> ");
            ExibeNumero(num1);
            printf(" e' maior do que ");
            ExibeNumero(num2);
         } else {
            printf("\n>>> ");
            ExibeNumero(num1);
            printf(" e' menor do que ");
            ExibeNumero(num2);
         }
         break;

      case 'R': /* Usurio deseja recomear */
      case 'r':
            /* */
         DestroiNumero(num1);
         DestroiNumero(num2);

         goto inicio; /* Sem espaguete */
         break;

      default:
         printf("\nIsto nao deveria ser exibido.\n");
         break;
      }
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************



**********************************************************/
