/****
 *
 * Exemplo: Listas Generalizadas
 *
 * Sees: 11.2
 *
 * Descrio: Demonstra o uso de listas generalizadas
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada     */
#include <stdlib.h>  /* Funes de alocao dinmica */

/****************** Definies de Tipos *******************/

typedef enum {ATOMO, LISTA} tAtomoOuLista;

typedef union {
           char                atomo;
           struct rotNoListaG *lista;
        } tConteudoListaG;

typedef struct rotNoListaG {
           tAtomoOuLista       indicador;
           tConteudoListaG     conteudo;
           struct rotNoListaG *proximo;
        } tNoListaG, *tListaG;

/************************* Aluses ************************/

extern tListaG CopiaListaG(tListaG);
extern int SaoIguaisListasG(tListaG, tListaG);
extern int ProfListaG(tListaG);
extern void ExibeListaG(tListaG);

/***************** Definies de Funes ******************/

/****
 *
 * CopiaListaG(): Retorna um ponteiro para uma lista que  uma duplicata de L
 *
 * Parmetros:
 *      L (entrada) - aponta para uma lista generalizada no recursiva
 *
 * Retorno: A lista resultante da cpia
 *
 ****/
tListaG CopiaListaG(tListaG  L)
{
   tListaG ptr = NULL;

   if (L) {
      ptr = malloc(sizeof(tNoListaG));
      if (L->indicador == ATOMO)
         ptr->conteudo.atomo = L->conteudo.atomo;
      else
         ptr->conteudo.lista = CopiaListaG(L->conteudo.lista);

      ptr->proximo = CopiaListaG(L->proximo);
      ptr->indicador = L->indicador;
   }

   return ptr;
}

/****
 *
 * SaoIguaisListasG(): Verifica se duas listas generalizadas so iguais
 *
 * Parmetros:
 *      L1, L2 (entrada) - listas generalizadas no-recursivas
 *
 * Retorno: 1 se L1 e L2 so iguais ou 0 caso contrrio
 *
 ****/
int SaoIguaisListasG(tListaG  L1, tListaG  L2)
{
   int resultado = 0;

   if (!L1 && !L2)
      resultado = 1;
   else if (L1 && L2)
      if (L1->indicador == L2->indicador) {
         if (L1->indicador == ATOMO)
             resultado = (L1->conteudo.atomo == L2->conteudo.atomo);
         else
            resultado = SaoIguaisListasG(L1->conteudo.lista, L2->conteudo.lista);

         if (resultado)
            resultado = SaoIguaisListasG(L1->proximo, L2->proximo);
      }

   return resultado;
}

/****
 *
 * ProfListaG(): Calcula a profundidade de uma lista generalizada
 *
 * Parmetros:
 *      L (entrada) - a lista cuja profundidade ser calculada
 *
 * Retorno: A profundidade da lista recebida como parmetro
 *
 ****/
int ProfListaG(tListaG  L)
{
   int     max, resultado;
   tListaG ptr;

   if (!L)
         /* A lista vazia tem profundidade igual a zero */
      return 0;

   max = 0;

   ptr = L;

      /* Calcula a profundidade de cada n */
   while (ptr) {
         /* Profundidade do n corrente */
      if (ptr->indicador == ATOMO)
         resultado = 0;
      else                     /* Idem */
         resultado = ProfListaG(ptr->conteudo.lista);

      if (resultado > max)
         max = resultado; /* Atualiza o maior valor encontrado at aqui */

      ptr = ptr->proximo;
   }

   return max + 1;
}

/****
 *
 * ExibeListaG(): Exibe na tela os ns de uma lista
 *                generalizada no recursiva
 *
 * Parmetros:
 *      p (entrada) - a lista que ser exibida
 *
 * Retorno: Nada
 *
 ****/
void  ExibeListaG(tListaG  p)
{
   if (!p) {
      printf("()");
      return;
   }

   putchar('(');

   while (p)  {
      if (p->indicador == ATOMO)
         putchar(p->conteudo.atomo);
      else
         ExibeListaG(p->conteudo.lista);

      p = p->proximo;
   }

   putchar(')');
}

/****
 *
 * main(): Testa operaes sobre listas generalizAdas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaG lista, ptr1, ptr2, lista2;

   /*                                    */
   /* Construo da lista: A = (a,(b,c)) */
   /*                                    */

   lista = malloc(sizeof(tNoListaG));   /* Cria o primeiro n */
   lista->indicador = ATOMO;   /* Este n contm um tomo */
   lista->conteudo.atomo = 'a';

   ptr1 = malloc(sizeof(tNoListaG)); /* Cria o segundo n */
   lista->proximo = ptr1; /* Faz primeiro n apontar para o segundo */
   ptr1->indicador = LISTA; /* Este n aponta para uma sublista */
   ptr1->proximo = NULL; /* Este n  o ltimo da lista */

   /*                                    */
   /* Criao da sublista do segundo n */
   /*                                    */

   ptr2 = malloc(sizeof(tNoListaG)); /* Cria o primeiro n da sublista */
      /* Este n contm um tomo */
   ptr2->indicador = ATOMO;
   ptr2->conteudo.atomo = 'b';

      /* Faz segundo n da lista apontar para o primeiro n da sublista */
   ptr1->conteudo.lista = ptr2;

   ptr1 = malloc(sizeof(tNoListaG)); /* Cria o segundo n da sublista */
      /* Faz o primeiro n da sublista apontar para o segundo */
   ptr2->proximo = ptr1;

   ptr1->indicador = ATOMO; /* Este n contm um tomo */
   ptr1->conteudo.atomo = 'c';
   ptr1->proximo = NULL; /* Este n  o ltimo da sublista */

   printf("\nPrimeira lista:\n");
   ExibeListaG(lista);

   lista2 = CopiaListaG(lista);

   printf("\n\nSegunda lista:\n");
   ExibeListaG(lista2);

   printf( "\n\nProfundidade da primeira lista: %d", ProfListaG(lista) );
   printf( "\n\nProfundidade da segunda lista: %d\n", ProfListaG(lista2) );

   if (SaoIguaisListasG(lista, lista2))
      printf("\nAs listas sao iguais\n");
   else
      printf("\nAs listas sao diferentes\n");

   return 0;
}

/****************** Resultado do Programa *****************

Primeira lista:
(a(bc))

Segunda lista:
(a(bc))

A profundidade da primeira lista e': 2

A profundidade da segunda lista e': 2

As listas sao iguais

**********************************************************/


