/****
 *
 * Ttulo: Usando bsearch() e qsort()
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

#include <stdlib.h>
#include <stdio.h>

   /* Macro usada para calcular o nmero */
   /* de elementos de um array           */
#define N_ELEMENTOS(ar) (sizeof(ar) / sizeof(ar[0]))

   /* Tipo do ponteiro de funo usado   */
   /* como ltimo argumento de bsearch() */
typedef int (*tFPtr)(const void*, const void*);

/****
 *
 * Compara1(): compara dois inteiros
 *
 * Parmetros:
 *      p1 (entrada) - ponteiro para o primeiro inteiro
 *      p2 (entrada) - ponteiro para o segundo inteiro
 *
 * Retorno:  0, se os inteiros forem iguais
 *           1, se o primeiro inteiro for maior
 *          -1, se o primeiro inteiro for menor
 *
 ****/
int Compara1(const void *p1, const void *p2)
{
   const int *e1 = (const int *)p1; /* Prov interpretao */
   const int *e2 = (const int *)p2; /* para os parmetros  */

      /* Se *e1 < *e2, o retorno ser negativo; */
      /* se *e1 == *e2, o retorno ser 0;       */
      /* se *e1 > *e2, o retorno ser positivo  */
   return *e1 - *e2;
}

/****
 *
 * Compara2(): compara dois inteiros
 *
 * Parmetros:
 *      p1 (entrada) - ponteiro para o primeiro inteiro
 *      p2 (entrada) - ponteiro para o segundo inteiro
 *
 * Retorno:  0, se os inteiros forem iguais
 *           1, se o primeiro inteiro for maior
 *          -1, se o primeiro inteiro for menor
 *
 ****/
int Compara2(const void *p1, const void *p2)
{
   return *(int *)p1 - *(int *)p2;
}

/****
 *
 * Compara3(): compara dois inteiros
 *
 * Parmetros:
 *      p1 (entrada) - ponteiro para o primeiro inteiro
 *      p2 (entrada) - ponteiro para o segundo inteiro
 *
 * Retorno:  0, se os inteiros forem iguais
 *           1, se o primeiro inteiro for maior
 *          -1, se o primeiro inteiro for menor
 *
 ****/
int Compara3(const int *p1, const int *p2)
{
   return *p1 - *p2;
}

/****
 *
 * TestaChave(): verifica se uma chave inteira
 *               faz parte de um array de inteiros
 *
 * Parmetros:
 *      chave (entrada) - a chave a ser testada
 *      ar (entrada) - o array que ser examinado
 *      nElem (entrada) - nmero de elementos do array
 *
 * Retorno: 1, se a chave fizer parte do array
 *          0, se a chave no fizer parte do array
 *
 ****/
int TestaChave(int chave, const int ar[], int nElem)
{
   int *p;

   p = bsearch(&chave, ar, nElem, sizeof(ar[0]), Compara3);

   return (p != NULL);
}

/****
 *
 * main(): Testa a funo bsearch()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int  array[] = {123, 456, 789, 888, 900, 933};

   if (TestaChave(888, array, N_ELEMENTOS(array)))
      printf("888 foi encontrado no array\n");
   else
      printf("888 NAO foi encontrado no array\n");

   if (TestaChave(111, array, N_ELEMENTOS(array)))
      printf("111 foi encontrado no array\n");
   else
      printf("111 NAO foi encontrado no array\n");

   return 0;
}

/****************** Resultado do Programa *****************

888 foi encontrado no array
111 NAO foi encontrado no array

**********************************************************/
