/****
 *
 * Ttulo: Ordenao Generalizada de Listas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Generalizao da implementao do mtodo
 *            da bolha usando ponteiros para funes
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

#include <stdio.h>   /* Entrada e sada             */
#include <string.h>  /* Funes strXXX() e memXXX() */
#include <stdlib.h>  /* Alocao dinmica           */

   /* Macro utilizada em tratamento */
   /* de condies de exceo       */
#define ASSEGURA(x, msg) if (!(x)) {\
                            fprintf(stderr, "\n%s\n", msg);\
                            exit(1); \
                         }

/****
 *
 * ComparaCrescente(): Compara dois valores inteiros
 *
 * Parmetros:
 *      a, b (entrada) - valores inteiros a ser comparados
 *
 * Retorno: 1, se a > b; 0, em caso contrrio
 *
 ****/
int ComparaCrescente(const void *a, const void *b)
{
   return *(int *)a - *(int *)b;
}

/****
 *
 * ComparaCrescente(): Compara dois valores inteiros
 *
 * Parmetros:
 *      a, b (entrada) - valores inteiros a ser comparados
 *
 * Retorno: 1, se a < b; 0, em caso contrrio
 *
 ****/
int ComparaDecrescente(const void *a, const void *b)
{
   return *(int *)b - *(int *)a;
}

/****
 *
 * BubbleSortGen(): Ordena uma lista em ordem crescente ou
 *                  decrescente de acordo com a funo de
 *                  comparao recebida como parmetro
 *
 * Parmetros:
 *      lista (entrada e sada) - a lista a ser ordenada
 *      nElementos (entrada) - nmero de elementos da lista
 *      Compara (entrada) - ponteiro para a funo que
 *                          compara dois elementos da lista
 *
 * Retorno: Nada
 *
 ****/
void BubbleSortGen(void *lista, int tamElemento, int nElementos,
                   int (*compara) (const void *, const void *))
{
   int   i, ordenada = 0;
   char *aux;

      /* Tenta alocar um bloco que auxilia a troca de elementos do array */
   ASSEGURA(aux = malloc(tamElemento), "Impossivel alocar bloco para ordenar array");

   while (!ordenada){
      ordenada = 1;  /* Supe que a lista est ordenada */

      for (i = 0; i < nElementos - 1; i++){
         if (compara( (char *) lista + i*tamElemento,
                      (char *) lista + (i+1)*tamElemento) > 0 ) {
            ordenada = 0;

               /* Troca elementos adjacentes */
            memcpy( aux, (char *)lista + i*tamElemento, tamElemento );
            memcpy( (char *) lista + i*tamElemento,
                    (char *) lista + (i + 1)*tamElemento,tamElemento );
            memcpy( (char *) lista + (i + 1)*tamElemento, aux, tamElemento );
         }
      }
   }

   free(aux); /* Libera o espao que foi alocado dinamicamente */
}

/****
 *
 * main(): Testa a funo BubbleSortGen()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int ar[] = {12, 55, 21, 1, 6, 8, 17, 220, 5, 83},
       i;

   printf("\t>>> Ordem original <<<\n");
   for (i = 0; i < sizeof(ar)/sizeof(ar[0]); i++)
      printf(" %d ", ar[i]);

   BubbleSortGen(ar, sizeof(ar[0]), sizeof(ar)/sizeof(ar[0]), ComparaCrescente);

   printf("\n\n\t>>> Ordem crescente <<<\n");
   for (i = 0; i < sizeof(ar)/sizeof(ar[0]); i++)
      printf(" %d ", ar[i]);

   BubbleSortGen(ar, sizeof(ar[0]), sizeof(ar)/sizeof(ar[0]), ComparaDecrescente);

   printf("\n\n\t>>> Ordem decrescente <<<\n");
   for (i = 0; i < sizeof(ar)/sizeof(ar[0]); i++)
      printf(" %d ", ar[i]);

   return 0;
}

/****************** Resultado do Programa *****************

        >>> Ordem original <<<
 12  55  21  1  6  8  17  220  5  83

        >>> Ordem crescente <<<
 1  5  6  8  12  17  21  55  83  220

        >>> Ordem decrescente <<<
 220  83  55  21  17  12  8  6  5  1

**********************************************************/
