/****
 *
 * Ttulo: rvores Binrias
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Testa operaes sobre rvores
 *
 * Entrada: No h
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada       */
#include <string.h>  /* Funes de Entrada/Sada       */
#include "Arvore.h"  /* Operaes com rvores binrias */

/***************** Definies de Funes ******************/

/****
 *
 * IndiceElemento(): Obtm o ndice de um elemento que
 *                   se encontra possivelmente num array
 *
 * Parmetros:
 *      valor (entrada) - contedo do elemento procurado
 *      ar[] (entrada) - o array no qual ser efetuada a busca
 *      inf (entrada) - limite (ndice) inferior da busca
 *      sup (entrada) - limite (ndice) superior da busca
 *
 * Retorno: Nenhum
 *
 ****/
int IndiceElemento( tConteudo valor, const tConteudo ar[], int inf, int sup )
{
      /* Procura o elemento no array */
   for (int i = inf; i <= sup; ++i)
      if (ar[i] == valor)
         return i; /* Encontrado */

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * ReconstroiArvoreBin(): Reconstri uma rvore binria a partir
 *                        de seus caminhamentos prefixo e infixo
 *
 * Parmetros:
 *      prefixo[] (entrada) - array que armazena o caminhamento prefixo
 *      infixo[] (entrada) - array que armazena o caminhamento infixo
 *      infInfixo (entrada) - ndice inferior do array infixo[]
 *      supInfixo (entrada) - ndice superior do array infixo[]
 *     *proximoNo (entrada e sada) - o prximo n a ser levado
 *                                    em considerao no caminhamento prefixo
 *
 * Retorno: Endereo da raiz da rvore reconstruda
 *
 ****/
tArvoreBin ReconstroiArvoreBin(const tConteudo prefixo[], const tConteudo infixo[],
                               int infInfixo, int supInfixo, int *proximoNo)
{
   tNoArvoreBin *no; /* Ponteiro para um novo n */
   int           indiceNo; /* ndice de um n no array que */
                           /* contm o caminhamento infixo */

      /* Se o ndice inferior do array infixo[] for maior do */
      /* que o seu ndice superior, no h mais o que fazer  */
   if(infInfixo > supInfixo)
      return NULL;

      /* Obtm o contedo do n corrente no caminhamento */
      /* prefixo e constri um novo n                   */
   no = ConstroiNoArvoreBin(prefixo[*proximoNo]);

   (*proximoNo)++; /* Passa para o prximo n no caminhamento prefixo */

      /* Se existir apenas um n no caminhamento infixo, esse */
      /* n no possui filhos e no h mais nada o que fazer  */
   if(infInfixo == supInfixo)
      return no;

      /* Obtm o ndice do n corrente no caminhamento infixo */
   indiceNo = IndiceElemento( no->conteudo, infixo, infInfixo, supInfixo );

      /*Constri a subrvore esquerda deste n */
   no->esquerda = ReconstroiArvoreBin( prefixo, infixo, infInfixo, indiceNo - 1,
                                       proximoNo );

      /*Constri a subrvore direita deste n */
   no->direita = ReconstroiArvoreBin( prefixo, infixo, indiceNo + 1, supInfixo,
                                      proximoNo );

   return no; /* Servio completo */
}

/****
 *
 * ExibeDado(): Exibe na tela o contedo do campo dado
 *              de um n de uma rvore binria
 *
 * Parmetros:
 *      dado (entrada) - item de dado que ser impresso
 *
 * Retorno: Nenhum
 *
 * Observao: Assume-se que o tipo do contedo a ser impresso  char
 *
 ****/
static void ExibeDado(tConteudo dado)
{
   putchar((int) dado);
}

/****
 *
 * main(): Testa a funo ReconstroiArvoreBin()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zer
 *
 ****/
int main (void)
{
   tArvoreBin arvore; /* Ponteiro para a raiz da rvore */
   char       infixo[] = "BFDAEGC", /* Caminhamento infixo */
              prefixo[] = "ABDFCEG"; /* Caminhamento prefixo */
   int        proximo = 0, /* Prximo n a ser considerado */
                           /* no caminhamento prefixo      */
              nNos; /* Nmero de ns da rvore */

      /* Calcula o nmero de ns da rvore. Como os   */
      /* caminhamentos so representados por strings, */
      /* usa-se strlen().                             */
   nNos = strlen(infixo);

      /* Reconstroi a rvore a partir de seus caminhamentos */
   arvore = ReconstroiArvoreBin( prefixo, infixo, 0, nNos - 1,
                              &proximo);

      /* Exibe a rvore */
   printf("\n\t*** Arvore ***\n\n");
   ExibeArvoreBin(arvore, ExibeDado);

   return 0;
}

/****************** Resultado do Programa ******************

        *** Arvore ***

A
        C
                E
                        G
        B
                D
                        F

***********************************************************/
