/****
 *
 * Exerccio de Programao EP11.18
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/08/2012
 * ltima modificao: 26/08/2012
 *
 * Descrio: Conta o nmero de ocorrncias de letras
 *            num arquivo de texto
 *
 * Entrada: Contedo do arquivo "ListaDePalavras.txt"
 *
 * Sada: Nmero de ocorrncia de cada letra do arquivo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada             */
#include <ctype.h>   /* Classificao de caracteres */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de uma palavra */
#define TAM_MAX_PALAVRA  30

   /* Numero de letras no alfabeto romano */
#define NUMERO_DE_LETRAS  26

   /* Nome do arquivo que ser utilizado */
#define NOME_ARQUIVO "ListaDePalavras.txt"

/************************* Aluses ************************/

extern void MaiorMenorOcorrencia(const int contaLetras[]);
extern int TotalDeLetras(const int contaLetras[]);
extern void ApresentaFrequenciaDeLetras(const int ar[]);
extern int IndiceDeLetra(int c);

/***************** Definies de Funes ******************/

/****
 *
 * MaiorMenorOcorrencia(): Apresenta as letras que possuem
 *                         a maior e menor ocorrncias no
 *                         array de ocorrncias
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero
 *                              de ocorrncias de cada
 *                              letra num arquivo
 *
 * Retorno: Nada.
 *
 *
 ****/
void MaiorMenorOcorrencia(const int contaLetras[])
{
   int maior,       /* Maior frequncia */
       maiorIndice, /* ndice da maior frequncia */
       menor,       /* Menor frequncia */
       menorIndice, /* ndice da menor frequncia */
       i;

      /* Inicia a menor e a maior frequncias */
      /* com a primeira frequncia do array   */
   maior = menor = contaLetras[0];

      /* Inicia os ndices da menor e da maior     */
      /* frequncia com o primeiro ndice do array */
   maiorIndice = menorIndice = 0;

   /************************************************/
   /*                                              */
   /* Examina cada frequncia do array. Se for     */
   /* encontrada uma frequncia maior do que       */
   /* aquela correntemente considerada maior, a    */
   /* frequncia encontrada passa a ser a maior e  */
   /* o ndice da maior frequncia passa a ser o   */
   /* ndice desta frequncia. Idntico raciocnio */
   /*  aplicada nas determinaes da menor        */
   /* frequncia e seu ndice.                     */
   /*                                              */
   /************************************************/

   for (i = 1; i < NUMERO_DE_LETRAS; ++i) {
      if (contaLetras[i] > maior) {
         maior = contaLetras[i];
         maiorIndice = i;
      }

      if (contaLetras[i] < menor) {
         menor = contaLetras[i];
         menorIndice = i;
      }
   }

   /**************************************************/
   /*                                                */
   /* A letra que corresponde a um dado ndice      */
   /* obtida somando-se o ndice com 'A', pois esta  */
   /* letra corresponde ao ndice 0, 'B' corresponde */
   /* ao ndice 1, e assim por diante.               */
   /*                                                */
   /**************************************************/

   printf( "\n>>> Letra com maior frequencia (%d vezes):"
           " '%c'", maior, maiorIndice + 'A' );

   printf( "\n>>> Letra com menor frequencia (%d vezes):"
           " '%c'\n", menor, menorIndice + 'A' );
}

/****
 *
 * TotalDeLetras(): Calcula o nmero total de ocorrncias de
 *                  letras no array recebido como parmetro
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero
 *                              de ocorrncias de cada
 *                              letra num arquivo
 *
 * Retorno: O nmero total de ocorrncias de letras
 *          no array recebido como parmetro
 *
 ****/
int TotalDeLetras(const int contaLetras[])
{
   int i, total = 0;

   for (i = 0; i < NUMERO_DE_LETRAS; ++i) {
      total = total + contaLetras[i];
   }

   return total;
}

/****
 *
 * ApresentaFrequenciaDeLetras(): Apresenta o nmero de
 *                                ocorrncias de cada
 *                                letra no array
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero
 *                              de ocorrncias de cada
 *                              letra num arquivo
 *
 * Retorno: Nada.
 *
 ****/
void ApresentaFrequenciaDeLetras(const int contaLetras[])
{
   int ocorrencias, c, total;

      /* Escreve cabealho da tabela */
   printf( "\nLetra\tOcorrencias\tPercentagem"
           "\n=====\t===========\t===========\n\n" );

   total = TotalDeLetras(contaLetras);

      /* Determina o nmero de ocorrncias de */
      /* cada letra no array contaLetras[]    */
   for (c = 'A'; c <= 'Z'; ++c) {

      /****************************************************/
      /*                                                  */
      /* IMPORTANTE: Em qualquer cdigo de caracteres, as */
      /*             letras aparecem em ordem alfabtica. */
      /*             Mas, isto no significa que qualquer */
      /*             caractere entre 'A' e 'Z' ou entre   */
      /*             'a' e 'z'  letra. Por isso, a       */
      /*             chamada de isalpha()  necessria.   */
      /*                                                  */
      /****************************************************/

         /* Verifica se o caractere  uma letra */
      if (isalpha(c)) { /* Caractere  uma letra */
            /* Obtm o nmero de ocorrncias da letra */
         ocorrencias = contaLetras[IndiceDeLetra(c)];

            /* Apresenta apenas ocorrncias */
            /* diferentes de zero           */
         if (ocorrencias != 0) {
            printf( "  %c\t   %5d\t   %5.2f%%\n", c,
                    ocorrencias,
                    ((double) ocorrencias/total)*100 );
         }
      }
   }
}

/****
 *
 * IndiceDeLetra(): Determina o ndice correspondente
 *                  a uma letra no array que conta
 *                  frequncias de letras
 *
 * Parmetros:
 *      c (entrada) - caractere que supostamente
 *                    representa uma letra
 *
 * Retorno: O referido ndice, se o parmetro representar
 *          uma letra. Caso contrrio, -1.
 *
 ****/
int IndiceDeLetra(int c)
{
   /*****************************************************/
   /*                                                   */
   /* Se o caractere for letra, converte-a em maiscula */
   /* e subtrai seu valor de 'A'. Assim, 'A' (ou 'a')   */
   /* tem ndice 0, 'B' (ou 'b') tem ndice 1, etc.     */
   /*                                                   */
   /*****************************************************/

      /* Verifica se o caractere  letra */
   if (isalpha(c)) { /*  letra */
      return toupper(c) - 'A';
   } else {  /* No  letra */
      return -1;
   }
}

/****
 *
 * main(): Conta o nmero de ocorrncias de letras
 *         num arquivo de texto e informa quais so
 *         as letras que possuem a maior e a menor
 *         frequncia.
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   int   c, indice, contaLetras[NUMERO_DE_LETRAS] = {0};
   FILE *stream;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa determina o numero"
           "\n\t>>> de ocorrencias de cada letra do"
           "\n\t>>> arquivo \"%s\".\n", NOME_ARQUIVO );

   stream = fopen(NOME_ARQUIVO, "r");

   if (!stream) {
      printf( "\n\t>>> Impossivel abrir arquivo \"%s\"\n",
              NOME_ARQUIVO );
      return 1;
   }

      /* O lao encerra quando o final do arquivo */
      /* for atingido ou ocorrer erro de leitura  */
   while (1) {
         /* L um caractere */
      c = fgetc(stream);

         /* Verifica se ocorreu erro */
      if (ferror(stream)) { /* Ocorreu erro de leitura */
         printf( "\n\t>>> Erro de leitura no arquivo "
                 "\"%s\"\n", NOME_ARQUIVO );

         fclose(stream); /* Fecha arquivo antes de partir */

         return 1; /* Encerra o programa */
      }

         /* Verifica se final do arquivo foi atingido */
      if (feof(stream)) { /* Acabou o arquivo */
         break; /* Encerra o lao */
      }

         /* Se o caractere lido for uma letra, obtm seu */
         /* ndice no array que armazena as ocorrncias  */
      if (isalpha(c)) {
            /* Obtm o ndice no array de ocorrncias */
            /* que corresponde  letra corrente       */
         indice = IndiceDeLetra(c);

            /* O ndice retornado no pode jamais */
            /* ser negativo neste caso, pois o    */
            /* caractere  uma letra              */
         if (indice < 0) {
            printf("\a\nEste programa contem um erro!\n");
            return 1;
         }

            /* Incrementa o nmero de ocorrncias */
            /* da letra corrente                  */
         ++contaLetras[indice];
      }
   }

   fclose(stream); /* Fecha arquivo */

      /* Apresenta o resultado */
   ApresentaFrequenciaDeLetras(contaLetras);

   MaiorMenorOcorrencia(contaLetras);

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa determina o numero
        >>> de ocorrencias de cada letra do
        >>> arquivo "ListaDePalavras.txt".

Letra   Ocorrencias     Percentagem
=====   ===========     ===========

  A        36225           14.26%
  B         3922            1.54%
  C        14081            5.54%
  D        10490            4.13%
  E        24887            9.80%
  F         3401            1.34%
  G         4472            1.76%
  H         2350            0.92%
  I        23378            9.20%
  J          777            0.31%
  K           40            0.02%
  L        10234            4.03%
  M         8251            3.25%
  N        14153            5.57%
  O        26022           10.24%
  P         6635            2.61%
  Q         1074            0.42%
  R        23069            9.08%
  S        12547            4.94%
  T        14282            5.62%
  U         7685            3.02%
  V         3746            1.47%
  W           26            0.01%
  X          954            0.38%
  Y           41            0.02%
  Z         1336            0.53%

>>> Letra com maior frequencia (36225 vezes): 'A'
>>> Letra com menor frequencia (26 vezes): 'W'

        >>> Obrigado por usar este programa.

**********************************************************/
